#
# $Id: AtmHost.tcl,v 1.4 1996-01-25 15:32:17-08 bmah Exp $
#
# AtmHost.tcl
# Bruce A. Mah (bmah@CS.Berkeley.EDU)
#
# Copyright (c) 1995, 1996 The Regents of the University of California
# All Rights Reserved
# Created by Bruce A. Mah
# Department of Electrical Engineering and Computer Sciences,
#   Computer Science Division
#

#
# atmFifoAdd
#
# Input:  name (of host), max VCIDs, link bandwidth (bw), host
# ATM address (addr), interface number, ip address, netmask
#
# Output:  none
#
# Attach a new ATM (FIFO) interface to an existing host's IP module.
#
proc atmFifoAdd {name maxvcid bw addr num ip netmask} {

    # Get binary representations for the dotted quads making up
    # the IP address and netmask
    set ipAddr [ inet_addr $ip ]
    set netmaskAddr [ inet_addr $netmask ]

    # Compute network address and broadcast address
    set netAddr [ expr $ipAddr & $netmaskAddr ]
    set broadcastAddr [ expr $netAddr | ~$netmaskAddr]
    set net [ inet_ntoa $netAddr ]
    set broadcast [ inet_ntoa $broadcastAddr ]

    # Create ATM interface and driver
    HostAdapterFifo $name.atm$num $maxvcid $bw $addr
    $name.atm$num.aal up $name.atmd$num

    AtmDeviceDriver $name.atmd$num 128 100 100
    $name.atmd$num qostype noqos
    $name.atmd$num muxtype router
    $name.atmd$num vctype svc
    $name.atmd$num sighost $name.atm$num.sighost
    $name.atmd$num down $name.atm$num.aal
    $name.atmd$num up $name.ip

    # Specify ATM (atmd) interface to the IP module
    $name.ip ifconfig $name.atmd$num $ip $netmask $broadcast
    $name.ip route add $net $netmask $ip

}

#
# atmRcspAdd
#
# Input:  name (of host), max VCIDs, link bandwidth (bw), number of
# priorities (np), host ATM address (addr), interface number, 
# ip address, netmask
#
# Output:  none
#
# Attach a new ATM (RCSP) interface to an existing host's IP module.
# The global variables $ConfAtmDeviceDriverQosType,
# $ConfAtmDeviceDriverMuxType, and $ConfAtmDeviceDriverVcType
# select the appropriate characteristics of the device driver.
# They define the device driver behavior governing QOS, 
# IP multiplexing, and virtual circuit usage.  The default behavior
# results in best-effort SVCs with per-router pair multiplexing.
#
proc atmRcspAdd {name maxvcid bw np addr num ip netmask} {

    global ConfAtmDeviceDriverQosType
    if {! [info exists ConfAtmDeviceDriverQosType] } {
	set confAtmDeviceDriverQosType noqos
    } else {
	set confAtmDeviceDriverQosType $ConfAtmDeviceDriverQosType
    }

    global ConfAtmDeviceDriverMuxType
    if {! [info exists ConfAtmDeviceDriverMuxType] } {
	set confAtmDeviceDriverMuxType router
    } else {
	set confAtmDeviceDriverMuxType $ConfAtmDeviceDriverMuxType
    }

    global ConfAtmDeviceDriverVcType
    if {! [info exists ConfAtmDeviceDriverVcType] } {
	set confAtmDeviceDriverVcType svc
    } else {
	set confAtmDeviceDriverVcType $ConfAtmDeviceDriverVcType
    }

    # Get binary representations for the dotted quads making up
    # the IP address and netmask
    set ipAddr [ inet_addr $ip ]
    set netmaskAddr [ inet_addr $netmask ]

    # Compute network address and broadcast address
    set netAddr [ expr $ipAddr & $netmaskAddr ]
    set broadcastAddr [ expr $netAddr | ~$netmaskAddr]
    set net [ inet_ntoa $netAddr ]
    set broadcast [ inet_ntoa $broadcastAddr ]

    # Create ATM interface and driver
    HostAdapterRcsp $name.atm$num $maxvcid $bw $np $addr
    $name.atm$num.aal up $name.atmd$num

    AtmDeviceDriver $name.atmd$num 128 100 100
    $name.atmd$num qostype $confAtmDeviceDriverQosType
    $name.atmd$num muxtype $confAtmDeviceDriverMuxType
    $name.atmd$num vctype $confAtmDeviceDriverVcType
    $name.atmd$num sighost $name.atm$num.sighost
    $name.atmd$num down $name.atm$num.aal
    $name.atmd$num up $name.ip

    # Specify ATM (atmd) interface to the IP module
    $name.ip ifconfig $name.atmd$num $ip $netmask $broadcast
    $name.ip route add $net $netmask $ip

}
