#
# $Id: HostAdapterFifo.tcl,v 1.7 1996-05-29 14:34:20-07 bmah Exp $
#
# HostAdapterFifo.tcl
# Bruce A. Mah (bmah@CS.Berkeley.EDU)
#
# Copyright (c) 1995, 1996 The Regents of the University of California
# All Rights Reserved
# Created by Bruce A. Mah
# Department of Electrical Engineering and Computer Sciences,
#   Computer Science Division
#

#
# HostAdapterFifo
#
# Input:  host adapter name (name), max number of VCIDs (v),
#         default link bandwidth (bw), host ATM address (addr)
#
# Output:  none
#
# Create an ATM host adapter for FIFO queueing.
#
proc HostAdapterFifo {name v bw addr} {

    # Create switch module (3 ports, 0 == sighost, 1 == host, 2 == net)
    SwitchModule $name.switch 3 $v

    # Create signalling entity (100 concurrent signalling 
    # calls max)
    SigHost $name.sighost $v 100 $addr
    $name.sighost switch $name.gobackn
    $name.sighost aal $name.aal
    $name.sighost net $name.output2

    # Create input and output ports for signalling entity on port 0
    # with no delay and arbitrarily large bandwidth
    CellInputPort $name.input0 $name.switch 0 0
    CellQueueFifo $name.output0
    $name.output0 output $name.gobackn
    $name.output0 latency 0
    $name.output0 bitrate 1000000000
    $name.switch output 0 $name.output0

    # Create input and output ports for host (100 vcids, 1 us gap, port 1)
    Aal $name.aal $v 0.000001 1
    $name.aal down $name.switch
    $name.switch output 1 $name.aal
    
    # Create input and output ports for network
    CellInputPort $name.input2 $name.switch 2 0
    CellQueueFifo $name.output2
    $name.output2 latency 0
    $name.output2 bitrate $bw
    $name.switch output 2 $name.output2

    # Create GoBackN entity for signalling messages
    CellGoBackN $name.gobackn 3 512 0.50 0.10
    $name.gobackn up $name.sighost
    $name.gobackn down $name.input0

    # Create initial translation tables for control channels in
    # switch module.  We want a passthrough for all data channels
    # (both inbound and outbound), and to divert control messages
    # to the SigHost object.
    for {set i 1} {$i < $v} {incr i} {
	$name.switch set 1 $i 2 $i
	$name.switch set 2 $i 1 $i
    }
    $name.switch set 1 0 0 1
    $name.switch set 0 1 1 0
    $name.switch set 2 0 0 2
    $name.switch set 0 2 2 0

    proc $name {} [ list puts -nonewline "$name (HostAdapterFifo)" ]

}

