#
# $Id: SwitchFifoN.tcl,v 1.7 1996-05-29 14:34:27-07 bmah Exp $
#
# SwitchFifoN.tcl
# Bruce A. Mah (bmah@CS.Berkeley.EDU)
# Copyright (c) 1995, 1996 The Regents of the University of California
# All Rights Reserved
# Created by Bruce A. Mah
# Department of Electrical Engineering and Computer Sciences,
#   Computer Science Division
#

#
# SwitchFifoN.tcl
#
# Input:  switch name (name), number of data ports (n),
#         max number of VCIDs (v), default link bandwidth (bw)
#
# Output:  none
#
# Create an nxn ATM switch with FIFO output queues.
#
proc SwitchFifoN {name n v bw} {
  
    # Create switch module (use n+1 ports...port 0 goes to the
    # switch controller)
    SwitchModule $name.switch [expr {$n+1}] $v

    # Create signalling entity ($v VCIs max, 256 routes max)
    Sig $name.sig [expr {$n+1}] $v 256
    $name.sig switch $name.switch
    $name.sig gobackn $name.gobackn

    # Create input port and output port for switch controller
    # on port 0, with no delay and arbitrarily large bandwidth
    CellInputPort $name.input0 $name.switch 0 0
    CellQueueFifo $name.output0
    $name.output0 output $name.gobackn
    $name.output0 latency 0
    $name.output0 bitrate 1000000000
    $name.switch output 0 $name.output0

    # Create CellGoBackN entity for signalling messages.
    CellGoBackN $name.gobackn [expr {$n+1}] 512 0.50 0.10
    $name.gobackn up $name.sig
    $name.gobackn down $name.input0

    # Loop over all data ports and create both input ports and output
    # ports.
    for {set i 1} {$i < [expr $n+1]} {incr i} {

	CellInputPort $name.input$i $name.switch $i 0
	CellQueueFifo $name.output$i
	$name.output$i latency 0
	$name.output$i bitrate $bw
	$name.switch output $i $name.output$i
	$name.sig output $i $name.output$i

    }

    # Create initial translation tables for control channels.  We
    # follow a scheme based on XUNET, where VCID 0 along each link is
    # used for signalling.  Cells arriving on port p are translated
    # to port 0 (to controller), VCID p to get send to the switch
    # controller.  Similarly, outgoing cells from port 0 (VCID v) are
    # translated to port v, VCID 0 to go to the next hop switch.
    for {set i 1} {$i < [expr $n+1]} {incr i} {
	$name.switch set $i 0 0 $i
	$name.switch set 0 $i $i 0
    }

    proc $name {} [ list puts -nonewline "$name (SwitchFifoN)" ]

}



