#
# $Id: util.tcl,v 1.3 1996-01-28 13:38:23-08 bmah Exp $
#
# util.tcl
#
# Copyright (c) 1996 The Regents of the University of California
# All Rights Reserved
# Created by Bruce A. Mah
# Department of Electrical Engineering and Computer Sciences,
#   Computer Science Division
#

#
# objects
#
# List all objects' state, in sorted order.  A single optional argument
# controls the depth of nesting of composite objects.
#
# Unfortunately, this function has gotten
# considerably more complex since first conceived.  The current version
# attempts to determine the names of composite objects from the names
# of atomic objects; if a Tcl command procedure for the composite
# object, it gets called to print the object's state.
#
# This operation is roughly analogous to doing a depth-first, pre-order
# traversal of a tree, given the traversal order of leaves 
# (but not internal nodes).  The tree traversal gets truncated (albeit
# inefficiently) at $level-1 levels away from the top level.
#
proc objects {{level 0}} {

    set lastcomps ""

    foreach object [lsort [split [sim list]]] {

	set comps [split $object .]
	set trycomps $comps
	set prefix ""

	# Find out if the current object is a part of some new composite
	# object.
#	puts "====== $lastcomps ?= $trycomps"
	set done 0
	while { ([llength $lastcomps] > 0) && ([llength $trycomps] > 0) && 
	        ($done == 0)} {
#	    puts "$lastcomps ?= $trycomps"
	    if {[lindex $trycomps 0] == [lindex $lastcomps 0]} {
#		puts "  match"
		set comp [ lindex $trycomps 0 ]
		set trycomps [ lreplace $trycomps 0 0 ]
		set lastcomps [ lreplace $lastcomps 0 0 ]
		lappend prefix $comp
	    } else {
#		puts "  done"
		set done 1
	    }
	}

#	puts "[join $prefix .] (prefix)"
	
	while { ([llength $trycomps] > 1) } {
	    lappend prefix [ lindex $trycomps 0 ]
	    set trycomps [ lreplace $trycomps 0 0 ]
	    set composite [join $prefix . ]
#	    puts [llength [info commands $composite]]

	    if {($level == 0) || ([llength $prefix] <= $level)} {
		if {[llength [info commands $composite]] > 0} {
		    puts "[$composite]"
		} else {
		    puts "[join $prefix . ] (composite)"
		}
	    }
	}

	if {($level == 0) || ([llength $comps] <= $level)} {
	    puts "[$object]"
	}
	set lastcomps $comps

    }

}

#
# connect
#
# Input:  obj1 port1 obj2 port2 delay
#
# Output:  None
#
# Bidirectional connection between input and output ports of the two
# objects passed, with the delay specified.
#
proc connect { obj1 port1 obj2 port2 delay } {

    $obj1.output$port1 output $obj2.input$port2
    $obj1.output$port1 latency $delay
    $obj2.output$port2 output $obj1.input$port1
    $obj2.output$port2 latency $delay

}
