// -*- c++ -*-
//
// $Id: Aal.h,v 1.7 1996-01-10 17:05:18-08 bmah Exp $
//
// Aal.h
// Bruce A. Mah (bmah@CS.Berkeley.EDU)
//
// Copyright (c) 1995 The Regents of the University of California
// All Rights Reserved
// Created by Bruce A. Mah
// Department of Electrical Engineering and Computer Sciences,
//   Computer Science Division
//

#ifndef AAL_H
#define AAL_H

#include "ScheduledObject.h"
#include "AtmCell.h"
#include "Event.h"

class Aal : public ScheduledObject {

    friend int AalCmd(ClientData clientData, Tcl_Interp *interp,
			 int argc, char *argv[]);
    friend int AalObjectCmd(ClientData clientData, Tcl_Interp *interp,
			    int argc, char *argv[]);

    public:
    const int maxVcids;		// maximum number of VCIDs supported
    const double gap;		// inter-cell gap
    const int port;		// port number for switching

    Aal(const String &name, Simulation *si, int v, double g, int p) :
      ScheduledObject(name, si), maxVcids(v), gap(g), port(p)
    {
	outPacketNumber = new int[maxVcids]; // get storage for packet number
					     // array
	inPacketNumber = new int[maxVcids];  // get storage for incoming
					     // packet numbers
	inCellNumber = new int[maxVcids];    // get storage for incoming
					     // cell numbers
	expectedTime = 0.0;	// reset expected transmission time
    }

    virtual ~Aal()
    {
	delete [] outPacketNumber;
	delete [] inPacketNumber;
	delete [] inCellNumber;
    }
			 
    virtual String Output(void);
    virtual void Handler(Event *e);

    void ResetIn(int vcid, int seqno);
    void ResetOut(int vcid, int seqno);

    protected:
    double expectedTime;	// expected transmission time
    int *outPacketNumber;	// outgoing packet number for each channel
    int *inPacketNumber;	// incoming packet number for each channel
    int *inCellNumber;		// incoming cell number for each channel

    String upName;			      
    ScheduledObject *up;	// client of AAL
    String downName;			      
    ScheduledObject *down;	// underlying network object

};

extern int AalCmd(ClientData clientData, Tcl_Interp *interp,
		  int argc, char *argv[]);
extern int AalObjectCmd(ClientData clientData, Tcl_Interp *interp,
			int argc, char *argv[]);
extern void AalDeleteProc(ClientData clientData);

#endif // AAL_H
