static char rcsid[] = "$Id: App.cc,v 1.5 1996-01-10 17:05:19-08 bmah Exp $";
//
// $Id: App.cc,v 1.5 1996-01-10 17:05:19-08 bmah Exp $
//
// App.cc
// Bruce A. Mah (bmah@CS.Berkeley.EDU)
//
// Copyright (c) 1995, 1996 The Regents of the University of California
// All Rights Reserved
// Created by Bruce A. Mah
// Department of Electrical Engineering and Computer Sciences,
//   Computer Science Division
//
// Application base class.  This class exists solely for the purpose
// of organizing the hierarchy of applications.
//

#include "App.h"
#include "Event.h"
#include "inline.h"

//
// App::App
//
// Input:  Name, simulation pointer, start time
//
// Output: None
// 
// Object constructor
//
App::App(const String &name, Simulation *si, double initTime) :
ScheduledObject(name, si) {

    // Post an event to ourselves to wake us up so we can
    // get started.  For various reasons, we want
    // connection setup (both passive and active)
    // to run inside event handlers, so
    // this forces our event handler to run (soon) after
    // we get created.
    Post(initTime, this, EventTimer, NULL);
    
}


//
// App::Handler
//
// (Event handler)
//
void App::Handler(Event *e)
{

    double time = e->ev_time;
    EventType type = e->ev_type;
    void *data = e->ev_data;

    // The base class shouldn't get any events, so everything's an error
    // at this point.
    IF_DEBUG(0, {
	DEBUG_PREFIX("App");
	cout << "bad EventType " << type << '\n';
    });

}

//
// App::Output
//
// Input:  None
//
// Output:  String with an ASCII representation of this object.
//
String App::Output()
{
    return name + " [App]";
}

