static char rcsid[] = "$Id: AppTcpClient.cc,v 1.4 1995-12-10 14:14:41-08 bmah Exp $";
//
// $Id: AppTcpClient.cc,v 1.4 1995-12-10 14:14:41-08 bmah Exp $
//
// AppTcpClient.cc
// Bruce A. Mah (bmah@CS.Berkeley.EDU)
//
// Copyright (c) 1995 The Regents of the University of California
// All Rights Reserved
// Created by Bruce A. Mah
// Department of Electrical Engineering and Computer Sciences,
//   Computer Science Division
//
// TCP client application base class.  This class supports TCP applications
// that make single TCP connections to their servers.  Applications with
// more sophisticated needs can override various methods or use various
// hooks left in the class.
//

#include "AppTcpClient.h"
#include "Event.h"
#include "inline.h"

//
// AppTcpClient::Handler
//
// (Event handler)
//
void
AppTcpClient::Handler(Event *e)
{

    double time = e->ev_time;
    EventType type = e->ev_type;
    void *data = e->ev_data;

    // Switch first on the state of the application, then the event type.
    switch (state) {

	// We're down.  We should have gotten an EventTimer, otherwise
	// it's an error.
	case (AppTcpClientStateDown):
	{

	    switch (type) {

		// We got a Timer event...kick off a connection
		// establishment.
		case (EventTimer):
		tp = tcpModule->UserConnect(time, server, port, this);
		state = AppTcpClientStateEstablishing;
		break;

		default:
		{
		    IF_DEBUG(0, {
			DEBUG_PREFIX("AppTcpClient");
			cout << "bad EventType " << type <<
			  " in state AppTcpClientStateDown\n";
		    });
		}
		break;

	    }

	}
	break;
	
	// We're in the middle of connecting.  We should get an
	// EventTcpConnect, but we could get an EventTcpError, in case
	// the connection establishment didn't work.
	case (AppTcpClientStateEstablishing):
	{

	    switch (state) {

		case (EventTcpConnect):
		{

		    // First check to make sure the connection matches
		    // one we know about
		    if (data != tp) {
			IF_DEBUG(0, {
			    DEBUG_PREFIX("AppTcpClient");
			    cout << "connection mismatch in state AppTcpClientStateEstablishing\n";
			});
		    }
		    else {

			state = AppTcpClientStateEstablished;

		    }
		    
		}
		break;

		case (EventTcpError):
		{
		    
		    // Establishment failed
		    state = AppTcpClientStateDown;

		    // Get rid of ourselves.  Real applications will most
		    // likely have overridden this behavior because they
		    // will want to post statistics to the statistics
		    // log.
		    delete this;

		}
		break;

		default:
		{
		    IF_DEBUG(0, {
			DEBUG_PREFIX("AppTcpClient");
			cout << "bad EventType " << type <<
			  " in state AppTcpClientStateEstablishing\n";
		    });
		}
		break;

	    }
	}
	break;

	// We're established.  We can handle data (but we just
	// throw it away) or connection closure.
	case (AppTcpClientStateEstablished):
	{

	    switch (type) {

		case (EventTcpArrival):
		{
		    TcpCall *tc = (TcpCall *) data;
		    
		    delete tc;
		}
		break;

		case (EventTcpClosed):
		{
		    // Set our new state
		    state = AppTcpClientStateDown;
		    
		    // We should get rid of ourselves now.
		}
		break;

		default:
		{
		    IF_DEBUG(0, {
			DEBUG_PREFIX("AppTcpClient");
			cout << "bad EventType " << type <<
			  " in state AppTcpClientStateEstablished\n";
		    });
		}
		break;

	    }
	}
	break;

	// We're in a state we don't recognize, so hand it off to the
	// parent class of TCP-using applications.
	default:
	{
	    AppTcp::Handler(e);
	}
	break;
    }
}

//
// AppTcpClient::Output
//
// Input:  None
//
// Output:  String with an ASCII representation of this object.
//
String
AppTcpClient::Output()
{
    return name + " [AppTcpClient]";
}



