static char rcsid[] = "$Id: AppTcpServer.cc,v 1.7 1995-12-10 14:14:42-08 bmah Exp $";
//
// $Id: AppTcpServer.cc,v 1.7 1995-12-10 14:14:42-08 bmah Exp $
//
// AppTcpServer.cc
// Bruce A. Mah (bmah@CS.Berkeley.EDU)
//
// Copyright (c) 1995 The Regents of the University of California
// All Rights Reserved
// Created by Bruce A. Mah
// Department of Electrical Engineering and Computer Sciences,
//   Computer Science Division
//
// Base class of servers using TCP as a transport.  This class is
// designed to support TCP servers that listen on a single TCP
// port and accept an unlimited number of connections on that port.
//

#include "AppTcpServer.h"
#include "Event.h"
#include "inline.h"

//
// AppTcpServer::Handler
//
// (Event handler)
//
void
AppTcpServer::Handler(Event *e)
{

    double time = e->ev_time;
    EventType type = e->ev_type;
    void *data = e->ev_data;

    // If we haven't been initialized yet, then do so now.
    if (!initFlag) {

	// We darn well better have gotten an EventTimer event
	if (type == EventTimer) {

	    tp = tcpModule->UserListen(time, port, this);
	    initFlag = true;

	}
	else {
	    IF_DEBUG(0, {
		DEBUG_PREFIX("AppTcpServer");
		*(sim->debugStream) << "bad EventType " << type <<
		  " in uninitialized state\n";
	    });
	}

    }
    else {

	// We're initialized.  First figure out what kind of event we
	// got...
	switch (type) {

	    // New TCP connection!
	    case (EventTcpConnect):
	    {
	    }
	    break;

	    // We can handle the arrival of data, but we just tally it and
	    // throw it away.
	    case (EventTcpArrival):
	    {
		TcpCall *tc = (TcpCall *) data;

		delete tc;
	    }
	    break;

	    // TCP connection closed by client
	    case (EventTcpClosed):
	    {
		tcpModule->UserClose(time, (TcpPcb *) data);
	    }
	    break;

	    default:
	    {
		AppTcp::Handler(e);
	    }
	    break;

	}

    }
    
}

//
// AppTcpServer::Output
//
// Input:  None
//
// Output:  String with an ASCII representation of this object.
//
String
AppTcpServer::Output()
{
    return name + " [AppTcpServer]";
}



