static char rcsid[] = "$Id: ArrivalProcess.cc,v 1.4 1996-01-31 10:51:04-08 bmah Exp $";
//
// $Id: ArrivalProcess.cc,v 1.4 1996-01-31 10:51:04-08 bmah Exp $
//
// ArrivalProcess.cc
// Bruce A. Mah (bmah@CS.Berkeley.EDU)
//
// Copyright (c) 1996 The Regents of the University of California
// All Rights Reserved
// Created by Bruce A. Mah
// Department of Electrical Engineering and Computer Sciences,
//   Computer Science Division
//
// Base class for arrival processes.  With each arrival, these
// arrival processes invoke a Tcl script.
//

#include <String.h>
#include "ArrivalProcess.h"
#include "Tcl.h"
#include "Event.h"
#include "inline.h"

//
// Declare global variables
//
extern Tcl *tcl;

//
// ArrivalProcess::ArrivalProcess
//
// (Object constructor)
//
ArrivalProcess::ArrivalProcess(const String &name, Simulation *si,
			       double start, String &s) :
ScheduledObject(name, si), script(s)
{

    // Post a timer event to get us started.
    next = start;
    Post(start, this, EventTimer, NULL);
    
}

//
// ArrivalProcess::Handler
//
// (Event handler)
//
void
ArrivalProcess::Handler(Event *e)
{
    double time = e->ev_time;
    EventType type = e->ev_type;
    void *data = e->ev_data;

    if (type == EventTimer) {

	int rc;			// Tcl return code
	
	IF_DEBUG(2, {
	    DEBUG_PREFIX("ArrivalProcess");
	    *(sim->debugStream) << "trigger\n";
	});

	// Set up the script environment:
	//
	// o Global variable time is set to current simulation time.
	tcl->SetVar("time", time, 0);

	// Evaluate the script that was supposed to be run at this point.
	rc = tcl->Eval(script);

	// If there was an error, print it and drop into the debugger
	// after we're done processing this event.
	if (rc != TCL_OK) {
	    tcl->AddErrorInfo("\n    caused by\nA TimerEvent sent to the ArrivalProcess " + name);
	    cerr << tcl->GetVar("errorInfo", TCL_GLOBAL_ONLY) << "\n";
	    sim->interactive = true;
	}

	// Post a timer event for the next point at which we should be
	// triggered.
	next = time + NextInterarrival();
	Post(next, this, EventTimer, NULL);

    }
    else {
	IF_DEBUG(0, {
	    DEBUG_PREFIX("ArrivalProcess");
	    *(sim->debugStream) << "bad event type " << type << "\n";
	});
    }
}

//
// ArrivalProcessObjectCmd
//
// (Tcl object command procedure)
//
int
ArrivalProcessObjectCmd(ClientData clientData, Tcl_Interp *interp,
		     int argc, char *argv[])
{
    return ScheduledObjectCmd(clientData, interp, argc, argv);
}

