// -*- c++ -*-
// $Id: AtmCellList.h,v 1.6 1996-03-12 16:03:07-08 bmah Exp $
//
// AtmCellList.h
//
// Copyright (c) 1996 The Regents of the University of California
// All Rights Reserved
// Created by Bruce A. Mah
// Department of Electrical Engineering and Computer Sciences,
//   Computer Science Division
//

#ifndef ATMCELLLIST_H
#define ATMCELLLIST_H

#include <stdlib.h>
#include <stdio.h>
#include <iostream.h>
#include "AtmCell.h"


class AtmCellList {

    public:

    //
    // Constructor initializes the pointers to the ends of the lists
    //
    AtmCellList() {
	head = NULL;
	tail = NULL;
	length = 0;
    }

    //
    // Destructor deallocates all of the cells that were in the list.
    //
    ~AtmCellList() {
	AtmCell *current, *next;
	for (current = head; current != NULL; current = next) {
	    next = current->next;
	    delete current;
	}
    }

    // Enqueue and dequeue member functions do the expected things
    EnqueueTail(AtmCell *cell) {
	cell->next = NULL;
	if (tail) {
	    tail->next = cell;
	}
	else {
	    head = cell;
	}
	tail = cell;
	length++;

	Sanity(*this);

    }

    EnqueueHead(AtmCell *cell) {
	cell->next = head;
	head = cell;
	if (tail == NULL) {
	    tail = cell;
	}
	length++;

	Sanity(*this);

    }

    AtmCell *Dequeue() {
	AtmCell *cell;

	cell = head;
	if (head) {
	    head = head->next;
	    if (head == NULL) {
		tail = NULL;
	    }
	    length--;
	}

	Sanity(*this);

	return cell;
    }

    // Append this list to another.  Current list is cleared.
    void Append(AtmCellList &other) {
	if (other.length == 0) {
	    other.head = head;
	}
	else {
	    other.tail->next = head;
	}
	if (length > 0) {
	    other.tail = tail;
	}
	other.length += length;

	Sanity(other);

	head = NULL;
	tail = NULL;
	length = 0;

	Sanity(*this);

    }

    AtmCell *operator[](int index) {
	int i;
	AtmCell *current;
	for (i = 0, current = head; (current != NULL) && (i != index);
	     i++, current = current->next) {
	}
	return current;
    }

    int length;

    protected:
    AtmCell *head, *tail;

    // Sanity check on lists
    void Sanity(AtmCellList &list) {
	if (list.length < 0) { 
	    cerr << "AtmCellList:  negative length!\n";
//	    abort();
	}
	else if (list.length == 0) {
	    if ((list.head != NULL) || (list.tail != NULL)) {
		cerr << "AtmCellList:  zero length but non-null pointer!\n";
//		abort();
	    }
	}
	else if (list.length > 0) {
	    if ((list.head == NULL) || (list.tail == NULL)) {
		cerr << "AtmCellList:  non-zero length but null pointer!\n";
//		abort();
	    }
	}
    }

};

#endif // ATMCELLLIST_H
