static char rcsid[] = "$Id: AtmTest.cc,v 1.7 1996-02-02 15:56:12-08 bmah Exp $";
//
// $Id: AtmTest.cc,v 1.7 1996-02-02 15:56:12-08 bmah Exp $
//
// AtmTest.cc
// Bruce A. Mah (bmah@CS.Berkeley.EDU)
//
// Copyright (c) 1995, 1996 The Regents of the University of California
// All Rights Reserved
// Created by Bruce A. Mah
// Department of Electrical Engineering and Computer Sciences,
//   Computer Science Division
//
// Test object for ATM network
//

#include "AtmTest.h"
#include "AtmCell.h"
#include "SigHost.h"
#include "AalFrame.h"
#include "SigMessage.h"
#include "util.h"
#include "inline.h"

//
// AtmTest::AtmTest
//
AtmTest::AtmTest(const String &n, Simulation *s, double initTime,
		 String &sighostname, String &aalname,
		 int d, double db, double xm, double xa, double i,
		 int num, int size, double g) :
		 ScheduledObject(n, s), 
		 sigHostName(sighostname), aalName(aalname),
		 dest(d), delay(db), xmin(xm), xave(xa), I(i),
		 sendNumber(num), sendSize(size), gap(g)
{
    sigHost = NULL;
    aal = NULL;
    state = AtmTestStateEstablish;
    Post(initTime, this, EventTimer, NULL);
}

//
// AtmTest::Handler
//
// (Event handler)
//
void
AtmTest::Handler(Event *e)
{

    double time = e->ev_time;
    EventType type = e->ev_type;
    void *data = e->ev_data;

    switch (state) {

	case (AtmTestStateEstablish):

	{
	    ApiMessage *apiMessage = new (ApiMessage);

	    IF_DEBUG(2, {
		DEBUG_PREFIX("AtmTest");
		cout << "in Handler state Establish\n";
	    });

	    QosRcsp *qos = new QosRcsp;

	    qos->type = QosTypeRcsp;
	    qos->d = delay;
	    qos->Xmin = xmin;
	    qos->Xave = xave;
	    qos->I = I;
	    
	    // Fire off an establishment message
	    apiMessage->type = ApiTypeEstablish;
	    apiMessage->client = this;
	    apiMessage->vcid = 0;
	    apiMessage->requestId = 1;
	    apiMessage->code = SigOK;
	    apiMessage->sourceAddress = 0;
	    apiMessage->sourceService = SigServiceIP;
	    apiMessage->destinationAddress = dest;
	    apiMessage->destinationService = SigServiceIP;
	    apiMessage->traffic = qos;
	    apiMessage->trafficLength = sizeof (QosRcsp);

	    Lookup(sigHost, sigHostName);
	    Post(time + 0.10, sigHost, EventCall, (void *) apiMessage);

	    state = AtmTestStateEstablishDone;

	}

	break;

	case (AtmTestStateEstablishDone):

	{

	    IF_DEBUG(2, {
		DEBUG_PREFIX("AtmTest");
		cout << "in Handler state EstablishDone\n";
	    });

	    // Should have gotten a Call message of type Return
	    // with return code of SigOK.
	    if (type == EventCall) {

		ApiMessage *apiMessage = (ApiMessage *) data;
		
		if ((apiMessage->type == ApiTypeReturn) &&
		    (apiMessage->code == SigOK)) {
		    
		    // got channel, prepare to send data
		    vcid = apiMessage->vcid;
		    packetsToSend = sendNumber;
		    state = AtmTestStateSending;

		    // send first data packet
		    AalFrame *aalFrame = new AalFrame(NULL);
		    aalFrame->vcid = vcid;
		    aalFrame->length = sendSize;
		    aalFrame->payload = NULL;

		    Lookup(aal, aalName);
		    Post(time, aal, EventFrameSend,
			 (void *) aalFrame);

		    // schedule timer for wakeup
		    Post(time + gap, this, EventTimer, NULL);

		}
		else {
		    // no channel or unexpected ApiMessage->type
		    state = AtmTestStateZombie;
		    IF_DEBUG(0, {
			DEBUG_PREFIX("AtmTest");
			cout << "couldn't get connection\n";
		    });
		}
	    
		delete apiMessage;

	    }
	    else {
		// unknown event type
		IF_DEBUG(0, {
		    DEBUG_PREFIX("AtmTest");
		    cout << "unknown event type " << type << "\n";
		});
	    }

	}

	break;

	case (AtmTestStateSending):
	{

	    IF_DEBUG(2, {
		DEBUG_PREFIX("AtmTest");
		cout << "in Handler state Sending\n";
	    });

	    // send data packet
	    AalFrame *aalFrame = new AalFrame(NULL);
	    aalFrame->vcid = vcid;
	    aalFrame->length = sendSize;
	    aalFrame->payload = NULL;
	    
	    Lookup(aal, aalName);
	    Post(time, aal, EventFrameSend,
		 (void *) aalFrame);

	    packetsToSend--;
	    if (packetsToSend == 0) {
		state = AtmTestStateSendingDone;
	    }
	    // schedule timer
	    Post(time + gap, this, EventTimer, NULL);

	}
	break;
	
	case (AtmTestStateSendingDone):
	{

	    IF_DEBUG(2, {
		DEBUG_PREFIX("AtmTest");
		cout << "in Handler state SendingDone\n";
	    });

	    ApiMessage *apiMessage = new (ApiMessage);

	    // Close channel
	    apiMessage->type = ApiTypeDelete;
	    apiMessage->client = this;
	    apiMessage->vcid = vcid;
	    apiMessage->requestId = 2;
	    apiMessage->code = SigOK;

	    Lookup(sigHost, sigHostName);
	    Post(time, sigHost, EventCall, (void *) apiMessage);

	    state = AtmTestStateZombie;

	}
	break;
	
	case (AtmTestStateZombie):
	{
	    IF_DEBUG(2, {
		DEBUG_PREFIX("AtmTest");
		cout << "in Handler state Zombie\n";
	    });

	    // Should have gotten a Call message of type Return
	    // with return code of SigOK.
	    if (type == EventCall) {

		ApiMessage *apiMessage = (ApiMessage *) data;
		
		if ((apiMessage->type == ApiTypeReturn) &&
		    (apiMessage->code == SigOK)) {
		    
		}
		else {
		    // unexpected ApiMessage->type
		    IF_DEBUG(0, {
			DEBUG_PREFIX("AtmTest");
			cout << "unexpected return code\n";
		    });
		}

		delete apiMessage;

	    }
	    else {
		// unknown event type
		IF_DEBUG(0, {
		    DEBUG_PREFIX("AtmTest");
		    cout << "unexpected event type " << type << "\n";
		});
	    }
	}
	break;

	default:
	break;
    }

}

//
// AtmTest::Output
//
// Input:  None
//
// Output:  String with an ASCII representation of this object.
//
String AtmTest::Output()
{
    return name + " [AtmTest]";
}

//
// AtmTestCmd
//
// (Tcl command procedure)
//
int AtmTestCmd(ClientData clientData, Tcl_Interp *interp, int argc,
	       char *argv[])
{

    if (argc != 13) {
	Tcl_SetResult(interp,
		      "Usage:  AtmTest name start sighost aal dest db xmin xave i num size gap",
		      TCL_STATIC);
	return(TCL_ERROR);
    }
    else {
	char *nm;		// name of new AtmTest object
	double initTime;	// startup time this object
	String sighostname;	// name of sighost object
	String aalname;		// name of aal object
	int d;			// destination address
	double db;		// delay bound
	double xm;		// xmin
	double xa;		// xave
	double i;		// i
	int num;		// number of sends
	int size;		// size of sends
	double g;		// gap between sends
	
	int rc;			// return code from Tcl library
	AtmTest *atmTest;	// new object
	extern Simulation *sim;	// global simualtion object

	// get name
	nm = argv[1];

	rc = Tcl_GetDouble(interp, argv[2], &initTime);
	if (rc == TCL_OK) {

	    sighostname = argv[3];
	    aalname = argv[4];

	    rc = Tcl_GetInt(interp, argv[5], &d);
	    if (rc == TCL_OK) {

		rc = Tcl_GetDouble(interp, argv[6], &db);
		if (rc == TCL_OK) {
		    
		    rc = Tcl_GetDouble(interp, argv[7], &xm);
		    if (rc == TCL_OK) {

			rc = Tcl_GetDouble(interp, argv[8], &xa);
			if (rc == TCL_OK) {

			    rc = Tcl_GetDouble(interp, argv[9], &i);
			    if (rc == TCL_OK) {
			
				rc = Tcl_GetInt(interp, argv[10], &num);
				if (rc == TCL_OK) {

				    rc = Tcl_GetInt(interp, argv[11], &size);
				    if (rc == TCL_OK) {

					rc = Tcl_GetDouble(interp, argv[12], &g);
					if (rc == TCL_OK) {
					
					    atmTest =
					      new AtmTest(nm, sim, initTime,
							  sighostname, aalname,
							  d, db, xm, xa, i,
							  num, size, g);
					    Tcl_CreateCommand(interp, nm,
							      AtmTestObjectCmd,
							      (ClientData) atmTest,
							      AtmTestDeleteProc);
					}
				    }
				}
			    }
			}
		    }
		}
	    }
	}
	return rc;
    }
}
	       
//
// AtmTestObjectCmd
//
// (Tcl object command procedure)
//
int AtmTestObjectCmd(ClientData clientData, Tcl_Interp *interp,
		     int argc, char *argv[])
{
    return ScheduledObjectCmd(clientData, interp, argc, argv);
}
		  
//
// AtmTestDeleteProc
//
// (Tcl delete procedure)
//
void AtmTestDeleteProc(ClientData clientData) {
    delete (AtmTest *) clientData;
}
