// -*- c++ -*-
//
// $Id: AtmTest.h,v 1.7 1996-01-29 21:37:14-08 bmah Exp $
//
// AtmTest.h
// Bruce A. Mah (bmah@CS.Berkeley.EDU)
//
// Copyright (c) 1995, 1996 The Regents of the University of California
// All Rights Reserved
// Created by Bruce A. Mah
// Department of Electrical Engineering and Computer Sciences,
//   Computer Science Division
//

#ifndef ATMTEST_H
#define ATMTEST_H

#include "ScheduledObject.h"
#include "AtmCell.h"
#include "Event.h"

enum AtmTestState {
    AtmTestStateEstablish,
    AtmTestStateEstablishDone,
    AtmTestStateSending,
    AtmTestStateSendingDone,
    AtmTestStateZombie
};

class AtmTest : public ScheduledObject {

    friend int
    AtmTestCmd(ClientData clientData, Tcl_Interp *interp,
	       int argc, char *argv[]);
    friend int
    AtmTestObjectCmd(ClientData clientData, Tcl_Interp *interp,
		     int argc, char *argv[]);

    public:

    const int dest;		// destination of channel
    const double delay, xmin, xave, I; // traffic parameters
    const int sendNumber, sendSize;
    const double gap;

    int state;			// current state of state machine
    int packetsToSend;		// number of packets remaining to send

    AtmTest(const String &n, Simulation *s, double initTime,
	    String &sighostname, String &aalname,
	    int d, double db, double xm, double xa, double i,
	    int num, int size, double g);

    virtual String Output(void);
    virtual void Handler(Event *e);

    protected:

    String sigHostName;		// name of signalling entity
    ScheduledObject *sigHost;
    String aalName;		// name of AAL entity
    ScheduledObject *aal;
    int vcid;			// VCID to use
			 
};

extern int AtmTestCmd(ClientData clientData, Tcl_Interp *interp,
		      int argc, char *argv[]);
extern int AtmTestObjectCmd(ClientData clientData, Tcl_Interp *interp,
			    int argc, char *argv[]);
extern void AtmTestDeleteProc(ClientData clientData);

#endif // ATMTEST_H
