// -*- c++ -*-
//
// $Id: AudioSource.h,v 1.3 1996-01-27 23:14:08-08 bmah Exp $
//
// AudioSource.h
// Bruce A. Mah (bmah@CS.Berkeley.EDU)
//
// Copyright (c) 1995, 1996 The Regents of the University of California
// All Rights Reserved
// Created by Bruce A. Mah
// Department of Electrical Engineering and Computer Sciences,
//   Computer Science Division
//

#ifndef AUDIOSOURCE_H
#define AUDIOSOURCE_H

#include "AppUdpClient.h"
#include "Simulation.h"

enum AudioSourceState {
    AudioSourceStateOn,
    AudioSourceStateOff
};

class AudioSource : public AppUdpClient {

    friend int
    AudioSourceCmd(ClientData clientData, Tcl_Interp *interp,
		   int argc, char *argv[]);
    friend int
    AudioSourceObjectCmd(ClientData clientData, Tcl_Interp *interp,
			 int argc, char *argv[]);

    public:

    AudioSource(const String &name, Simulation *si, double initTime,
		String &u, InAddr s, short p, int pay, double g);

    virtual
    ~AudioSource()
    {
	*(sim->logStream) << name << " [AudioSource] lifetime " << lifeTime << " packets " << packetsSent << "\n";

    }

    virtual String Output(void);
    virtual void Handler(Event *e);

    const double gap;		// sending gap
    const int payloadBytes;	// number of UDP payload bytes

    protected:				  

    double stateChangeTime;	// time of next state change
    double startTime;		// starting time
    double lifeTime;		// connection lifetime
    double endTime;		// ending time of this object
    int packetsSent;		// number of packets sent
    AudioSourceState state;	// current on/off state

};

extern int AudioSourceCmd(ClientData clientData, Tcl_Interp *interp,
			  int argc, char *argv[]);
extern int AudioSourceObjectCmd(ClientData clientData, Tcl_Interp *interp,
				int argc, char *argv[]);
extern void AudioSourceDeleteProc(ClientData clientData);

#endif // AUDIOSOURCE_H
