// -*- c++ -*-
//
// $Id: CellGoBackN.h,v 1.8 1996-03-12 16:04:08-08 bmah Exp $
//
// CellGoBackN.h
// Bruce A. Mah (bmah@CS.Berkeley.EDU)
//
// Copyright (c) 1995, 1996 The Regents of the University of California
// All Rights Reserved
// Created by Bruce A. Mah
// Department of Electrical Engineering and Computer Sciences,
//   Computer Science Division
//

#ifndef CELLGOBACKN_H
#define CELLGOBACKN_H

#include "ScheduledObject.h"
#include "Simulation.h"
#include "AtmCell.h"
#include "AtmCellList.h"

class CellGoBackN : public ScheduledObject {

    friend int
    CellGoBackNCmd(ClientData clientData, Tcl_Interp *interp,
		   int argc, char *argv[]);
    friend int
    CellGoBackNObjectCmd(ClientData clientData, Tcl_Interp *interp,
			 int argc, char *argv[]);
    
    public:
    const int ports;		// number of ports
    const int maxCells;		// maximum number of outstanding cells
    const double timeout;	// timeout period
    const double poll;		// polling interval of timeout queue

    CellGoBackN(const String &name, Simulation *si, int p, int mc, double t,
		double po);
    virtual ~CellGoBackN() {

	delete [] retransmitQueue;
	delete [] incomingSeq;
	delete [] outgoingSeq;

    }
			       
    virtual String Output(void);
    virtual void Handler(Event *e);

    protected:
    String upName;			      
    ScheduledObject *up;	// client of CellGoBackN
    String downName;			      
    ScheduledObject *down;	// underlying network object
    AtmCellList *retransmitQueue; // list of packets to resend
    int *incomingSeq;		 // expected sequence numbers for
				 // incoming cells
    int *outgoingSeq;		 // outgoing sequence numbers
    double nextTimeout;		 // next timeout fires at...
      
};

extern int CellGoBackNCmd(ClientData clientData, Tcl_Interp *interp,
			   int argc, char *argv[]);
extern int CellGoBackNObjectCmd(ClientData clientData, Tcl_Interp *interp,
				 int argc, char *argv[]);
void CellGoBackNDeleteProc(ClientData clientData);

#endif // CELLGOBACKN_H



