static char rcsid[] = "$Id: CellInputPort.cc,v 1.5 1996-01-10 17:05:35-08 bmah Exp $";
//
// $Id: CellInputPort.cc,v 1.5 1996-01-10 17:05:35-08 bmah Exp $
//
// CellInputPort.cc
// Bruce A. Mah (bmah@CS.Berkeley.EDU)
// 
// Copyright (c) 1995 The Regents of the University of California
// All Rights Reserved
// Created by Bruce A. Mah
// Department of Electrical Engineering and Computer Sciences,
//   Computer Science Division
//
// Part of an ATM switch, this object simulates the input port
// to an output-queued switch.
//

#include <iostream.h>
#include "user.h"
#include "AtmCell.h"
#include "CellInputPort.h"
#include "Tcl.h"
#include "Simulation.h"
#include "String.ScheduledObjectPtr.CHMap.h"
#include "inline.h"

//
// CellInputPort::Handler
//
// Input:  (event handler)
//
// Output:  None
//
// Handle incoming cells, tagging the port number for input->output
// port mapping.
//
void
CellInputPort::Handler(Event *e)
{

    double time = e->ev_time;
    EventType type = e->ev_type;
    void *data = e->ev_data;

    switch (type) {
    
	//
	// EventCellArrival
	//
	// ATM cell arrived at input port.  Tag port number field
	// and forward the cell after the specified amount of latency.
	// Don't bother doing any queueing or anything, since this
	// particular operation isn't associated with any bottlenecks
	// in an output-queued switch.
	//
	case EventCellArrival:
	{
	    ((AtmCell *) data)->port = port;

	    Lookup(sinkObject, sink);
	    Post(time + latency, sinkObject, type, data);

	}
	break;

	default:
	{
	    IF_DEBUG(0, {
		DEBUG_PREFIX("CellInputPort");
		cout << "bad EventType " << type << "\n";
	    });
	}
	break;
    }

}

//
// CellInputPort::Output
//
// Input:  None.
//
// Output:  String
//
// Returns an ASCII representation of the CellInputPort.
//
String CellInputPort::Output()
{
    String out;
    char buf[30];

    sprintf(buf, "%d", port);
    out = name + " [CellInputPort] port " + buf + " output " + sink;

    sprintf(buf, "%f", latency);
    out += (String) " latency " + buf;
    
    return out;
    
}

//
// CellInputPortCommand
//
// Input:  (Tcl command handler)
//
// Output:
//
// CellInputPort (data type) Tcl command handler.  This command
// creates a new CellInputPortObject and registers a Tcl command
// with the same name as the object so that it can be manipulated.
//
// Form of this command is:
//
// CellInputPort name sink port latency
//
int CellInputPortCmd(ClientData clientData, Tcl_Interp *interp,
			 int argc, char *argv[])
{

    if (argc != 5) {
	Tcl_SetResult(interp,
		      "Usage:  CellInputPort name sink port latency",
		      TCL_STATIC);
	return(TCL_ERROR);
    }
    else {
	int rc;
	char *nm;
	String s;
	int p;
	double l;
	CellInputPort *cip;
	extern Simulation *sim;

	// get string arguments
	nm = argv[1];
	s = argv[2];
	// get numeric arguments
	rc = Tcl_GetInt(interp, argv[3], &p);
	if (rc == TCL_OK) {
	    rc = Tcl_GetDouble(interp, argv[4], &l);
	    if (rc == TCL_OK) {

		// Make new object and make a Tcl command corresponding
		// to the object.
		cip = new CellInputPort(nm, sim, p, s, l);
		Tcl_CreateCommand(interp, nm,
				  CellInputPortObjectCmd,
				  (ClientData) cip,
				  CellInputPortDeleteProc);
	    }
	}
	return rc;
    }
}
    
//
// CellInputPortObjectCommand
//
// Input:  (as a Tcl comand handler)
//
// Output:
//
// Handle Tcl commands corresponding to one particular CellInputPort.
//
int CellInputPortObjectCmd(ClientData clientData, Tcl_Interp *interp,
			   int argc, char *argv[])
{
    return (ScheduledObjectCmd(clientData, interp, argc, argv));
}

//
// CellInputPortDeleteProc
//
// (Tcl DeleteProc)
//
void CellInputPortDeleteProc(ClientData clientData)
{
    delete ((CellInputPort *) clientData);
}

