static char rcsid[] = "$Id: CellQueue.cc,v 1.3 1996-01-10 17:05:36-08 bmah Exp $";
//
// $Id: CellQueue.cc,v 1.3 1996-01-10 17:05:36-08 bmah Exp $
//
// CellQueue.cc
// Bruce A. Mah (bmah@CS.Berkeley.EDU)
// 
// Copyright (c) 1995, 1996 The Regents of the University of California
// All Rights Reserved
// Created by Bruce A. Mah
// Department of Electrical Engineering and Computer Sciences,
//   Computer Science Division
//
// Part of an ATM switch, this is a base class for a set of
// cell output queues.  Subclasses of this class will handle
// different queueing disciplines.
//

#include <iostream.h>
#include <String.h>
#include "user.h"
#include "AtmCell.h"
#include "CellQueue.h"
#include "Tcl.h"
#include "Simulation.h"
#include "String.ScheduledObjectPtr.CHMap.h"
#include "inline.h"

//
// CellQueue::Output
//
// Input:  None
//
// Output:  String with an ASCII representation of this object.
//
String CellQueue::Output()
{

    String out;

    out = name + " [unknown CellQueue object] " + Output2();

}

String CellQueue::Output2()
{

    String out;
    char buf[30];

    sprintf(buf, "%f", latency);
    out = " output " + sink + " latency " + buf;
    
    sprintf(buf, "%f", bitrate);
    out += (String) " bitrate " + buf;

    return out;
    
}

//
// CellQueueObjectCmd
//
// (Tcl object command handler)
//
int CellQueueObjectCmd(ClientData clientData, Tcl_Interp *interp,
		       int argc, char *argv[])
{
    CellQueue *object = (CellQueue *) clientData;

    if (argc == 1) {
	return(ScheduledObjectCmd(clientData, interp, argc, argv));
    }
    else {

	//
	// cq output name
	//
	// Set output of cells
	//
	if (!strcmp(argv[1], "output")) {

	    if (argc == 3) {

		String outName;	// object name

		outName = argv[2];
		
		// set new object...zero out cached object pointer
		object->sink = outName;
		object->sinkObject = NULL;
		
		return (TCL_OK);

	    }
	    else {
		Tcl_SetResult(interp, "wrong # args", TCL_STATIC);
		return(TCL_ERROR);
	    }
	}

	//
	// cq latency number
	//
	// Set output port latency
	//
	else if (!strcmp(argv[1], "latency")) {

	    if (argc == 3) {

		double lat;	// new latency
		int rc;		// Tcl return code

		rc = Tcl_GetDouble(interp, argv[2], &lat);
		if (rc == TCL_OK) {

		    object->latency = lat;
		
		}
		return (rc);

	    }
	    else {
		Tcl_SetResult(interp, "wrong # args", TCL_STATIC);
		return(TCL_ERROR);
	    }
	}

	//
	// cq bitrate number
	//
	// Set output port bitrate
	//
	else if (!strcmp(argv[1], "bitrate")) {

	    if (argc == 3) {

		double br;	// bitrate
		int rc;		// Tcl return code

		rc = Tcl_GetDouble(interp, argv[2], &br);
		if (rc == TCL_OK) {

		    object->bitrate = br;
		
		}
		return (rc);

	    }
	    else {
		Tcl_SetResult(interp, "wrong # args", TCL_STATIC);
		return(TCL_ERROR);
	    }
	}
	else {
	    return (ScheduledObjectCmd(clientData, interp, argc, argv));
	}

    }

}
