static char rcsid[] = "$Id: CellQueueFifo.cc,v 1.7 1996-01-10 17:05:38-08 bmah Exp $";
//
// $Id: CellQueueFifo.cc,v 1.7 1996-01-10 17:05:38-08 bmah Exp $
//
// CellQueueFifo.cc
// Bruce A. Mah (bmah@CS.Berkeley.EDU)
// 
// Copyright (c) 1995, 1996 The Regents of the University of California
// All Rights Reserved
// Created by Bruce A. Mah
// Department of Electrical Engineering and Computer Sciences,
//   Computer Science Division
//
// FIFO output queue for ATM cells.  Not normally used alone, but
// as a part of an ATM switch.
//

#include <iostream.h>
#include <String.h>
#include "user.h"
#include "AtmCell.h"
#include "AtmCellList.h"
#include "Tcl.h"
#include "Simulation.h"
#include "String.ScheduledObjectPtr.CHMap.h"
#include "CellQueue.h"
#include "CellQueueFifo.h"
#include "inline.h"


//
// CellQueueFifo::Handler
//
// Input:  (event handler)
//
// Output:
//
void
CellQueueFifo::Handler(Event *e)
{

    double time = e->ev_time;
    EventType type = e->ev_type;
    void *data = e->ev_data;

    switch (type) {
	
	//
	// EventCellArrival
	//
	// Cell showed up.  If we're currently busy (transmitting),
	// then enqueue the cell.  Otherwise, send it out immediately.
	//
	case (EventCellArrival):
	{
	    AtmCell *cell = (AtmCell *) data;

	    if (busy) {
		queue.EnqueueTail(cell);
	    }
	    else {

		// Give the cell to the next object
		Lookup(sinkObject, sink);
		Post(time + latency + AtmCellSize/bitrate, sinkObject,
		     EventCellArrival, data);

		// We're busy now, but be ready for next cell
		busy = true;
		Post(time + AtmCellSize/bitrate,
		     this, EventCellTransmitComplete, NULL);

	    }
	}
	break;

	//
	// EventCellTransmitComplete
	//
	// Cell just finished transmitting.  If more cells queued up,
	// grab the next one and send it.
	//
	case (EventCellTransmitComplete):
	{
	    if (queue.length) {

		// Transmit the next cell on the queue
		Lookup(sinkObject, sink);
		Post(time + latency + AtmCellSize/bitrate, sinkObject,
		     EventCellArrival,
		     queue.Dequeue());

		// Wake us up when cell is gone
		Post(time + AtmCellSize/bitrate,
		     this, EventCellTransmitComplete, NULL);
		
	    }
	    else {

		// No more work to be done!
		busy = false;
	    }
	}
	break;

	default:
	{
	    IF_DEBUG(0, {
		DEBUG_PREFIX("CellQueueFifo");
		*(sim->debugStream) << "bad EventType " << type << "\n";
	    });
	}
	break;

    }

}

//
// CellQueueFifo::Output
//
// Input:  None
//
// Output:  String with an ASCII representation of this object.
//
String CellQueueFifo::Output()
{

    String out;

    out = name + " [CellQueueFifo]" + Output2();

    return out;
    
}

//
// CellQueueFifoCommand
//
// Input:  (Tcl command handler)
//
// Output:
//
// CellQueueFifo (data type) Tcl command handler.  This command
// creates a new CellQueueFifoObject and registers a Tcl command
// with the same name as the object so that it can be manipulated.
//
// Form of this command is:
//
// CellQueueFifo name
//
int CellQueueFifoCmd(ClientData clientData, Tcl_Interp *interp,
			 int argc, char *argv[])
{

    if (argc != 2) {
	Tcl_SetResult(interp, "Usage:  CellQueueFifo name", TCL_STATIC);
	return(TCL_ERROR);
    }
    else {
	int rc = TCL_OK;
	char *nm;
	CellQueueFifo *cqf;
	extern Simulation *sim;

	// get string arguments
	nm = argv[1];

	// Make new object and make a Tcl command corresponding
	// to the object.
	cqf = new CellQueueFifo(nm, sim);
	Tcl_CreateCommand(interp, nm,
			  CellQueueFifoObjectCmd,
			  (ClientData) cqf,
			  CellQueueFifoDeleteProc);

	return rc;
    }
}
    
//
// CellQueueFifoObjectCommand
//
// (Tcl object command handler)
//
int CellQueueFifoObjectCmd(ClientData clientData, Tcl_Interp *interp,
			   int argc, char *argv[])
{
    return CellQueueObjectCmd(clientData, interp, argc, argv);
}

//
// CellQueueFifoDeleteProc
//
// (Tcl DeleteProc)
//
void CellQueueFifoDeleteProc(ClientData clientData)
{
    delete ((CellQueueFifo *) clientData);
}



