static char rcsid[] = "$Id: CellSink.cc,v 1.4 1996-01-10 17:05:41-08 bmah Exp $";
//
// $Id: CellSink.cc,v 1.4 1996-01-10 17:05:41-08 bmah Exp $
//
// CellSink.cc
// Bruce A. Mah (bmah@CS.Berkeley.EDU)
// 
// Copyright (c) 1995, 1996 The Regents of the University of California
// All Rights Reserved
// Created by Bruce A. Mah
// Department of Electrical Engineering and Computer Sciences,
//   Computer Science Division
//

#include "CellSink.h"

//
// CellSink::Handler
//
// (simulation event handler
//
void CellSink::Handler(Event *e)
{
    double time = e->ev_time;
    EventType type = e->ev_type;
    void *data = e->ev_data;

    switch (type) {

	//
	// EventCellArrival
	//
	case (EventCellArrival):
	{
	    AtmCell *cell = (AtmCell *) data;

	    // if debugging on, print cell data on stdout
	    IF_DEBUG(2, {
		cout << time << ": " << name << " [CellSink] " <<
		  (char *) cell->payload << "\n";
	    });

	    // delete cell to prevent memory leaks
	    delete ((void *) cell->payload);
	    delete cell;
	}
	break;

	default:
	{
	    IF_DEBUG(0, {
		DEBUG_PREFIX("CellSink");
		cout << "bad EventType " << type << "\n";
	    });
	}
	break;

    }

}

//
// CellSink::Output
//
// Input:  None
//
// Output:  String containing ASCII representation of CellSource object.
//
String CellSink::Output()
{
    return name + " [CellSink]";
}

//
// CellSinkCmd
//
// (Tcl command handler)
//
// Create a new CellSink object.  Syntax:
//
// CellSink name
//
int CellSinkCmd(ClientData clientData, Tcl_Interp *interp,
		  int argc, char *argv[])
{

    if (argc != 2) {
	Tcl_SetResult(interp, "usage:  CellSink name", TCL_STATIC);
	return(TCL_ERROR);
    }
    else {
	char *nm;		// object name
	CellSink *theCellSink;
	extern Simulation *sim;

	nm = argv[1];
	
	theCellSink = new CellSink(nm, sim);
	Tcl_CreateCommand(interp, nm,
			  CellSinkObjectCmd,
			  (ClientData) theCellSink,
			  CellSinkDeleteProc);
	return(TCL_OK);

    }
}

//
// CellSinkObjectCmd
//
// (Tcl object command handler)
//
int CellSinkObjectCmd(ClientData clientData, Tcl_Interp *interp,
		      int argc, char *argv[])
{
    return(ScheduledObjectCmd(clientData, interp, argc, argv));
}

//
// CellSinkDeleteProc
//
// (Tcl DeleteProc)
//
void CellSinkDeleteProc(ClientData clientData)
{
    delete ((CellSink *) clientData);
}

