static char rcsid[] = "$Id: CellSource.cc,v 1.5 1996-01-10 17:05:42-08 bmah Exp $";
//
// $Id: CellSource.cc,v 1.5 1996-01-10 17:05:42-08 bmah Exp $
//
// CellSource.cc
// Bruce A. Mah (bmah@CS.Berkeley.EDU)
// 
// Copyright (c) 1995, 1996 The Regents of the University of California
// All Rights Reserved
// Created by Bruce A. Mah
// Department of Electrical Engineering and Computer Sciences,
//   Computer Science Division
//

#include <Poisson.h>
#include "CellSource.h"
#include "CellGoBackN.h"
#include "inline.h"

//
// CellSource::CellSource
//
CellSource::CellSource(const String &name, Simulation *si, String &theSink,
		       int theVci, double theXave) :
		       ScheduledObject(name, si),
		       sinkName(theSink), sink(NULL),
		       vcid(theVci), xave(theXave) {

	       // Get new random number generator
	       p = new NegativeExpntl(xave, sim->rng);

	       // Calculate initial starting time
	       double start = (*p)();

	       // Post event for first cell
	       Post(sim->scheduler->clock + start, this,
		    EventTimer, NULL);

	   }


//
// CellSource::Handler
//
// (simulation event handler
//
void CellSource::Handler(Event *e)
{
    double time = e->ev_time;
    EventType type = e->ev_type;
    void *data = e->ev_data;

    switch (type) {

	//
	// EventTimer
	//
	// Fire off another cell and reset
	//
	case (EventTimer):
	{
	    char *cellData = new char[100];

	    // Make up a cell and fire it off.
	    sprintf(cellData, "Cell created time %f", time);
	    AtmCell *cell = new AtmCell(0, (void *) cellData, 0, 0);
	    cell->vcid = vcid;
	    Lookup(sink, sinkName);
	    Post(time, sink, EventCellSend, cell);

	    // Set new timer value
	    double timerValue = (*p)();
	    Post(time + timerValue, this, EventTimer, NULL);
	    break;
	}
	
	default:
	{
	    IF_DEBUG(0, {
		DEBUG_PREFIX("CellSource");
		cout << "bad EventType " << type << "\n";
	    });
	}
	break;

    }

}

//
// CellSource::Output
//
// Input:  None
//
// Output:  String containing ASCII representation of CellSource object.
//
String CellSource::Output()
{
    String out;
    char buf[30];
    
    out = name + " [CellSource] sink " + sinkName + " vcid ";

    sprintf(buf, "%d", vcid);
    out += (String) buf + " xave ";

    sprintf(buf, "%f", xave);
    out += (String) buf;

    return out;
    
}

//
// CellSourceCmd
//
// (Tcl command handler)
//
// Create a new CellSource object.  Syntax:
//
// CellSource name sink vci xave
//
int CellSourceCmd(ClientData clientData, Tcl_Interp *interp,
		  int argc, char *argv[])
{

    if (argc != 5) {
	Tcl_SetResult(interp, "Usage;  CellSource name sink vcid xave",
		      TCL_STATIC);
	return(TCL_ERROR);
    }
    else {
	char *nm;		// object name
	String sk;		// sink name
	int theVcid;		// VCID
	double theXave;		// average interarrival time
	int rc;			// Tcl return code
	CellSource *theCellSource;
	extern Simulation *sim;

	nm = argv[1];
	sk = argv[2];
	rc = Tcl_GetInt(interp, argv[3], &theVcid);
	if (rc == TCL_OK) {
	    rc = Tcl_GetDouble(interp, argv[4], &theXave);
	    if (rc == TCL_OK) {
		theCellSource = new CellSource(nm, sim, sk, theVcid, theXave);
		Tcl_CreateCommand(interp, nm,
				  CellSourceObjectCmd,
				  (ClientData) theCellSource,
				  CellSourceDeleteProc);
	    }
	}
	return rc;
    }
}

//
// CellSourceObjectCmd
//
// (Tcl object command handler)
//
int CellSourceObjectCmd(ClientData clientData, Tcl_Interp *interp,
			int argc, char *argv[])
{
    return(ScheduledObjectCmd(clientData, interp, argc, argv));
}

//
// CellSourceDeleteProc
//
// (Tcl DeleteProc)
//
void CellSourceDeleteProc(ClientData clientData)
{
    delete ((CellSource *) clientData);
}

