// -*- c++ -*-
// $Id: CellSource.h,v 1.4 1996-01-10 17:05:42-08 bmah Exp $
//
// CellSource.h
// Bruce A. Mah (bmah@CS.Berkeley.EDU)
//
// Copyright (c) 1995, 1996 The Regents of the University of California
// All Rights Reserved
// Created by Bruce A. Mah
// Department of Electrical Engineering and Computer Sciences,
//   Computer Science Division
//

#ifndef CELLSOURCE_H
#define CELLSOURCE_H

#include <NegExp.h>
#include "AtmCell.h"
#include "ScheduledObject.h"
#include "Simulation.h"

class CellSource : public ScheduledObject {

    friend int CellSourceObjectCmd(ClientData clientData, Tcl_Interp *interp,
				   int argc, char *argv[]);
    friend void CellSourceDeleteProc(ClientData clientData);

    public:

    CellSource(const String &name, Simulation *si, String &theSink,
	       int theVci, double theXave);

    ~CellSource() {
	delete p;
    }

    virtual String Output();

    virtual void Handler(Event *e);

    protected:

    const String sinkName;	// name of recipient of cells
    ScheduledObject *sink;	// pointer to recipient of cells
    const int vcid;		// VCID to use for cells
    const double xave;		// Xave for cell stream
    NegativeExpntl *p;		// negative exponential random number process

};

extern int CellSourceCmd(ClientData clientData, Tcl_Interp *interp,
			 int argc, char *argv[]);
extern int CellSourceObjectCmd(ClientData clientData, Tcl_Interp *interp,
			       int argc, char *argv[]);
extern void CellSourceDeleteProc(ClientData clientData);

#endif //
