// -*- c++ -*-
//
// $Id: DeviceDriver.h,v 1.5 1996-02-02 15:56:12-08 bmah Exp $
//
// DeviceDriver.h
// Bruce A. Mah (bmah@CS.Berkeley.EDU)
//
// Copyright (c) 1995, 1996 The Regents of the University of California
// All Rights Reserved
// Created by Bruce A. Mah
// Department of Electrical Engineering and Computer Sciences,
//   Computer Science Division
//
// IP device driver parent class.
//

#ifndef DEVICEDRIVER_H
#define DEVICEDRIVER_H

#include "ScheduledObject.h"
#include "IpPacket.h"
#include "UdpPacket.h"
#include "TcpSegment.h"

class DeviceDriver : public ScheduledObject {

    friend int
    DeviceDriverObjectCmd(ClientData clientData, Tcl_Interp *interp,
			  int argc, char *argv[]);

    public:

    DeviceDriver(const String &name, Simulation *si) :
    ScheduledObject(name, si) {
	if_ipackets = 0;
	if_ierrors = 0;
	if_opackets = 0;
	if_oerrors = 0;
	if_ibytes = 0;
	if_obytes = 0;
	if_qdrops = 0;
    }
    
    // Interface statistics, essentially the same as for
    // the BSD ifnet structure.
    int if_ipackets; 			// packets received on interface
    int if_ierrors;			// input errors on interface
    int if_opackets;			// packets sent on interface
    int if_oerrors;			// output errors on interface
    int if_ibytes;			// total number of octets received
    int if_obytes;			// total number of octets sent
    int if_qdrops;			// dropped on input, this interface

    protected:

    String upName;		// name of upper-layer protocol object
    ScheduledObject *up;	// upper-layer protocol object

    String downName;		// lower-layer protocol object
    ScheduledObject *down;
};

extern int DeviceDriverObjectCmd(ClientData clientData, Tcl_Interp *interp,
				 int argc, char *argv[]);

#endif // DEVICEDRIVER_H
