static char rcsid[] = "$Id: Empirical.cc,v 1.5 1995-12-01 13:14:08-08 bmah Exp $";
// -*- c++ -*-
// $Id: Empirical.cc,v 1.5 1995-12-01 13:14:08-08 bmah Exp $
//
// Empirical.h
// Bruce A. Mah (bmah@CS.Berkeley.EDU)
//
// Copyright (c) 1995 The Regents of the University of California
// All Rights Reserved
// Created by Bruce A. Mah
// Department of Electrical Engineering and Computer Sciences,
//   Computer Science Division
//
// Base class for empirical distribution classes.

#include "Empirical.h"

//
// EmpiricalDouble::operator()
//
// Input:  none
//
// Output:  Number from the empirically-defined distribution.
//
double Empirical::operator()() {

    register int bottom, top, current; // fenceposts for binary search

    // Get a Uniform[0,1] number from the RNG.
    double u = pGenerator->asDouble();

    // Special-case when we're below the lowest value in the
    // table.
    if (u < elements[0].cdf) {
	return (elements[0].value);
    }

    // Do a binary search.  As an invariant, we are always considering
    // the elements [bottom..top], inclusive.  A test of the element
    // current consists of checking to see if the value u lies in
    // the range elements[current].cdf to elements[current+1].cdf.
    // Note the loop is guaranteed to terminate because the range
    // [bottom..top] is shrinking by at least one element per loop
    // iteration.
    else {

	bottom = 0;
	top = numElements - 1;

	while (true) {

	    // Find midpoint.
	    current = (top + bottom) >> 1;
	    
	    // Test to see if u lies in the range of elements[current]...
	    // elements[current + 1].
	    if ((u >= elements[current].cdf) &&
		(u < elements[current + 1].cdf)) {

		return
		  interpolate(elements[current].cdf, elements[current + 1].cdf,
			      elements[current].value, elements[current + 1].value,
			      u);

	    }

	    // Check to see if u is too small (in which case we move the
	    // top fencepost down below current).
	    else if (u < elements[current].cdf) {
		top = current - 1;
	    }

	    // Check to see if u is too big (in which case we move the
	    // bottom fencepost above current)
	    else if (u > elements[current + 1].cdf) {
		bottom = current + 1;
	    }

	    // Shouldn't get down here...this means something's wrong.
	    else {
		abort();
	    }

	}
    }
}
