// -*- c++ -*-
// $Id: Empirical.h,v 1.2 1995-12-01 13:12:25-08 bmah Exp $
//
// Empirical.h
// Bruce A. Mah (bmah@CS.Berkeley.EDU)
//
// Copyright (c) 1995 The Regents of the University of California
// All Rights Reserved
// Created by Bruce A. Mah
// Department of Electrical Engineering and Computer Sciences,
//   Computer Science Division
//
// Empirical random number distribution class.  This class is
// a base class for continuous and discrete distributions.
//

#ifndef EMPIRICALDOUBLE_H
#define EMPIRICALDOUBLE_H

#include <Random.h>

// An array of DoubleDistElements expresses the CDF of a distribution,
// sorted so we can access its quickly via binary search.  We do
// linear interpolation between the values in the array.
struct DoubleDistElement {
    double value;
    double cdf;
};

class Empirical : public Random {

public:
    Empirical(int num, const DoubleDistElement *dde, RNG *gen) :
    numElements(num), elements(dde), Random(gen) {
    }

    virtual double operator()();    

    virtual double interpolate(double x1, double x2,
			       double y1, double y2,
			       double x) = 0;

protected:
    const int numElements;
    const DoubleDistElement *elements;

};

#endif // EMPIRICAL_H
