// -*- c++ -*-
//
// $Id: Event.h,v 1.9 1996-01-31 11:54:13-08 bmah Exp $
//
// Event.h
// Bruce A. Mah (bmah@CS.Berkeley.EDU)
//
// Copyright (c) 1995, 1996 The Regents of the University of California
// All Rights Reserved
// Created by Bruce A. Mah
// Department of Electrical Engineering and Computer Sciences,
//   Computer Science Division
//
// Definitions for a single simulation event.  This class gets used in
// two different contexts:
//
// 1.  The Calendar class uses Events as a part of its scheduler.
// 
// 2.  Certain objects may have need to store objects (the class motivating
// this construction is the HttpServer class).
//

#ifndef EVENT_H
#define EVENT_H

#include <iostream.h>

class ScheduledObject;		// Want to declare this class during
				// compilation without defining it

enum EventType {

    EventNull = 0,
    
    EventCellArrival,
    EventCellTransmitComplete,

    EventFrameSend,
    EventFrameArrival,
    EventFrameTransmitComplete,

    EventTimer,
    EventTimer2,

    EventCellSend,

    EventCall,

    EventHalt,
    EventDebug,
      
    EventPacketSend,
    EventPacketArrival,

    EventIpSend,
    EventIpArrival,

    EventUdpSend,
    EventUdpArrival,

    EventTcpSend,
    EventTcpArrival,
    EventTcpConnect,
    EventTcpClosed,
    EventTcpError

};

//
// Structure of a single simulation event.
//
class Event {

    friend class Calendar;
    friend class EventList;

    public:
    double ev_time;		// ev_time is the only required public
				// member (event time)

    ScheduledObject *ev_sender;	// object sending event
    ScheduledObject *ev_object;	// object to receive event
    EventType ev_type;		// event type
    void *ev_data;		// event data

    Event() {
	ev_sender = NULL;
    }
    Event(double time, ScheduledObject *object, EventType type,
	  void *data) :
    ev_time(time), ev_object(object), ev_type(type), ev_data(data) {
	ev_sender = NULL;
    }
    
    private:
    Event *ev_prev, *ev_next;	// pointers for DLL

};

extern ostream& operator<<(ostream &s, EventType et);

#endif // EVENT_H
