static char rcsid[] = "$Id: EventSink.cc,v 1.5 1996-01-10 17:05:45-08 bmah Exp $";
//
// $Id: EventSink.cc,v 1.5 1996-01-10 17:05:45-08 bmah Exp $
//
// EventSink.cc
// Bruce A. Mah (bmah@CS.Berkeley.EDU)
//
// Copyright (c) 1995, 1996 The Regents of the University of California
// All Rights Reserved
// Created by Bruce A. Mah
// Department of Electrical Engineering and Computer Sciences,
//   Computer Science Division
//
// Event sink (throws away all events it's given).  Used primarily
// for debugging
//

#include "Tcl.h"
#include "EventSink.h"
#include "inline.h"

//
// EventSink::Handler
//
// (Event handler)
//
void EventSink::Handler(Event *e)
{
    double time = e->ev_time;
    EventType type = e->ev_type;
    void *data = e->ev_data;

    IF_DEBUG(2, {
	cout << time << ": " << name << " [EventSink] got event type " <<
	  type << "\n";
    });

    // To prevent memory leaks, we need to discard the data payload
    // of the event.
    delete data;
}

//
// EventSink::Output
//
// Input:  None
//
// Output:  String with an ASCII representation of this object.
//
String EventSink::Output()
{
    return (String) (name + " [EventSink]");
}

//
// EventSinkCmd
//
// (Tcl command handler procedure)
//
// EventSink name vcids gap port
//
int EventSinkCmd(ClientData clientData, Tcl_Interp *interp,
	   int argc, char *argv[])
{
    if (argc != 2) {
	Tcl_SetResult(interp, "usage:  EventSink name", TCL_STATIC);
	return(TCL_ERROR);
    }
    else {

	char *nm;		// object name
	extern Simulation *sim;	// global simulation object

	nm = argv[1];

	EventSink *es = new EventSink(nm, sim);
	Tcl_CreateCommand(interp, nm,
			  EventSinkObjectCmd,
			  (ClientData) es,
			  EventSinkDeleteProc);
	return(TCL_OK);
    }
}

//
// EventSinkObjectCmd
//
// (Tcl object handler procedure)
//
int EventSinkObjectCmd(ClientData clientData, Tcl_Interp *interp,
		 int argc, char *argv[])
{
    return(ScheduledObjectCmd(clientData, interp, argc, argv));
}

//
// EventSinkDeleteProc
//
// (Tcl DeleteProc)
//
void EventSinkDeleteProc(ClientData clientData)
{
    delete ((EventSink *) clientData);
}
