// -*- c++ -*-
//
// $Id: FtpClient.h,v 1.7 1996-01-29 16:31:01-08 bmah Exp $
//
// FtpClient.h
// Bruce A. Mah (bmah@CS.Berkeley.EDU)
//
// Copyright (c) 1995, 1996 The Regents of the University of California
// All Rights Reserved
// Created by Bruce A. Mah
// Department of Electrical Engineering and Computer Sciences,
//   Computer Science Division
//

#ifndef FTPCLIENT_H
#define FTPCLIENT_H

#include <SmplStat.h>
#include <iostream.h>
#include "AppTcpClient.h"
#include "Simulation.h"
#include "ftpproto.h"

//
// FTP client state machine states
//
enum FtpClientState {
    FtpClientStateStart,	        // Uninitialized
    FtpClientStateConnectingControl,	// Waiting for connect(control) to
					// complete
    FtpClientStateAwaitPortResponse,	// Waiting for response to PORT command
    FtpClientStateDataTransfer,		// Transferring data
    FtpClientStateDoneData,		// Waiting to pick next item (or quit)
    FtpClientStateDone			// Session complete (not used)
};

extern ostream& operator<<(ostream &s, FtpClientState ftpcs);

class FtpClient : public AppTcpClient {

    friend int
    FtpClientCmd(ClientData clientData, Tcl_Interp *interp,
	   int argc, char *argv[]);
    friend int
    FtpClientObjectCmd(ClientData clientData, Tcl_Interp *interp,
		 int argc, char *argv[]);

    public:

    FtpClient(const String &name, Simulation *si, double initTime,
	      String &t, InAddr s, short p) :
    AppTcpClient(name, si, initTime, t, s, p) {

	startTime = initTime;
	endTime = initTime;
	state = FtpClientStateStart;
	totalBytes = 0;
	totalItems = 0;
	
    }

    virtual ~FtpClient() {
	*(sim->logStream) << name << " [FtpClient] session lifetime " << endTime - startTime << " bytes " << totalBytes << " items " << totalItems << "\n";
    }

    virtual String Output(void);
    virtual void Handler (Event *e);

    protected:				  
    
    double startTime;		// object start time
    double endTime;		// object ending time
    double dataStartTime;	// dat sending start time

    int numItems;		// number of items to send
    
    private:

    FtpClientState state;	// overrides AppTcpClient::state
    TcpPcb *tpControl;		// control connection (AppTcpClient::tp not
				// used)
    TcpPcb *tpDataListen;	// listening socket
    TcpPcb *tpData;		// data connection

    double transferStartTime;	// starting time of transfer
    int transferBytes;		// number of bytes this transfer
    int totalBytes;		// total number of bytes transferred
    int totalItems;		// total number of items transferred

};

extern int FtpClientCmd(ClientData clientData, Tcl_Interp *interp,
		  int argc, char *argv[]);
extern int FtpClientObjectCmd(ClientData clientData, Tcl_Interp *interp,
			int argc, char *argv[]);
extern void FtpClientDeleteProc(ClientData clientData);

#endif // FTPCLIENT_H


