// -*- c++ -*-
//
// $Id: FtpServer.h,v 1.7 1996-04-16 11:03:16-07 bmah Exp $
//
// FtpServer.h
// Bruce A. Mah (bmah@CS.Berkeley.EDU)
//
// Copyright (c) 1996 The Regents of the University of California
// All Rights Reserved
// Created by Bruce A. Mah
// Department of Electrical Engineering and Computer Sciences,
//   Computer Science Division
//

#ifndef FTPSERVER_H
#define FTPSERVER_H

#include <SmplStat.h>
#include <iostream.h>
#include "AppTcpServer.h"
#include "Simulation.h"
#include "ftpproto.h"

//
// FTP server state machine states (per-session)
//
enum FtpServerState {
    FtpServerStateStart,	        // Uninitialized
    FtpServerStateAwaitPort,		// Awaiting PORT command from client
    FtpServerStateConnectData,		// Connecting to client for data
    FtpServerStateData,		        // Data send
    FtpServerStateDone			// Session complete (not used)
};

typedef struct FtpServerSession {
    FtpServerState state;		// state of this session
    TcpPcb *tpControl;			// control connection
    TcpPcb *tpData;			// data connection
    int clientPort;			// client's data port (from control
					// message)
    FtpServerSession *next;		// next pointer for SLL
};

extern ostream& operator<<(ostream &s, FtpServerState ftpcs);

class FtpServer : public AppTcpServer {

    friend int
    FtpServerCmd(ClientData clientData, Tcl_Interp *interp,
	   int argc, char *argv[]);
    friend int
    FtpServerObjectCmd(ClientData clientData, Tcl_Interp *interp,
		 int argc, char *argv[]);

    public:

    FtpServer(const String &name, Simulation *si, double initTime,
	      String &t, short p, short dp) :
    AppTcpServer(name, si, initTime, t, p), dataPort (dp) {
	sessions = NULL;
    }

    virtual ~FtpServer() {
    }

    virtual String Output(void);
    virtual void Handler (Event *e);

    const unsigned short dataPort; // data port

    protected:				  
    
    private:
    FtpServerSession *sessions;	// SLL of currently active FTP sessions
    TcpPcb *tpControlListen;	// listen socket for control connections
    int bytesRemaining;		// bytes remaining to send in the current item

    FtpServerSession *FindSession(TcpPcb *tp);

};

extern int FtpServerCmd(ClientData clientData, Tcl_Interp *interp,
		  int argc, char *argv[]);
extern int FtpServerObjectCmd(ClientData clientData, Tcl_Interp *interp,
			int argc, char *argv[]);
extern void FtpServerDeleteProc(ClientData clientData);

#endif // FTPCLIENT_H


