// -*- c++ -*-
//
// $Id: HttpClient.h,v 1.10 1996-03-06 20:45:36-08 bmah Exp $
//   based on:
// $ID: FtpClient.h,v 1.2 1995-10-25 14:12:29-07 bmah Exp $
//
// HttpClient.h
// Bruce A. Mah (bmah@CS.Berkeley.EDU)
//
// Copyright (c) 1995, 1996 The Regents of the University of California
// All Rights Reserved
// Created by Bruce A. Mah
// Department of Electrical Engineering and Computer Sciences,
//   Computer Science Division
//

#ifndef HTTPCLIENT_H
#define HTTPCLIENT_H

#include <SmplStat.h>
#include <iostream.h>
#include "AppTcpClient.h"
#include "Simulation.h"
#include "Zipf.h"

//
// FTP client state machine states
//
enum HttpClientState {
    HttpClientStateStart,	// uninitialized
    HttpClientStateAwaitConnect, // waiting for server connection
    HttpClientStateAwaitReply,	 // waiting for reply from server
    HttpClientStateNextPage	 // waiting to select next page
};

extern ostream& operator<<(ostream &s, HttpClientState ftpcs);

class HttpClient : public AppTcpClient {

    friend int
    HttpClientCmd(ClientData clientData, Tcl_Interp *interp,
		  int argc, char *argv[]);
    friend int
    HttpClientObjectCmd(ClientData clientData, Tcl_Interp *interp,
			int argc, char *argv[]);

    public:

    HttpClient(const String &name, Simulation *si, double initTime,
	       String &t, int numserveraddrs, InAddr *serveraddrs, short p,
	       double th) :
    // Fifth parameter to AppTcpClient initializer might be a little
    // confusing.  Basically we need to give *some* IP address to the
    // AppTcpClient constructor, even though it's going to be overwritten.
    // Rather than try to make up a dummy address and pass it in, we give
    // it the first IP address from the server list, which is a little
    // easier to do.
    AppTcpClient(name, si, initTime, t, serveraddrs[0], p),
    numServers(numserveraddrs), servers(serveraddrs), threshold(th) {

	extern RNG *rng;
	
	startTime = initTime;
	endTime = initTime;
	state = HttpClientStateStart;

	serverIndex = new Zipf(numServers, rng);
	
    }

    virtual ~HttpClient() {

	delete [] servers;
	delete serverIndex;

	*(sim->logStream) << name << " [HttpClient] session lifetime " << endTime - startTime << " requestbytes " << totalRequestBytes << " replybytes " << totalReplyBytes << " items " << totalItems << " pages " << totalPages << "\n";

    }

    virtual String Output(void);
    virtual void Handler (Event *e);

    protected:				  
    
    double startTime;		// object start time
    double endTime;		// object ending time

    int numPagesOnServer;	// number of pages to get from current server
    int numItems;		// number of items to send on current page
    
    InAddr *servers;		// IP addresses of HTTP servers
    int numServers;		// number of HTTP servers in list

    virtual InAddr SelectServer(InAddr oldServer);

    const double threshold;	// think time threshold

    Zipf *serverIndex;		// random number distribution to pick server

    private:

    HttpClientState state;	// overrides AppTcpClient::state
    TcpPcb *tpData;		// data connection

    double itemStartTime;	// start time of current item
    int itemRequestBytes;	// number of bytes this item
    int itemReplyBytes;
    double pageStartTime;	// starting time of current web page
    int pageRequestBytes;	// number of bytes this page
    int pageReplyBytes;
    int pageItems;		// number of items this page
    int totalRequestBytes;	// total number of bytes transferred
    int totalReplyBytes;
    int totalItems;		// total number of items transferred
    int totalPages;		// total number of pages transferred

};

extern int HttpClientCmd(ClientData clientData, Tcl_Interp *interp,
			 int argc, char *argv[]);
extern int HttpClientObjectCmd(ClientData clientData, Tcl_Interp *interp,
			       int argc, char *argv[]);
extern void HttpClientDeleteProc(ClientData clientData);

#endif // HTTPCLIENT_H


