// -*- c++ -*-
//
// $Id: HttpServer.h,v 1.5 1996-03-05 15:00:33-08 bmah Exp $
//   based on:
// $ID: FtpServer.h,v 1.2 1995-10-25 11:30:48-07 bmah Exp $
//
// HttpServer.h
// Bruce A. Mah (bmah@CS.Berkeley.EDU)
//
// Copyright (c) 1995, 1996 The Regents of the University of California
// All Rights Reserved
// Created by Bruce A. Mah
// Department of Electrical Engineering and Computer Sciences,
//   Computer Science Division
//

#ifndef HTTPSERVER_H
#define HTTPSERVER_H

#include <SmplStat.h>
#include <iostream.h>
#include "AppTcpServer.h"
#include "Simulation.h"
#include "EventList.h"
#include "httpproto.h"

//
// HTTP server state machine states (per-session)
//
enum HttpServerState {
    HttpServerStateStart,	        // Uninitialized
    HttpServerStateConnected		// Awaiting request
};

typedef struct HttpServerSession {
    HttpServerState state;
    TcpPcb *tp;
    HttpServerSession *next;
};

extern ostream& operator<<(ostream &s, HttpServerState httpcs);

class HttpServer : public AppTcpServer {

    friend int
    HttpServerCmd(ClientData clientData, Tcl_Interp *interp,
		  int argc, char *argv[]);
    friend int
    HttpServerObjectCmd(ClientData clientData, Tcl_Interp *interp,
			int argc, char *argv[]);

    public:

    HttpServer(const String &name, Simulation *si, double initTime,
	       String &t, short p) :
    AppTcpServer(name, si, initTime, t, p) {
	state = HttpServerStateStart;
	totalRequestBytes = 0;
	totalReplyBytes = 0;
	totalItems = 0;
	sessions = 0;
    }

    virtual ~HttpServer() {

	*(sim->logStream) << name << " [HttpServer] requestbytes " << totalRequestBytes << " replybytes " << totalReplyBytes << " items " << totalItems << "\n";

    }

    virtual String Output(void);
    virtual void Handler (Event *e);

    protected:				  
    EventList eq;		// event queue
    HttpServerSession *sessions; // SLL of currently active HTTP sessions
    HttpServerState state;	// overrides AppTcpServer state
    TcpPcb *tpListen;		// listen socket for control connections
    TcpPcb *tpData;		// data socket
    int bytesRemaining;		// bytes remaining to send in the current item
    int totalRequestBytes;	// total number of request bytes from clients
    int totalReplyBytes;	// total number of reply bytes from server
    int totalItems;		// total number of items

};

extern int HttpServerCmd(ClientData clientData, Tcl_Interp *interp,
			 int argc, char *argv[]);
extern int HttpServerObjectCmd(ClientData clientData, Tcl_Interp *interp,
			       int argc, char *argv[]);
extern void HttpServerDeleteProc(ClientData clientData);

#endif // HTTPSERVER_H


