// -*- c++ -*-
//
// $Id: Ip.h,v 1.9 1996-02-02 15:56:18-08 bmah Exp $
//
// Ip.h
// Bruce A. Mah (bmah@CS.Berkeley.EDU)
//
// Copyright (c) 1995, 1996 The Regents of the University of California
// All Rights Reserved
// Created by Bruce A. Mah
// Department of Electrical Engineering and Computer Sciences,
//   Computer Science Division
//

#ifndef IP_H
#define IP_H

#include <String.h>
#include "ScheduledObject.h"
#include "DeviceDriver.h"
#include "IpPacket.h"

//
// IpInterface structure keeps information on a single IP interface
// attached to the host.  This structure is vaguely reminiscent of
// the in_ifaddr structure from BSD 4.4.
//
typedef struct IpInterface {

    InAddr ia_addr;		// IP address of interface
    InAddr ia_subnetmask;	// Subnet mask for this interface
    InAddr ia_broadaddr;	// Broadcast address for this subnet

    int mtu;			// MTU for this interface (currently
				// not used)

    String ia_objectName;
    DeviceDriver *ia_object;	// Device driver object for this
				// interface
    
    struct IpInterface *next;	// Next pointer for SLL

};

//
// A set of bitflags associated with each route
//
enum RouteFlags {
    RouteFlagNone = 0,		// no flags set
    RouteFlagGateway = 1,	// indirect route through a gateway
};

//
// IpRoute structure is one entry in the IP node's routing table.
// It is most similar to the BSD 4.3 hash-table-based routing tables,
// as described in [Leffler89], rather than the BSD 4.3Reno
// radix-tree-based routing described in [Wright95].
//
typedef struct IpRoute {

    InAddr rt_dst;		// Destination address
    InAddr rt_mask;		// Subnet mask for use with destination
				// address
    InAddr rt_gateway;		// Gateway address for this route
    
    RouteFlags rt_flags;	// Flags as described above

    IpInterface *rt_if;		// interface associated with this route

    struct IpRoute *next;	// Next pointer for SLL

};

//
// IpProto structure is similar to the proto_sw structure in BSD,
// except that the collection of these structures forms a linked list
// instead of a two-level lookup.
//
typedef struct IpProto {

    short pr_protocol;		// protocol number

    String pr_objectName;	// name of protocol object
    ScheduledObject *pr_object;	// pointer to protocol object

    struct IpProto *next;	// next pointer for SLL

};

class Ip : public ScheduledObject {

    friend int
    IpObjectCmd(ClientData clientData, Tcl_Interp *interp,
		int argc, char *argv[]);

    public:
    
    Ip(const String &name, Simulation *si) :
    ScheduledObject(name, si) {
	interfaces = NULL;
	routes = NULL;
	protocols = NULL;
    }

    virtual ~Ip();

    virtual String Output(void);
    virtual void Handler(Event *e);

    IpRoute *RouteLookup(InAddr target);
    void GetRouteInterface(IpRoute *r);

    protected:				  

    struct IpInterface *interfaces; // list of interface
    struct IpRoute *routes;	    // list of routes
    struct IpProto *protocols;	    // list of protocols

};

extern int IpCmd(ClientData clientData, Tcl_Interp *interp,
		 int argc, char *argv[]);
extern int IpObjectCmd(ClientData clientData, Tcl_Interp *interp,
		       int argc, char *argv[]);
extern void IpDeleteProc(ClientData clientData);

#endif // IP_H


