// -*- c++ -*-
//
// $Id: IpPacket.h,v 1.4 1996-02-21 10:35:11-08 bmah Exp $
//   based on:
// $ID: ipproto.h,v 1.5 1996-01-27 23:14:21-08 bmah Exp $
//
// IpPacket.h
// Bruce A. Mah (bmah@CS.Berkeley.EDU)
//
// Copyright (c) 1996 The Regents of the University of California
// All Rights Reserved
// Created by Bruce A. Mah
// Department of Electrical Engineering and Computer Sciences,
//   Computer Science Division
//
// IP packet definitions
//

#ifndef IPPACKET_H
#define IPPACKET_H

#include <stdlib.h>
#include "Pdu.h"
#include <iostream.h>

//
// Get some IP-related definitions we need.  Originally these were to
// come from the BSD 4.4Lite distribution, but compatability with some
// IP routines forced us to use the "native" versions of these files.
// So once again we're at the mercy of the operating system.
//
#include <sys/types.h>
#include <netinet/in.h>
#include <arpa/inet.h>

// Non-IP multicast-capable systems (i.e. Ultrix) require this defined
#ifndef IPPROTO_IGMP
#define IPPROTO_IGMP 2
#endif

//
// Definition of IP addresses
//
typedef struct in_addr InAddr;
#define InetNtoA(x) inet_ntoa(x)
#define InetAddr(x) inet_addr(x)
ostream& operator<<(ostream &s, InAddr ia);

//
// Definition of an IP message.  Basically this includes most of the fields
// in the IP header, plus some other needed for simulation efficiency.
// The main fields that are missing are those dealing with header lengths
// or with checksums.  Where possible, fields retain the same names and
// types they have in <netinet/ip.h> from the BSD 4.4Lite networking code.
// In particular, the various constants and macro definitions from
// the 4.4Lite sources should still be applicable here.
//
class IpPacket : public Pdu {

    public:
      
    IpPacket(Pdu *pay) : payload(pay) {
	if (payload) {
	    payload->refcount++;
	}
    }

    virtual ~IpPacket() {
	if (payload) {
	    (payload->refcount)--;
	    if (payload->refcount == 0) {
		delete payload;
	    }
	}
    }

    void Detatch() {
	if (payload) {
	    (payload->refcount)--;
	}
	payload = NULL;
    }

    virtual ostream& Print(ostream &s);

    u_char	ip_tos;		// Type of Service field.
    short	ip_len;		// Datagram total packet length
    u_short	ip_id;		// Datagram ID
    short	ip_off;		// Fragment offset
    u_char	ip_ttl;		// Time To Live
    u_char	ip_p;		// Transport layer protocol number
    InAddr	ip_src,		// Source and destination IP addresses
    		ip_dst,
    		ip_next;	// Next-hop IP gateway (if any), as
				// determined by IP routing.

    Pdu		*payload;	// packet payload (contents depend on
				// ip_p field)
};

const int IpHeaderSize = 20;

#endif // IPPACKET_H
