// -*- c++ -*-
//
// $Id: Lan.h,v 1.8 1996-02-21 16:13:50-08 bmah Exp $
//
// Lan.h
// Bruce A. Mah (bmah@CS.Berkeley.EDU)
//
// Copyright (c) 1995, 1996 The Regents of the University of California
// All Rights Reserved
// Created by Bruce A. Mah
// Department of Electrical Engineering and Computer Sciences,
//   Computer Science Division
//

#ifndef LAN_H
#define LAN_H

#include "ScheduledObject.h"
#include "Tcl.h"
#include "IpPacket.h"
#include "LanFrame.h"
#include "Event.h"

// An array of ArpEntry structures maps from IP addresses to objects.
typedef struct ArpEntry {
    struct InAddr ip;		// IP address
    String objectName;		// name of object
    ScheduledObject *object;	// object
};

class Lan : public ScheduledObject {

    friend int
    LanCmd(ClientData clientData, Tcl_Interp *interp,
	   int argc, char *argv[]);
    friend int
    LanObjectCmd(ClientData clientData, Tcl_Interp *interp,
		 int argc, char *argv[]);

    public:

    const int maxArp;	// maximum ARP table size
    const double latency;	// fixed latency
    const double bandwidth;	// bandwidth of this network
    ArpEntry *arpMap;		// ARP table

    double interval;		// statistics printing interval
    int intervalBytes;		// number of bytes per interval
    int intervalFrames;		// number of frames per interval
    int totalBytes;		// number of bytes
    int totalFrames;		// number of frames

    Lan(const String &name, Simulation *si, int ma, double l, double bw);
    ~Lan();

    virtual String Output(void);
    virtual void Handler(Event *e);

    protected:				  

    double expectedTime;

};

extern int LanCmd(ClientData clientData, Tcl_Interp *interp,
		  int argc, char *argv[]);
extern int LanObjectCmd(ClientData clientData, Tcl_Interp *interp,
			int argc, char *argv[]);
extern void LanDeleteProc(ClientData clientData);

#endif // LAN_H
