// -*- c++ -*-
//
// $Id: LanDeviceDriver.h,v 1.6 1996-02-02 15:56:22-08 bmah Exp $
//
// LanDeviceDriver.h
// Bruce A. Mah (bmah@CS.Berkeley.EDU)
//
// Copyright (c) 1995, 1996 The Regents of the University of California
// All Rights Reserved
// Created by Bruce A. Mah
// Department of Electrical Engineering and Computer Sciences,
//   Computer Science Division
//

#ifndef LANDEVICEDRIVER_H
#define LANDEVICEDRIVER_H

#include "DeviceDriver.h"
#include "Lan.h"
#include "ScheduledObject.h"
#include "String.h"
#include "IpPacket.h"
#include "LanFrame.h"

class LanDeviceDriver : public DeviceDriver {

    friend int
    LanDeviceDriverCmd(ClientData clientData, Tcl_Interp *interp,
	   int argc, char *argv[]);
    friend int
    LanDeviceDriverObjectCmd(ClientData clientData, Tcl_Interp *interp,
		 int argc, char *argv[]);

    public:

    LanDeviceDriver(const String &name, Simulation *si, int mq, double l) :
    DeviceDriver(name, si), maxQueued(mq), latency(l) {
	queued = 0;
	queue = new (LanFrame *)[maxQueued];
	head = 0;
	tail = 0;

    }

    ~LanDeviceDriver() {
	delete [] queue;
    }

    virtual String Output(void);
    virtual void Handler(Event *e);

    const int maxQueued;	// max number of packets we can queue
    const double latency;	// driver latency

    protected:				  

    int queued;			// number of packets queued by driver
    LanFrame **queue;		// circular queue
    int head, tail;		// head and tail pointers into circular queue

};

extern int LanDeviceDriverCmd(ClientData clientData, Tcl_Interp *interp,
		  int argc, char *argv[]);
extern int LanDeviceDriverObjectCmd(ClientData clientData, Tcl_Interp *interp,
			int argc, char *argv[]);
extern void LanDeviceDriverDeleteProc(ClientData clientData);

#endif // LANDEVICEDRIVER_H
