static char rcsid[] = "$Id: LoopbackDeviceDriver.cc,v 1.5 1996-02-02 15:56:24-08 bmah Exp $";
//
// $Id: LoopbackDeviceDriver.cc,v 1.5 1996-02-02 15:56:24-08 bmah Exp $
//
// LoopbackDeviceDriver.cc
// Bruce A. Mah (bmah@CS.Berkeley.EDU)
//
// Copyright (c) 1995, 1996 The Regents of the University of California
// All Rights Reserved
// Created by Bruce A. Mah
// Department of Electrical Engineering and Computer Sciences,
//   Computer Science Division
//
// Loopback device driver class.
//

#include "LoopbackDeviceDriver.h"
#include "IpPacket.h"
#include "Event.h"
#include "inline.h"

//
// LoopbackDeviceDriver::Handler
//
// (Event handler)
//
void LoopbackDeviceDriver::Handler(Event *e)
{

    double time = e->ev_time;
    EventType type = e->ev_type;
    void *data = e->ev_data;

    switch (type) {

	case (EventPacketSend):
	{

	    IpPacket *ipPacket = (IpPacket *) data;

	    // If the packet showed up earlier than we could handle
	    // it (expected arrival time), then delay its transmission
	    // until some (presumed prior) packet completes transmission.
	    if (expectedTime > time) {
		time = expectedTime;
	    }

	    // Return the packet to the IP module, unchanged.
	    Lookup(up, upName);
	    Post(time + latency, up, EventPacketArrival, data);

	    expectedTime = time + latency;

	    // update stats
	    if_opackets++;
	    if_obytes += ipPacket->ip_len;

	}
	break;

	default:
	{
	    IF_DEBUG(0, {
		DEBUG_PREFIX("LoopbackDeviceDriver");
		cout << "bad EventType " << type << "\n";
	    });
	}
	break;

    }

}

//
// LoopbackDeviceDriver::Output
//
// Input:  None
//
// Output:  String with an ASCII representation of this object.
//
String LoopbackDeviceDriver::Output()
{
    return name + " [LoopbackDeviceDriver]";
}

//
// LoopbackDeviceDriverCmd
//
// (Tcl command procedure)
//
int LoopbackDeviceDriverCmd(ClientData clientData, Tcl_Interp *interp,
			    int argc, char *argv[])
{

    if (argc != 3) {
	Tcl_SetResult(interp, "Usage:  LoopbackDeviceDriver name latency",
		      TCL_STATIC);
	return(TCL_ERROR);
    }
    else {
	char *nm;		// new object name
	LoopbackDeviceDriver *ldd; // new device driver
	double latency;		// latency of driver
	int rc = TCL_OK;	// Tcl return code
	extern Simulation *sim;	// global simulation object

	nm = argv[1];
	rc = Tcl_GetDouble(interp, argv[2], &latency);

	if (rc == TCL_OK) {

	    ldd = new LoopbackDeviceDriver(nm, sim, latency);
	    Tcl_CreateCommand(interp, nm,
			      LoopbackDeviceDriverObjectCmd,
			      (ClientData) ldd,
			      LoopbackDeviceDriverDeleteProc);

	}
	return (rc);

    }

}

//
// LoopbackDeviceDriverObjectCmd
//
// (Tcl object command procedure)
//
int LoopbackDeviceDriverObjectCmd(ClientData clientData, Tcl_Interp *interp,
				  int argc, char *argv[])
{
	return(DeviceDriverObjectCmd(clientData, interp, argc, argv));
}

//
// LoopbackDeviceDriverDeleteProc
//
// (Tcl delete procedure)
//
void LoopbackDeviceDriverDeleteProc(ClientData clientData)
{
    delete (LoopbackDeviceDriver *) clientData;
}
