static char rcsid[] = "$Id: NullDeviceDriver.cc,v 1.2 1996-03-25 18:38:26-08 bmah Exp $";
//
// $Id: NullDeviceDriver.cc,v 1.2 1996-03-25 18:38:26-08 bmah Exp $
//   based on:
// $ID: LanDeviceDriver.cc,v 1.10 1996-02-16 07:36:59-08 bmah Exp $
//
// NullDeviceDriver.cc
// Bruce A. Mah (bmah@CS.Berkeley.EDU)
//
// Copyright (c) 1995, 1996 The Regents of the University of California
// All Rights Reserved
// Created by Bruce A. Mah
// Department of Electrical Engineering and Computer Sciences,
//   Computer Science Division
//
// Null device driver class.  This device driver throws away all packets
// it's given.  Mostly it's used as a primitive form of filtering for
// stray packets.
//

#include "NullDeviceDriver.h"
#include "IpPacket.h"
#include "Event.h"
#include "inline.h"

//
// NullDeviceDriver::Handler
//
// (Event handler)
//
void NullDeviceDriver::Handler(Event *e)
{

    double time = e->ev_time;
    EventType type = e->ev_type;
    void *data = e->ev_data;

    switch (type) {

	//
	// EventPacketSend:  Packet outbound
	//
	case (EventPacketSend):
	{

	    IpPacket *packet = (IpPacket *) data;

	    IF_DEBUG(1, {
		DEBUG_PREFIX("NullDeviceDriver");
		cout << "discard packet " << packet << "\n";
	    });

	    delete packet;
	}
	break;

	default:
	    IF_DEBUG(0, {
		DEBUG_PREFIX("NullDeviceDriver");
		cout << "bad EventType " << type << "\n";
	    });
	break;

    }

}

//
// NullDeviceDriver::Output
//
// Input:  None
//
// Output:  String with an ASCII representation of this object.
//
String NullDeviceDriver::Output()
{
    return name + " [NullDeviceDriver]";
}

//
// NullDeviceDriverCmd
//
// (Tcl command procedure)
//
int NullDeviceDriverCmd(ClientData clientData, Tcl_Interp *interp,
		       int argc, char *argv[]) {

    if (argc != 2) {
	Tcl_SetResult(interp,
		      "usage:  NullDeviceDriver name",
		      TCL_STATIC);
	return(TCL_ERROR);
    }
    else {
	int rc;			// Tcl return code
	char *nm;		// name of new NullDeviceDriver
	int mq;			// maximum queue length
	double l;		// driver latency
	NullDeviceDriver *ndd;	// new object
	extern Simulation *sim;	// simulation object

	// get string argument
	nm = argv[1];

	// Make new object and make a Tcl command corresponding
	// to the object.
	ndd = new NullDeviceDriver(nm, sim);
	Tcl_CreateCommand(interp, nm,
			  NullDeviceDriverObjectCmd,
			  (ClientData) ndd,
			  NullDeviceDriverDeleteProc);
	return TCL_OK;
    }
}

//
// NullDeviceDriverObjectCmd
//
// (Tcl object command procedure)
//
int NullDeviceDriverObjectCmd(ClientData clientData, Tcl_Interp *interp,
			     int argc, char *argv[]) {
    return DeviceDriverObjectCmd(clientData, interp, argc, argv);
}

//
// NullDeviceDriverDeleteProc
//
// (Tcl delete procedure)
//
void NullDeviceDriverDeleteProc(ClientData clientData) {
    delete (NullDeviceDriver *) clientData;
}
