// -*- c++ -*-
//
// $Id: Pdu.h,v 1.3 1996-02-09 10:03:37-08 bmah Exp $
//
// Pdu.h
// Bruce A. Mah (bmah@CS.Berkeley.EDU)
//
// Copyright (c) 1996 The Regents of the University of California
// All Rights Reserved
// Created by Bruce A. Mah
// Department of Electrical Engineering and Computer Sciences,
//   Computer Science Division
//
// Base class for all packet-like objects.
//


#ifndef PDU_H
#define PDU_H

#include <iostream.h>

class Pdu {

    public:

    // Default constructor.  Base class must set refcount = 0.
    // Derived classes can do basically anything.
    Pdu() {
	refcount = 0;
    };

    // Copy constructor.  Base class must set refcount = 0.
    // Derived classes must be sure to copy all enclosed payloads
    // (not just the pointers to same).
    Pdu(const Pdu &p) {
	refcount = 0;
    }

    // Some derived classes might have an encapsulating constructor,
    // which take some other derived Pdu as an argument.  The idea
    // is that a lower layer packet is being constructed to hold a
    // higher layer packet.  In this case, the higher layer packet
    // should have its refcount incremented.

    // Some derived classes implement a Detatch method, which
    // is roughly the inverse of the encapsulating constructor.
    // If there is an inner (higher layer) payload, it should have its
    // refcout decremented and the outer payload should be set to NULL.
    // The outer packet can then be destroyed.

    // Destructor.  Should be virtual.  Packets which contain other
    // packets (i.e. an AalFrame holding an IpPacket should check their
    // payload fields and decrement the refcounts of the payload fields
    // if appropriate.
    virtual ~Pdu();
    
    int refcount;		// Reference count for this PDU.
				// Principally used in cases of fragmentation
				// to figure out when a higher-layer PDU
				// should be deleted.

    // Print the representation of the packet on the output stream.
    // Very similar (and called by) operator<<().
    virtual ostream& Print(ostream &s);

    // Return a copy of the current object.  Used primarily by
    // creating copy constructors.
    virtual Pdu *Copy();

    protected:				  

};

extern ostream& operator<<(ostream &s, Pdu &pdu);
extern ostream& operator<<(ostream &s, Pdu *pdu);

#endif // PDU_H
