static char rcsid[] = "$Id: PeriodicProcess.cc,v 1.1 1996-01-27 16:44:04-08 bmah Exp $";
//
// $Id: PeriodicProcess.cc,v 1.1 1996-01-27 16:44:04-08 bmah Exp $
//
// PeriodicProcess.cc
// Bruce A. Mah (bmah@CS.Berkeley.EDU)
//
// Copyright (c) 1996 The Regents of the University of California
// All Rights Reserved
// Created by Bruce A. Mah
// Department of Electrical Engineering and Computer Sciences,
//   Computer Science Division
//
// Arrival process with a constant interarrival time.  Runs a Tcl
// script periodically.
//

#include "PeriodicProcess.h"
#include "Event.h"
#include "inline.h"

//
// PeriodicProcess::PeriodicProcess
//
// (object constructor)
//
PeriodicProcess::PeriodicProcess(const String &name, Simulation *si,
			       double start, double p, String &s) :
ArrivalProcess(name, si, start, s), period(p)
{
}

//
// PeriodicProcess::NextInterarrival
//
// Input:  None
//
// Output:  Nextinterarrival time (in return value).
//
// Return a constant interarrival time.
//
double
PeriodicProcess::NextInterarrival()
{
    return period;
}

//
// PeriodicProcess::Output
//
// Input:  None
//
// Output:  String with an ASCII representation of this object.
//
String
PeriodicProcess::Output()
{
    char chars[80];
    sprintf(chars, " period %f next %f", period, next);

    return name + " [PeriodicProcess] " + chars;
}

//
// PeriodicProcessCmd
//
// (Tcl command procedure)
//
int
PeriodicProcessCmd(ClientData clientData, Tcl_Interp *interp,
	       int argc, char *argv[])
{

    if (argc != 5) {
	Tcl_SetResult(interp, "usage:  PeriodicProcess name start lambda script", TCL_STATIC);
	return(TCL_ERROR);
    }
    else {
	int rc = TCL_OK;	// Tcl return code
	PeriodicProcess *pp;	// new object
	char *nm;		// name of new client object
	double startTime;	// starting time to run
	double p;		// interarrival time (constant)
	String s;		// script to run
	extern Simulation *sim;	// simulation object

	nm = argv[1];
	rc = Tcl_GetDouble(interp, argv[2], &startTime);
	if (rc == TCL_OK) {

	    rc = Tcl_GetDouble(interp, argv[3], &p);
	    if (rc == TCL_OK) {

		s = argv[4];

		pp = new PeriodicProcess(nm, sim, startTime, p, s);
		Tcl_CreateCommand(interp, nm,
				  PeriodicProcessObjectCmd,
				  (ClientData) pp,
				  PeriodicProcessDeleteProc);
	    }
	}
	return rc;
    }
}

//
// PeriodicProcessObjectCmd
//
// (Tcl object command procedure)
//
int
PeriodicProcessObjectCmd(ClientData clientData, Tcl_Interp *interp,
		     int argc, char *argv[])
{
    return ArrivalProcessObjectCmd(clientData, interp, argc, argv);
}

//
// PeriodicProcessDeleteProc
//
// (Tcl delete procedure)
//
void
PeriodicProcessDeleteProc(ClientData clientData)
{
    delete (PeriodicProcess *) clientData;
}
