static char rcsid[] = "$Id: PoissonProcess.cc,v 1.1 1996-01-27 16:44:05-08 bmah Exp $";
//
// $Id: PoissonProcess.cc,v 1.1 1996-01-27 16:44:05-08 bmah Exp $
//
// PoissonProcess.cc
// Bruce A. Mah (bmah@CS.Berkeley.EDU)
//
// Copyright (c) 1996 The Regents of the University of California
// All Rights Reserved
// Created by Bruce A. Mah
// Department of Electrical Engineering and Computer Sciences,
//   Computer Science Division
//
// Poisson arrival process that runs a Tcl script with every event.
//

#include "PoissonProcess.h"
#include "Event.h"
#include "inline.h"

//
// Global declarations
//
extern RNG *rng; // random number generator

//
// PoissonProcess::PoissonProcess
//
// (object constructor)
//
PoissonProcess::PoissonProcess(const String &name, Simulation *si,
			       double start, double l, String &s) :
ArrivalProcess(name, si, start, s), lambda(l)
{

    // Initialize the negative exponential distribution
    distrib = new NegativeExpntl(lambda, rng);

}

//
// PoissonProcess::NextInterarrival
//
// Input:  None
//
// Output:  Nextinterarrival time (in return value).
//
// Generate an exponentially-distributed interarrival time for this
// Poisson process.
//
double
PoissonProcess::NextInterarrival()
{
    return((*distrib)());
}

//
// PoissonProcess::Output
//
// Input:  None
//
// Output:  String with an ASCII representation of this object.
//
String
PoissonProcess::Output()
{
    char chars[80];
    sprintf(chars, "period %f next %f", lambda, next);

    return name + " [PoissonProcess] lambda " + chars;
}

//
// PoissonProcessCmd
//
// (Tcl command procedure)
//
int
PoissonProcessCmd(ClientData clientData, Tcl_Interp *interp,
	       int argc, char *argv[])
{

    if (argc != 5) {
	Tcl_SetResult(interp, "usage:  PoissonProcess name start lambda script", TCL_STATIC);
	return(TCL_ERROR);
    }
    else {
	int rc = TCL_OK;	// Tcl return code
	PoissonProcess *pp;	// new object
	char *nm;		// name of new client object
	double startTime;	// starting time to run
	double l;		// mean interarrival
	String s;		// script to run
	extern Simulation *sim;	// simulation object

	nm = argv[1];
	rc = Tcl_GetDouble(interp, argv[2], &startTime);
	if (rc == TCL_OK) {

	    rc = Tcl_GetDouble(interp, argv[3], &l);
	    if (rc == TCL_OK) {

		s = argv[4];

		pp = new PoissonProcess(nm, sim, startTime, l, s);
		Tcl_CreateCommand(interp, nm,
				  PoissonProcessObjectCmd,
				  (ClientData) pp,
				  PoissonProcessDeleteProc);
	    }
	}
	return rc;
    }
}

//
// PoissonProcessObjectCmd
//
// (Tcl object command procedure)
//
int
PoissonProcessObjectCmd(ClientData clientData, Tcl_Interp *interp,
		     int argc, char *argv[])
{
    return ArrivalProcessObjectCmd(clientData, interp, argc, argv);
}

//
// PoissonProcessDeleteProc
//
// (Tcl delete procedure)
//
void
PoissonProcessDeleteProc(ClientData clientData)
{
    delete (PoissonProcess *) clientData;
}
