static char rcsid[] = "$Id: SampleTcpClient.cc,v 1.9 1996-01-27 23:14:14-08 bmah Exp $";
//
// $Id: SampleTcpClient.cc,v 1.9 1996-01-27 23:14:14-08 bmah Exp $
//
// SampleTcpClient.cc
// Bruce A. Mah (bmah@CS.Berkeley.EDU)
//
// Copyright (c) 1995, 1996 The Regents of the University of California
// All Rights Reserved
// Created by Bruce A. Mah
// Department of Electrical Engineering and Computer Sciences,
//   Computer Science Division
//
// Sample TCP client application, used to test TCP.
//

#include <String.h>
#include "SampleTcpClient.h"
#include "Event.h"
#include "inline.h"

//
// SampleTcpClient::Handler
//
// (Event handler)
//
void
SampleTcpClient::Handler(Event *e)
{

    double time = e->ev_time;
    EventType type = e->ev_type;
    void *data = e->ev_data;

    if ((state == AppTcpClientStateEstablishing) &&
	(type == EventTcpConnect)) {
	Post(time + gap, this, EventTimer, NULL);
	state = AppTcpClientStateEstablished;

	dataStartTime = time;

	*(sim->logStream) << name << " [SampleTcpClient] connect-took " <<
	    (time - startTime) << "\n";

    }

    else if ((state == AppTcpClientStateEstablished) &&
	(type == EventTimer)) {

	if (count--) {
	    tcpModule->UserSend(time, tp, length);
	    Post(time + gap, this, EventTimer, NULL);
	}
	else {
	    tcpModule->UserClose(time, tp);
	    totalBytesSent = tp->snd_bytes;
	    totalBytesReceived = tp->rcv_bytes;
	    endTime = time;
	    state = AppTcpClientStateDown;
	    delete this;
	}
    }

    else {
	AppTcpClient::Handler(e);
    }

}

//
// SampleTcpClient::Output
//
// Input:  None
//
// Output:  String with an ASCII representation of this object.
//
String
SampleTcpClient::Output()
{
    return name + " [SampleTcpClient]";
}

//
// SampleTcpClientCmd
//
// (Tcl command procedure)
//
int
SampleTcpClientCmd(ClientData clientData, Tcl_Interp *interp,
		       int argc, char *argv[]) {

    if (argc != 9) {
	Tcl_SetResult(interp,
		      "Usage:  SampleTcpClient name start tcp server port length count gap",
		      TCL_STATIC);
	return(TCL_ERROR);
    }
    else {
	int rc = TCL_OK;	// Tcl return code
	SampleTcpClient *stc;	// new client object
	char *nm;		// name of new client object
	double startTime;	// starting time to run
	String tcpObjectName;	// name of Tcp object
	InAddr server;		// server address to connect to
	int port;		// port number
	int l;			// send length
	int c;			// send count
	double g;		// gap
	extern Simulation *sim;	// simulation object

	nm = argv[1];
	rc = Tcl_GetDouble(interp, argv[2], &startTime);
	if (rc == TCL_OK) {

	    tcpObjectName = argv[3];
	    server.s_addr = InetAddr(argv[4]);
	    rc = Tcl_GetInt(interp, argv[5], &port);

	    if (rc == TCL_OK) {

		rc = Tcl_GetInt(interp, argv[6], &l);
		
		if (rc == TCL_OK) {
		    
		    rc = Tcl_GetInt(interp, argv[7], &c);
		    
		    if (rc == TCL_OK) {

			rc = Tcl_GetDouble(interp, argv[8], &g);

			if (rc == TCL_OK) {
			    
			    // Get pointer to TCP object (implication:  Tcp
			    // object *must* be created before any applications
			    // are created.
			    stc = new SampleTcpClient(nm, sim, startTime,
						      tcpObjectName,
						      server, port, l, c, g);
			    Tcl_CreateCommand(interp, nm,
					      SampleTcpClientObjectCmd,
					      (ClientData) stc,
					      SampleTcpClientDeleteProc);
			}
		    }
		}
	    }

	}

	return rc;
    }
}

//
// SampleTcpClientObjectCmd
//
// (Tcl object command procedure)
//
int
SampleTcpClientObjectCmd(ClientData clientData, Tcl_Interp *interp,
			     int argc, char *argv[]) {
    return ScheduledObjectCmd(clientData, interp, argc, argv);
}

//
// SampleTcpClientDeleteProc
//
// (Tcl delete procedure)
//
void
SampleTcpClientDeleteProc(ClientData clientData) {
    delete (SampleTcpClient *) clientData;
}


