static char rcsid[] = "$Id: SampleTcpServer.cc,v 1.6 1996-01-10 17:05:58-08 bmah Exp $";
//
// $Id: SampleTcpServer.cc,v 1.6 1996-01-10 17:05:58-08 bmah Exp $
//
// SampleTcpServer.cc
// Bruce A. Mah (bmah@CS.Berkeley.EDU)
//
// Copyright (c) 1995, 1996 The Regents of the University of California
// All Rights Reserved
// Created by Bruce A. Mah
// Department of Electrical Engineering and Computer Sciences,
//   Computer Science Division
//
// Sample TCP client application, used to test TCP.
//

#include "SampleTcpServer.h"
#include "Event.h"
#include "inline.h"

//
// SampleTcpServer::Handler
//
// (Event handler)
//
void
SampleTcpServer::Handler(Event *e)
{
    double time = e->ev_time;
    EventType type = e->ev_type;
    void *data = e->ev_data;

    if (!initFlag) {
	AppTcpServer::Handler(e);
    }
    else {
	switch (type) {
	    case (EventTcpConnect):
	    {
		SampleTcpServerConnection *newConnection =
		  new SampleTcpServerConnection;
		newConnection->tcp = (TcpPcb *) data;
		newConnection->startTime = time;
		newConnection->totalBytesSent = 0;
		newConnection->totalBytesReceived = 0;
		newConnection->next = active;
		active = newConnection;
		AppTcpServer::Handler(e);
	    }
	    break;

	    case (EventTcpArrival):
	    {
		SampleTcpServerConnection *current;
		SampleTcpServerConnection *previous = NULL;
		SampleTcpServerConnection *die = NULL;
		TcpPcb *tp;

		TcpCall *tc = (TcpCall *) data;

		// Locate connection
		for (current = active;
		     current != NULL;
		     current = current->next) {

		    if (current->tcp == tc->pcb) {
			die = current;
			break;
		    }
		}

		if (die) {
		    die->totalBytesReceived += tc->th_ulen;

		    IF_DEBUG(2, {
			DEBUG_PREFIX("SampleTcpServer");
			*(sim->debugStream) << "received " << tc->th_ulen << " bytes total " << die->totalBytesReceived << "\n";
		    });

		}

		AppTcpServer::Handler(e);
	    }
	    break;

	    case (EventTcpClosed):
	    {
		SampleTcpServerConnection *current;
		SampleTcpServerConnection *previous = NULL;
		SampleTcpServerConnection *die = NULL;
		TcpPcb *tp = (TcpPcb *) data;;

		// Delete this connection from the connection list
		for (current = active;
		     current != NULL;
		     previous = current, current = current->next) {

		    if (current->tcp == tp) {
			if (previous) {
			    previous->next = current->next;
			}
			else {
			    active = current->next;
			}
			die = current;
			break;
		    }
		}

		tcpModule->UserClose(time, tp);

		//
		if (die) {
		    *(sim->logStream) << name <<
		      " [SampleTcpServer] end send " <<
		      die->totalBytesSent << " received " <<
			die->totalBytesReceived <<
			" time " << (time - die->startTime) << "\n";

		    delete die;
		}
	    }

	    default:
	    AppTcpServer::Handler(e);
	    break;
	}
    }
}

//
// SampleTcpServer::Output
//
// Input:  None
//
// Output:  String with an ASCII representation of this object.
//
String
SampleTcpServer::Output()
{
    return name + " [SampleTcpServer]";
}

//
// SampleTcpServerCmd
//
// (Tcl command procedure)
//
int SampleTcpServerCmd(ClientData clientData, Tcl_Interp *interp,
		       int argc, char *argv[]) {

    if (argc != 5) {
	Tcl_SetResult(interp, "Usage:  SampleTcpServer name start tcp port",
		      TCL_STATIC);
	return(TCL_ERROR);
    }
    else {
	int rc = TCL_OK;	// Tcl return code
	SampleTcpServer *sts;	// new server object
	char *nm;		// name of new server object
	double startTime;	// starting time to run
	String tcpObjectName;	// name of Tcp object
	int port;		// port number
	extern Simulation *sim;	// simulation object

	nm = argv[1];
	rc = Tcl_GetDouble(interp, argv[2], &startTime);
	if (rc == TCL_OK) {

	    tcpObjectName = argv[3];
	    rc = Tcl_GetInt(interp, argv[4], &port);

	    if (rc == TCL_OK) {

		// Get pointer to TCP object (implication:  Tcp
		// object *must* be created before any applications
		// are created.
		sts = new SampleTcpServer(nm, sim, startTime, tcpObjectName,
					  port);
		Tcl_CreateCommand(interp, nm,
				  SampleTcpServerObjectCmd,
				  (ClientData) sts,
				  SampleTcpServerDeleteProc);
	    }

	}

	return rc;
    }
}

//
// SampleTcpServerObjectCmd
//
// (Tcl object command procedure)
//
int SampleTcpServerObjectCmd(ClientData clientData, Tcl_Interp *interp,
			     int argc, char *argv[]) {
    return ScheduledObjectCmd(clientData, interp, argc, argv);
}

//
// SampleTcpServerDeleteProc
//
// (Tcl delete procedure)
//
void SampleTcpServerDeleteProc(ClientData clientData) {
    delete (SampleTcpServer *) clientData;
}


