// -*- c++ -*-
// $Id: ScheduledObject.h,v 1.20 1996-01-12 14:21:59-08 bmah Exp $
//
// ScheduledObject.h
//
// Copyright (c) 1995, 1996 The Regents of the University of California
// All Rights Reserved
// Created by Bruce A. Mah
// Department of Electrical Engineering and Computer Sciences,
//   Computer Science Division
//

#ifndef SCHEDULEDOBJECT_H
#define SCHEDULEDOBJECT_H

// Macros to make debugging statements a little easier.  IF_DEBUG
// and DEBUG_PREFIX2
// should only be invoked from within a ScheduledObject (or subclass)
// method.  DEBUG_PREFIX should only be invoked from within a ScheduledObject
// (or subclass) Handler method (or any place where a double time is in
// scope).
#define IF_DEBUG(debug_min, action) if (debug >= debug_min) \
        {action; if (flushFlag) {*(sim->debugStream) << flush;}};
#define DEBUG_PREFIX(type) *(sim->debugStream) << time << ": " << name << " [" << type << "] ";
#define DEBUG_PREFIX2(type) *(sim->debugStream) << "---.------: " << name << " [" << type << "] ";

//
// Pre-declare some data types so that our object registry works
// correctly.
//
class ScheduledObject;
typedef ScheduledObject *ScheduledObjectPtr;
class StringScheduledObjectPtrCHMap;
class StringScheduledObjectPtrMap;
class Simulation;

#include <iostream.h>
#include <String.h>
#include "util.h"
#include "Event.h"
#include "Tcl.h"

class Aal;
class CellGoBackN;
class DeviceDriver;
class Ip;
class SwitchModule;
class Tcp;
class Udp;

class ScheduledObject {

    friend void
    Initialize(void);

    friend int
    ScheduledObjectCmd(ClientData clientData, Tcl_Interp *interp,
		       int argc, char *argv[]);

    friend int
    SimulationCmd(ClientData clientData, Tcl_Interp *interp,
		  int argc, char *argv[]);

    friend ostream&
    operator<<(ostream &s, ScheduledObject &o);

    //
    // Public members
    //
    public:
    static StringScheduledObjectPtrCHMap *registry; // object registry
    const String name;		// object name
    bool cache;			// cache object references
    static bool flushFlag;	// flush debugging stream after output
    int debug;			// debugging level
    
    ScheduledObject(const String &newName, Simulation *s);
    virtual ~ScheduledObject();
    virtual void Handler(Event *e) = 0;

    // Output returns a string with a representation of the object
    virtual String Output();
    Print() { cout << this; };
    
    // Shutdown method performs end-of-simulation logging
    virtual void Shutdown() { } ;

    //
    // Protected members
    //
    protected:
    Simulation *sim;		// Simulation meta-object, for name
				// registry lookup and scheduling

    void Post(double time, ScheduledObject *object, EventType type,
	      void *data);
    void Lookup(ScheduledObjectPtr &theObject, const String &theName,
		bool fatal = true);

    // Glue to get around dogmatic type enforcing.
    void Lookup(Aal *&theObject, const String &theName,
		bool fatal = true);
    void Lookup(CellGoBackN *&theObject, const String &theName,
		bool fatal = true);
    void Lookup(DeviceDriver *&theObject, const String &theName,
		bool fatal = true);
    void Lookup(Ip *&theObject, const String &theName,
		bool fatal = true);
    void Lookup(SwitchModule *&theObject, const String &theName,
		bool fatal = true);
    void Lookup(Tcp *&theObject, const String &theName,
		bool fatal = true);
    void Lookup(Udp *&theObject, const String &theName,
		bool fatal = true);

};

#endif // SCHEDULEDOBJECT_H
