// -*- c++ -*-
//
// $Id: Sig.h,v 1.14 1996-05-29 14:34:55-07 bmah Exp $
//
// Sig.h
// Bruce A. Mah (bmah@CS.Berkeley.EDU)
//
// Copyright (c) 1995, 1996 The Regents of the University of California
// All Rights Reserved
// Created by Bruce A. Mah
// Department of Electrical Engineering and Computer Sciences,
//   Computer Science Division
//

#ifndef SIG_H
#define SIG_H

#include "SigMessage.h"
#include "ScheduledObject.h"
#include "CellQueue.h"
#include "AtmCell.h"
#include "CellGoBackN.h"

class SwitchModule;

enum SigMode {
    SigModeWorkConserving,
    SigModeNonWorkConserving,
    SigModeNoRateControl,
};

// Translation entry data structure
typedef struct {
    int port;			// output port
    int vcid;			// output vcid
} SigTranslateEntry;

// VCI state structure
enum SigState {
    SigStateUnused,		// channel not in use
    SigStateEstablishing,	// channel being established
    SigStateEstablished,	// channel established, ready for data
    SigStateDeleting,		// channel being deleted
    SigStateDeleted,		// channel deleted (zombie state)
    SigStateReserved		// channel reserved
};

typedef struct {
    SigState state;		// channel state
} SigStateEntry;

class Sig : public ScheduledObject {

    friend int
    SigCmd(ClientData clientData, Tcl_Interp *interp,
	   int argc, char *argv[]);

    friend int
    SigObjectCmd(ClientData clientData, Tcl_Interp *interp,
		 int argc, char *argv[]);
    
    public:
    const int numPorts;		// number of translate ports
				// (in general, number of switch ports
				// plus one for signalling)
    
    const int maxVcids;		// maximum number of Vcids
				// (VCIDs go from 0 to numVcids-1)

    const int maxRoutes;	// maximum number of routes
				// (specifies outgoing link to get to some
				// some ATM address, where the ATM
		      		// address is specified as an integer)

    Sig(const String &name, Simulation *si, int p, int mv, int mr);
    virtual ~Sig();
			       
    virtual String Output(void);
    virtual void Handler(Event *e);

    double interval;		// statistics printing interval

    protected:
    SigMode mode;		// admission control test mode

    String *outputPortName;	// names of output port queues
    ScheduledObject **outputPort; // pointers to output port queues

    SigTranslateEntry **forward; // forward translation array
    SigTranslateEntry **reverse; // reverse translation array
    SigStateEntry **state;	 // VC state array, canonically indexed
				 // by *upstream* port/VCID

    String theGoBackNName;	// name of associated gobackn reliability
				// module
    CellGoBackN *theGoBackN;	// pointer to associated reliability module

    String theSwitchName;	// name of associated switch module
    SwitchModule *theSwitch;	// pointer to associated switch module

    int *routes;		// array of link numbers for routing table

    virtual SigReturnCode AdmissionControl(double time,
					   int link, int vcid,
					   SigMessage *mesg, void *traffic);
    virtual void CommitResources(double time,
				 int link, int vcid, void *traffic);
    virtual void ReleaseResources(double time,
				  int link, int vcid);

    int intervalCalls;		// number of call requests per interval

};

extern int SigCmd(ClientData clientData, Tcl_Interp *interp,
			   int argc, char *argv[]);
extern int SigObjectCmd(ClientData clientData, Tcl_Interp *interp,
				 int argc, char *argv[]);
void SigDeleteProc(ClientData clientData);

extern ostream& operator<<(ostream& s, SigState ss);
extern ostream& operator<<(ostream& s, SigMode mode);

#endif // SWITCHMODULE_H
