// -*- c++ -*-
//
// $Id: SigHost.h,v 1.19 1996-05-29 14:34:58-07 bmah Exp $
//
// SigHost.h
// Bruce A. Mah (bmah@CS.Berkeley.EDU)
//
// Copyright (c) 1995, 1996 The Regents of the University of California
// All Rights Reserved
// Created by Bruce A. Mah
// Department of Electrical Engineering and Computer Sciences,
//   Computer Science Division
//
// Host signalling entity.  Note that compared to the switch signalling
// object (Sig), this class is considerably simplified.
//

#ifndef SIGHOST_H
#define SIGHOST_H

#include <iostream.h>
#include "SigMessage.h"
#include "apiproto.h"
#include "ScheduledObject.h"
#include "CellQueue.h"
#include "AtmCell.h"
#include "Aal.h"

const int sighostPorts = 3;	// number of "ports" to do signalling
const int sighostPortSigHost = 0;
const int sighostPortHost = 1;
const int sighostPortNet = 2;

class SwitchModule;

enum SigHostMode {
    SigHostModeWorkConserving,
    SigHostModeNonWorkConserving,
    SigHostModeNoRateControl,
};

// VCI state structure
enum SigHostState {
    SigHostStateUnused,		// channel not in use
    SigHostStateEstablishing,	// channel being established
    SigHostStateEstablished,	// channel established, ready for data
    SigHostStateDeleting,	// channel being deleted
    SigHostStateDeleted,	// channel deleted (zombie state)
    SigHostStateReserved	// channel reserved
};

typedef struct {
    SigHostState state;		// channel state
} SigHostStateEntry;

class SigHost : public ScheduledObject {

    friend int
    SigHostCmd(ClientData clientData, Tcl_Interp *interp,
	       int argc, char *argv[]);
    friend int
    SigHostObjectCmd(ClientData clientData, Tcl_Interp *interp,
		     int argc, char *argv[]);
    
    public:
    const int maxVcids;		// maximum number of Vcids
				// (VCIDs go from 0 to numVcids-1)

    const int maxDownCalls;	// maximum number of concurrent
				// signalling requests

    const SigAddress address;	// endhost address (like an NSAP)

    SigHost(const String &name, Simulation *si, int mv, int down, int addr);
    virtual ~SigHost();
			       
    virtual String Output(void);
    virtual void Handler(Event *e);

    double interval;		// statistics printing interval

    protected:
    SigHostMode mode;		// admission control test mode

    String networkPortName;	// name of network port
    ScheduledObject *networkPort;
		      
    String aalName;		// name of AAL object for this host
    Aal *aal;

    SigHostStateEntry **state;	 // VC state array

    String theSwitchName;	// name of associated switch module
    SwitchModule *theSwitch;	// pointer to associated switch module

    ApiMessage **downCalls;

    virtual SigReturnCode AdmissionControl(double time,
					   int vcid, void *traffic);
    virtual void CommitResources(double time,
				 int vcid, void *traffic);
    virtual void ReleaseResources(double time,
				  int vcid);

    int intervalCalls;		// number of call requests per interval

};

extern int SigHostCmd(ClientData clientData, Tcl_Interp *interp,
			   int argc, char *argv[]);
extern int SigHostObjectCmd(ClientData clientData, Tcl_Interp *interp,
				 int argc, char *argv[]);
void SigHostDeleteProc(ClientData clientData);

extern ostream& operator<<(ostream& s, SigHostState shs);
extern ostream& operator<<(ostream& s, SigHostMode mode);

#endif // SIGHOST_H
