// -*- c++ -*-
//
// $Id: SigHostRcsp.h,v 1.5 1996-05-29 14:35:01-07 bmah Exp $
//
// SigHostRcsp.h
// Bruce A. Mah (bmah@CS.Berkeley.EDU)
//
// Copyright (c) 1995, 1996 The Regents of the University of California
// All Rights Reserved
// Created by Bruce A. Mah
// Department of Electrical Engineering and Computer Sciences,
//   Computer Science Division
//

#ifndef SIGHOSTRCSP_H
#define SIGHOSTRCSP_H

#include "SigHost.h"
#include "SigRcsp.h"

//
// Buffering constants
//
const int SigHostRcspBuffersExcessDefault = 48;
					 // Excess buffers to use for packet
					 // segmentation (host entity only).
					 // Should be at least one MTU.
					 // Current value provides 2K of
					 // buffers (4 512-byte TCP segments).

const int SigHostRcspBuffersBestEffort = 8; // Number of buffers to reserve
					    // for best-effort connections.
					    // Yet another voodoo constant.

class SigHostRcsp : public SigHost {

    friend int
    SigHostRcspCmd(ClientData clientData, Tcl_Interp *interp,
		   int argc, char *argv[]);

    friend int
    SigHostRcspObjectCmd(ClientData clientData, Tcl_Interp *interp,
			 int argc, char *argv[]);
    
    public:

    SigHostRcsp(const String &name, Simulation *si, int mv, int down,
		int addr);
    virtual ~SigHostRcsp();
			       
    virtual String Output(void);

    protected:

    LinkState linkState;		// link state descriptors
    ChannelState *channelState;		// channel state descriptors
    int buffersExcess;			// per-VC excess buffers
    
    virtual SigReturnCode AdmissionControl(double time,
					   int vcid, void *traffic);
    virtual void CommitResources(double time,
				 int vcid, void *traffic);
    virtual void ReleaseResources(double time,
				  int vcid);

};

extern int SigHostRcspCmd(ClientData clientData, Tcl_Interp *interp,
			  int argc, char *argv[]);
extern int SigHostRcspObjectCmd(ClientData clientData, Tcl_Interp *interp,
				int argc, char *argv[]);
void SigHostRcspDeleteProc(ClientData clientData);

#endif // SIGHOSTRCSP_H
