static char rcsid[] = "$Id: SigMessage.cc,v 1.4 1996-04-03 20:48:17-08 bmah Exp $";
//
// $Id: SigMessage.cc,v 1.4 1996-04-03 20:48:17-08 bmah Exp $
//
// SigMessage.cc
// Bruce A. Mah (bmah@CS.Berkeley.EDU)
//
// Copyright (c) 1996 The Regents of the University of California
// All Rights Reserved
// Created by Bruce A. Mah
// Department of Electrical Engineering and Computer Sciences,
//   Computer Science Division
//
// Utility functions for signalling protocol.
//

#include <iostream.h>
#include <String.h>
#include "SigMessage.h"
#include "apiproto.h"
#include "inline.h"

int SigProtoReservedChannels = 16;

char *SigTypeName[] = {
    "SigTypeEstablish",
    "SigTypeEstablishAck",
    "SigTypeEstablishNak",
    "SigTypeDelete"
};

char *SigReturnCodeName[] = {
    "SigOK",
    "SigNoVcids",
    "SigBadDest",
    "SigBadService",
    "SigBadRoute",
    "SigBadParms",
    "SigFailDelay",
    "SigFailBandwidth",
    "SigFailBuffer",
    
    "ApiNoVcids",
    "ApiNoDownCalls"
};

char *QosTypeName[] = {
    "QosTypeNone",
    "QosTypeRcsp"
};

//
// SigMessage::Copy
//
// Input:  None
//
// Output:  new SigMessage in return value.
//
// Returns a copy of this.
//
Pdu *SigMessage::Copy() {
    SigMessage *sm = new SigMessage;

    sm->type = type;
    sm->vcid = vcid;
    sm->code = code;
    sm->sourceAddress = sourceAddress;
    sm->sourceService = sourceService;
    sm->destinationAddress = destinationAddress;
    sm->destinationService = destinationService;
    sm->hopcount = hopcount;
    sm->trafficLength = trafficLength;
    if (trafficLength > 0) {
	sm->traffic = new char[sm->trafficLength];
	bcopy((char *) traffic,
	      (char *) sm->traffic,
	      sm->trafficLength);
    }
    else {
	sm->traffic = NULL;
    }

    return sm;
}

ostream& SigMessage::Print(ostream &s)
{
    return s << "([SigMessage] type " << type <<
      " vcid " << vcid << " hopcount " << hopcount <<
      " refcount " << refcount << ")";
}

//
// operator<<
//
// Input:  Output stream (s), SigMessage (mesg)
//
// Output:  Output stream
//
// Print a representation of the signalling message on the output stream s.
//
ostream& operator<<(ostream &s, SigMessage *mesg)
{
    mesg->Print(s);
    return s;
}

//
// operator<<
//
// Input:  Output stream (s), SigType (st)
//
// Output:  Output stream
//
// Print the name of the signalling message type st on the output stream s.
//
ostream& operator<<(ostream &s, SigType st)
{
    char *output;
    
    output = SigTypeName[st];

    return s << output;

}

//
// operator<<
//
// Input:  Output stream (s), return code (rc)
//
// Output:  Output stream
//
// Print the name of the return code rc on the output stream s.
//
ostream& operator<<(ostream &s, SigReturnCode rc)
{
    char *output;
    
    output = SigReturnCodeName[rc];

    return s << output;

}

//
// operator<<
//
// Input:  Output stream (s), QOS type (qt)
//
// Output:  Output stream
//
// Print the name of the QOS type qt on the output stream s.
//
ostream& operator<<(ostream &s, QosType qt)
{
    char *output;
    
    output = QosTypeName[qt];

    return s << output;

}
