// -*- c++ -*-
//
// $Id: SigMessage.h,v 1.4 1996-04-03 20:48:18-08 bmah Exp $
//   based on:
// $ID: sigproto.h,v 1.8 1996-01-31 14:07:35-08 bmah Exp $
//
// SigMessage.h
// Bruce A. Mah (bmah@CS.Berkeley.EDU)
//
// Copyright (c) 1996 The Regents of the University of California
// All Rights Reserved
// Created by Bruce A. Mah
// Department of Electrical Engineering and Computer Sciences,
//   Computer Science Division
//
// Definitions for an ATM-like signalling protocol.  This protocol is
// implemented by the Sig and Sighost classes, and is roughly equivalent
// to either the Real-Time Channel Adminstration Protocol (RCAP) or the
// the Generic Universal Line Protocol (GULP).
//

#ifndef SIGMESSAGE_H
#define SIGMESSAGE_H

#include <String.h>
#include "Pdu.h"

//
// Types of signalling messages (based heavily on RCAP)
//
enum SigType {
    SigTypeEstablish,		// request to establish a new connection
				// (establish_request)
    SigTypeEstablishAck,	// acknowledgement (establish_accept)
    SigTypeEstablishNak,	// negative acknowledgement (establish_denied)
    SigTypeDelete,		// teardown request
};

//
// Return codes (for SigMessage->code)
//
enum SigReturnCode {
    SigOK,
    SigNoVcids,
    SigBadDest,
    SigBadService,
    SigBadRoute,
    SigBadParms,
    SigFailDelay,
    SigFailBandwidth,
    SigFailBuffer,
    
    ApiNoVcids,
    ApiNoDownCalls
};

//
// Addressing definitions
//
// Based heavily on XUNET.
//
typedef int SigAddress;

enum SigService {
    SigServiceNone,		// no service
    SigServiceRaw,		// raw ATM service
    SigServiceIP		// IP-over-ATM service
};

//
// Signalling message
//
class SigMessage : public Pdu {

    public:
      
    SigMessage() : hopcount(0) { }

    virtual ~SigMessage() {
	if (traffic) {
	    delete traffic;
	}
    }

    virtual ostream& Print(ostream &s);

    virtual Pdu *Copy();

    SigType type;		// message type (ALL)

    int vcid;			// VCID to operate on (ALL)

    SigReturnCode code;		// return code (SigTypeEstablishNak)

    SigAddress sourceAddress;	// source of establishment (SigTypeEstablish
    SigService sourceService;	// only)

    SigAddress destinationAddress; // destination of establishment
    SigService destinationService; // (SigTypeEstablish only)
    int hopcount;		   // number of hops into the network
				   // (SigTypeEstablish only)

    void *traffic;		// traffic specification (SigTypeEstablish
				// and SigTypeEstablishAck only).  In a
				// real signalling protocol, we'd just
				// put this data in the signalling message,
				// but to support our object paradigm we've
				// chosen to put a pointer to a scheduler-
				// dependent parameter block.
    int trafficLength;		// length of traffic descriptor

};

//
// Parameter blocks for various schedulers follow:  Each parm block
// has a type field, used for error checking.
//
enum QosType {
    QosTypeNone,
    QosTypeRcsp
};

//
// QOS parameters for the RCSP scheduler.
//
// This structure is passed between signalling entities
// to communicate QOS requirements and traffic parameters.
//
typedef struct QosRcsp {
    
    QosType type;		// type used for error checking

    double Xmin;
    double Xave;
    double I;
    double d;			// delay each node
    double D;			// end to end delay (ignored for now)

};

extern int SigProtoReservedChannels;

extern ostream& operator<<(ostream &s, SigMessage &mesg);
extern ostream& operator<<(ostream &s, SigType st);
extern ostream& operator<<(ostream &s, SigReturnCode rc);
extern ostream& operator<<(ostream &s, QosType qt);

#endif // SIGMESSAGE_H
