// -*- c++ -*-
//
// $Id: SigRcsp.h,v 1.8 1996-05-29 14:35:03-07 bmah Exp $
//
// SigRcsp.h
// Bruce A. Mah (bmah@CS.Berkeley.EDU)
//
// Copyright (c) 1996 The Regents of the University of California
// All Rights Reserved
// Created by Bruce A. Mah
// Department of Electrical Engineering and Computer Sciences,
//   Computer Science Division
//

#ifndef SIGRCSP_H
#define SIGRCSP_H

#include "Sig.h"

//
// Bufering constants
//
const int SigRcspBuffersBestEffort = 8; // Number of buffers to reserve
                                        // for best-effort connections.
                                        // Yet another voodoo constant.

const int SigRcspMinBuffers = 2;	// Minimum number of cell buffers
					// to reserve.  Reserving at least
					// two buffers makes some of the
					// early packet discard algorithm
					// in the RCSP queue work a little
					// more smoothly.

const int SigRcspBuffersExcessDefault = 0;
                                        // Excess buffers (per VC).
					// Current value provides 2K of
					// buffers (4 512-byte TCP segments).

//
// Internal data structures to keep signalling state
//
typedef struct LinkState {
    int n;			// number of levels
    double *d;			// array of delay bounds
    double tick;		// tick value
    double l;			// line speed
};

typedef struct ChannelState {
    double Xmin;
    double Xave;
    double I;		       
    int level;			// priority level
    int buffers;		// number of cell buffers
};

class SigRcsp : public Sig {

    friend int
    SigRcspCmd(ClientData clientData, Tcl_Interp *interp,
	   int argc, char *argv[]);

    friend int
    SigRcspObjectCmd(ClientData clientData, Tcl_Interp *interp,
		 int argc, char *argv[]);
    
    public:

    SigRcsp(const String &name, Simulation *si, int p, int mv, int mr);
    virtual ~SigRcsp();
			       
    virtual String Output(void);

    protected:

    LinkState *linkState;		// link state descriptors
    ChannelState **channelState;	// channel state descriptors
    int buffersExcess;			// per-VC excess buffers
    
    virtual SigReturnCode AdmissionControl(double time,
					   int link, int vcid,
					   SigMessage *mesg, void *traffic);
    virtual void CommitResources(double time,
				 int link, int vcid, void *traffic);
    virtual void ReleaseResources(double time,
				  int link, int vcid);

};

extern int SigRcspCmd(ClientData clientData, Tcl_Interp *interp,
		      int argc, char *argv[]);
extern int SigRcspObjectCmd(ClientData clientData, Tcl_Interp *interp,
			    int argc, char *argv[]);
void SigRcspDeleteProc(ClientData clientData);

extern double RcspB(double u, double xmin, double xave, double i);

#endif // SIGRCSP_H
