static char rcsid[] = "$Id: SmtpClient.cc,v 1.2 1996-05-29 14:35:04-07 bmah Exp $";
//
// $Id: SmtpClient.cc,v 1.2 1996-05-29 14:35:04-07 bmah Exp $
//   based on:
// $ID: SampleTcpClient.cc,v 1.9 1996-01-27 23:14:14-08 bmah Exp $
//
// SmtpClient.cc
// Bruce A. Mah (bmah@CS.Berkeley.EDU)
//
// Copyright (c) 1995, 1996 The Regents of the University of California
// All Rights Reserved
// Created by Bruce A. Mah
// Department of Electrical Engineering and Computer Sciences,
//   Computer Science Division
//
// Simple SMTP-like client.  This process connects to a remote TCP
// port and sends a single file from the smtp.itemsize tcplib distribution.
// It's not particularly realistic...in particular, note that the
// the initial SMTP handshaking isn't done at all.
//

#include <String.h>
#include "SmtpClient.h"
#include "EmpiricalDiscrete.h"
#include "Event.h"
#include "inline.h"

//
// Imported probability distributions
//
extern RNG *rng;		// global random number generator

// Mail message length distribution
extern const DoubleDistElement *SmtpItemLengthDistTable;
extern int SmtpItemLengthDistLength;

//
// SmtpClient::Handler
//
// (Event handler)
//
void
SmtpClient::Handler(Event *e)
{

    double time = e->ev_time;
    EventType type = e->ev_type;
    void *data = e->ev_data;

    static EmpiricalDiscrete SmtpItemLengthDist(SmtpItemLengthDistLength,
						SmtpItemLengthDistTable,
						rng);

    if ((state == AppTcpClientStateEstablishing) &&
	(type == EventTcpConnect)) {
	Post(time + gap, this, EventTimer, NULL);
	state = AppTcpClientStateEstablished;

	dataStartTime = time;
	length = (int) SmtpItemLengthDist();

//	*(sim->logStream) << name << " [SmtpClient] connect-took " <<
//	    (time - startTime) << "\n";

	IF_DEBUG(2, {
	    DEBUG_PREFIX("SmtpClient");
	    *(sim->debugStream) << length << " byte message\n";
	});

    }

    else if ((state == AppTcpClientStateEstablished) &&
	(type == EventTimer)) {

	if (count--) {
	    tcpModule->UserSend(time, tp, length);
	    Post(time + gap, this, EventTimer, NULL);
	}
	else {
	    tcpModule->UserClose(time, tp);
	    totalBytesSent = tp->snd_bytes;
	    totalBytesReceived = tp->rcv_bytes;
	    endTime = time;
	    state = AppTcpClientStateDown;
	    delete this;
	}
    }

    else {
	AppTcpClient::Handler(e);
    }

}

//
// SmtpClient::Output
//
// Input:  None
//
// Output:  String with an ASCII representation of this object.
//
String
SmtpClient::Output()
{
    return name + " [SmtpClient]";
}

//
// SmtpClientCmd
//
// (Tcl command procedure)
//
int
SmtpClientCmd(ClientData clientData, Tcl_Interp *interp,
		       int argc, char *argv[]) {

    if (argc != 7) {
	Tcl_SetResult(interp,
		      "Usage:  SmtpClient name start tcp server port gap",
		      TCL_STATIC);
	return(TCL_ERROR);
    }
    else {
	int rc = TCL_OK;	// Tcl return code
	SmtpClient *mc;		// new client object
	char *nm;		// name of new client object
	double startTime;	// starting time to run
	String tcpObjectName;	// name of Tcp object
	InAddr server;		// server address to connect to
	int port;		// port number
	double g;		// gap
	extern Simulation *sim;	// simulation object

	nm = argv[1];
	rc = Tcl_GetDouble(interp, argv[2], &startTime);
	if (rc == TCL_OK) {

	    tcpObjectName = argv[3];
	    server.s_addr = InetAddr(argv[4]);
	    rc = Tcl_GetInt(interp, argv[5], &port);

	    if (rc == TCL_OK) {

		rc = Tcl_GetDouble(interp, argv[6], &g);

		if (rc == TCL_OK) {
			    
		    // Get pointer to TCP object (implication:  Tcp
		    // object *must* be created before any applications
		    // are created.
		    mc = new SmtpClient(nm, sim, startTime,
					tcpObjectName,
					server, port, g);
		    Tcl_CreateCommand(interp, nm,
				      SmtpClientObjectCmd,
				      (ClientData) mc,
				      SmtpClientDeleteProc);
		}
	    }
	}

	return rc;
    }
}

//
// SmtpClientObjectCmd
//
// (Tcl object command procedure)
//
int
SmtpClientObjectCmd(ClientData clientData, Tcl_Interp *interp,
			     int argc, char *argv[]) {
    return ScheduledObjectCmd(clientData, interp, argc, argv);
}

//
// SmtpClientDeleteProc
//
// (Tcl delete procedure)
//
void
SmtpClientDeleteProc(ClientData clientData) {
    delete (SmtpClient *) clientData;
}


