static char rcsid[] = "$Id: SwitchModule.cc,v 1.6 1996-01-10 17:06:08-08 bmah Exp $";
//
// $Id: SwitchModule.cc,v 1.6 1996-01-10 17:06:08-08 bmah Exp $
//
// SwitchModule.cc
// Bruce A. Mah (bmah@CS.Berkeley.EDU)
//
// Copyright (c) 1995, 1996 The Regents of the University of California
// All Rights Reserved
// Created by Bruce A. Mah
// Department of Electrical Engineering and Computer Sciences,
//   Computer Science Division
//
// Switch module for an output-queued ATM switch.  Named after the
// XUNET ATM switch module.
//

#include "SwitchModule.h"
#include "inline.h"

//
// SwitchModule::Handler
//
// (Event handler)
//
void SwitchModule::Handler(Event *e)
{

    double time = e->ev_time;
    EventType type = e->ev_type;
    void *data = e->ev_data;

    AtmCell *cell = (AtmCell *) data;
    
    switch (type) {

	case (EventCellArrival):
	{
	    int outgoingPort;
	    int outgoingVcid;
	    
	    IF_DEBUG(3, {
		cout << time << ": " << name <<
		  " [SwitchModule] got cell on " <<
		    cell->port << "." << cell->vcid << " => " <<
		      translate[cell->port][cell->vcid].port << "." <<
			translate[cell->port][cell->vcid].vcid << "\n";
	    });

	    // Calculate output port and VCI
	    outgoingPort = translate[cell->port][cell->vcid].port;
	    outgoingVcid = translate[cell->port][cell->vcid].vcid;

	    // Modify cell's VCI
	    cell->vcid = outgoingVcid;

	    // Send it on its way...  Note that since we're assuming
	    // output queueing, we don't bother to add any latency to
	    // cells during translation.  Should we change this?
	    Lookup(outputPort[outgoingPort], outputPortName[outgoingPort]);
	    Post(time, outputPort[outgoingPort],
		 EventCellArrival, data);
	}
	break;

	default:
	{
	    IF_DEBUG(0, {
		DEBUG_PREFIX("SwitchModule");
		cout << "bad EventType " << type << "\n";
	    });
	}
	break;

    }

}

//
// SwitchModule::Output
//
// Input:  None
//
// Output:  String with an ASCII representation of this object.
//
String SwitchModule::Output()
{

    String out;
    char buf[30];

    sprintf(buf, "%d", ports);
    out = name + " [SwitchModule] ports " + buf;

    sprintf(buf, "%d", maxVcis);
    out += (String) " maxVcis " + buf;

    return out;
    
}
//
// SwitchModuleCmd
//
// (Tcl command handler procedure)
//
// SwitchModule name numports maxvci
//
int SwitchModuleCmd(ClientData clientData, Tcl_Interp *interp,
		    int argc, char *argv[])
{

    if (argc != 4) {
	Tcl_SetResult(interp, "Usage:  SwitchModule name ports maxvcid",
		      TCL_STATIC);
	return(TCL_ERROR);
    }
    else {
	char *nm;		// name of new SwitchModule
	int np;			// num ports
	int maxvci;		// maximum number of VCIs
	int rc;			// return code from Tcl library
	SwitchModule *sm;	// new switch module object
	extern Simulation *sim;	// global simulation object

	// get name
	nm = argv[1];

	// get numeric arguments
	rc = Tcl_GetInt(interp, argv[2], &np);
	if (rc == TCL_OK) {

	    rc = Tcl_GetInt(interp, argv[3], &maxvci);
	    if (rc == TCL_OK) {

		sm = new SwitchModule(nm, sim, np, maxvci);
		Tcl_CreateCommand(interp, nm,
				  SwitchModuleObjectCmd,
				  (ClientData) sm,
				  SwitchModuleDeleteProc);

	    }

	}

	return rc;
	
    }

}

//
// SwitchModuleObjectCmd
//
// (Tcl object handler procedure)
//
int SwitchModuleObjectCmd(ClientData clientData, Tcl_Interp *interp,
			  int argc, char *argv[])
{
    
    SwitchModule *sm = (SwitchModule *) clientData;

    if (argc == 1) {
	return(ScheduledObjectCmd(clientData, interp, argc, argv));
    }
    else {

	//
	// sm set input-port input-vcid output-port output-vcid
	//
	// Manipulate translation table
	//
	if (!strcmp(argv[1], "set")) {

	    if (argc == 6) {

		int inPort, inVcid, outPort, outVcid; // input parameters
		int rc;				      // Tcl return code
		
		rc = Tcl_GetInt(interp, argv[2], &inPort);
		if (rc == TCL_OK) {
		    
		    if ((inPort < 0) || (inPort >= sm->ports)) {
			Tcl_SetResult(interp, "input port out of bounds",
				      TCL_STATIC);
			return(TCL_ERROR);
		    }
		    
		    rc = Tcl_GetInt(interp, argv[3], &inVcid);
		    if (rc == TCL_OK) {

			if ((inVcid < 0) || (inVcid >= sm->maxVcis)) {
			    Tcl_SetResult(interp, "input VCID out of bounds",
					  TCL_STATIC);
			    return(TCL_ERROR);
			}
		    
			rc = Tcl_GetInt(interp, argv[4], &outPort);
			if (rc == TCL_OK) {

			    if ((outPort < 0) || (outPort >= sm->ports)) {
				Tcl_SetResult(interp,
					      "output port out of bounds",
					      TCL_STATIC);
				return(TCL_ERROR);
			    }
		    
			    rc = Tcl_GetInt(interp, argv[5], &outVcid);
			    if (rc == TCL_OK) {

				if ((outVcid < 0) || (outVcid >= sm->maxVcis)) {
				    Tcl_SetResult(interp,
						  "output VCID out of bounds",
						  TCL_STATIC);
				    return(TCL_ERROR);
				}
		    
				// got args, set translation entry
				sm->translate[inPort][inVcid].port = outPort;
				sm->translate[inPort][inVcid].vcid = outVcid;

			    }
			}
		    }
		}
		return (rc);

	    }
	    else {
		Tcl_SetResult(interp, "wrong # args", TCL_STATIC);
		return(TCL_ERROR);
	    }
		
	}

	//
	// sm output num object
	//
	// Connect switch module output to objects (such as CellQueues).
	//
	else if (!strcmp(argv[1], "output")) {

	    if (argc == 4) {

		int outPort;	// input parameters
		String outName;	// object name
		int rc;		// Tcl return code

		rc = Tcl_GetInt(interp, argv[2], &outPort);
		if (rc == TCL_OK) {

		    if ((outPort < 0) || (outPort >= sm->ports)) {
			Tcl_SetResult(interp,
				      "output port out of bounds",
				      TCL_STATIC);
			return(TCL_ERROR);
		    }

		    outName = argv[3];

		    // set new object...zero out cached object pointer
		    sm->outputPortName[outPort] = outName;
		    sm->outputPort[outPort] = NULL;

		}

		return (rc);
	    }
	    else {
		Tcl_SetResult(interp, "wrong # args", TCL_STATIC);
		return(TCL_ERROR);
	    }
	}
	else {
	    return(ScheduledObjectCmd(clientData, interp, argc, argv));
	}
	
    }
}

//
// SwitchModuleDeleteProc
//
// (Tcl DeleteProc)
//
void SwitchModuleDeleteProc(ClientData clientData)
{
    delete ((SwitchModule *) clientData);
}
