// -*- c++ -*-
//
// $Id: SwitchModule.h,v 1.6 1996-01-10 17:06:08-08 bmah Exp $
//
// SwitchModule.h
// Bruce A. Mah (bmah@CS.Berkeley.EDU)
//
// Copyright (c) 1995, 1996 The Regents of the University of California
// All Rights Reserved
// Created by Bruce A. Mah
// Department of Electrical Engineering and Computer Sciences,
//   Computer Science Division
//

#ifndef SWITCHMODULE_H
#define SWITCHMODULE_H

#include "ScheduledObject.h"
#include "CellQueue.h"
#include "AtmCell.h"
#include "Sig.h"

//
// Translation entry data structure
//
typedef struct {
    int port;			// output port
    int vcid;			// output vcid
} SwitchModuleTranslateEntry;

class Sig;

class SwitchModule : public ScheduledObject {

    friend int SwitchModuleCmd(ClientData clientData, Tcl_Interp *interp,
			       int argc, char *argv[]);
    friend int SwitchModuleObjectCmd(ClientData clientData, Tcl_Interp *interp,
				     int argc, char *argv[]);
    friend void Sig::Handler(Event *e);

    public:
    const int ports;		// number of translate ports
				// (in general, number of switch ports
				// plus one for signalling)

    const int maxVcis;		// maximum number of VCIs
				// (VCIDs go from 0 to numVcis-1)

    SwitchModule(const String &name, Simulation *si, int p, int mv) :
      ScheduledObject(name, si), ports(p), maxVcis(mv) {

	// Allocate memory for translation tables
	translate = new (SwitchModuleTranslateEntry *)[ports];
	for (int i = 0; i < ports; i++) {
	    translate[i] = new (SwitchModuleTranslateEntry)[maxVcis];
	}

	// Allocate memory for output ports
	outputPortName = new (String)[ports];
	outputPort = new (ScheduledObject *)[ports];

    }

    virtual ~SwitchModule() {

	// Free up translation tables' memory.
	for (int i = 0; i < ports; i++) {
	    delete [] translate[i];
	}
	delete [] translate;
	delete [] outputPortName;
	delete [] outputPort;
    }
			       
    virtual String Output(void);
    virtual void Handler(Event *e);

    protected:
    String *outputPortName;	// names of output port queues
    ScheduledObject **outputPort; // pointers to output port queues
    SwitchModuleTranslateEntry **translate; // translation arrays

    // Set translation table entry
    void Set(int upPort, int upVcid, int downPort, int downVcid) {
	(translate[upPort][upVcid]).port = downPort;
	(translate[upPort][upVcid]).vcid = downVcid;
    }

    // Unset translation table entry
    // (currently doesn't do anything)
    void Unset(int upPort, int upVcid) {
    }

};

extern int SwitchModuleCmd(ClientData clientData, Tcl_Interp *interp,
			   int argc, char *argv[]);
extern int SwitchModuleObjectCmd(ClientData clientData, Tcl_Interp *interp,
				 int argc, char *argv[]);
void SwitchModuleDeleteProc(ClientData clientData);

#endif // SWITCHMODULE_H
