// -*- c++ -*-
//
// $Id: Tcp.h,v 1.17 1996-03-07 10:59:08-08 bmah Exp $
//
// Tcp.h
// Bruce A. Mah (bmah@CS.Berkeley.EDU)
//
// Copyright (c) 1995, 1996 The Regents of the University of California
// All Rights Reserved
// Created by Bruce A. Mah
// Department of Electrical Engineering and Computer Sciences,
//   Computer Science Division
//
// This file contains material which is:

/*
 * Copyright (c) 1982, 1986, 1993
 *      The Regents of the University of California.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#ifndef TCP_H
#define TCP_H

#include <Uniform.h>
#include <iostream.h>
#include "ScheduledObject.h"
#include "Ip.h"
#include "IpPacket.h"
#include "TcpSegment.h"
#include "TcpData.h"
#include "TcpDataList.h"

//
// TCP definitions.  These are taken straight from BSD 4.4's .h files.
// We bring in the definitions that we need to avoid having to ship around
// a copy of the BSD 4.4 source code.  Earlier versions depended on the
// system include files in /usr/include, but this made for quite a few
// incompatabilities.
//

//
// #include <netinet/tcp.h>
//
#define	TH_FIN	0x01
#define	TH_SYN	0x02
#define	TH_RST	0x04
#define	TH_PUSH	0x08
#define	TH_ACK	0x10
#define	TH_URG	0x20

#define	TCP_MAXWIN	65535	/* largest value for (unscaled) window */

//
// #include <netinet/tcp_timer.h>
//

#define	TCPT_NTIMERS	4

#define	TCPT_REXMT	0		/* retransmit */
#define	TCPT_PERSIST	1		/* retransmit persistance */
#define	TCPT_KEEP	2		/* keep alive */
#define	TCPT_2MSL	3		/* 2*msl quiet time timer */

#define	TCPTV_MSL	( 30*PR_SLOWHZ)		/* max seg lifetime (hah!) */
#define	TCPTV_SRTTBASE	0			/* base roundtrip time;
						   if 0, no idea yet */
#define	TCPTV_SRTTDFLT	(  3*PR_SLOWHZ)		/* assumed RTT if no info */

#define	TCPTV_PERSMIN	(  5*PR_SLOWHZ)		/* retransmit persistance */
#define	TCPTV_PERSMAX	( 60*PR_SLOWHZ)		/* maximum persist interval */

#define	TCPTV_KEEP_INIT	( 75*PR_SLOWHZ)		/* initial connect keep alive */
#define	TCPTV_KEEP_IDLE	(120*60*PR_SLOWHZ)	/* dflt time before probing */
#define	TCPTV_KEEPINTVL	( 75*PR_SLOWHZ)		/* default probe interval */
#define	TCPTV_KEEPCNT	8			/* max probes before drop */

#define	TCPTV_MIN	(  1*PR_SLOWHZ)		/* minimum allowable value */
#define	TCPTV_REXMTMAX	( 64*PR_SLOWHZ)		/* max allowable REXMT value */

#define	TCP_LINGERTIME	120			/* linger at most 2 minutes */

#define	TCP_MAXRXTSHIFT	12			/* maximum retransmits */

//
// #include <sys/protosw.h>
//
#define PR_SLOWHZ 2
#define PR_FASTHZ 5

//
// #include <netinet/tcp_var.h>
//
#define TF_ACKNOW       0x01            /* ack peer immediately */
#define TF_DELACK       0x02            /* ack, but try to delay it */
#define TF_NODELAY      0x04            /* don't delay packets to coalesce */
#define TF_NOOPT        0x08            /* don't use tcp options */
#define TF_SENTFIN      0x10            /* have sent FIN */
#define TF_REQ_SCALE    0x0020          /* have/will request window scaling */
#define TF_RCVD_SCALE   0x0040          /* other side has requested scaling */
#define TCP_RTT_SCALE           8       /* multiplier for srtt; 3 bits frac. */
#define TCP_RTT_SHIFT           3       /* shift for srtt; 3 bits frac. */
#define TCP_RTTVAR_SCALE        4       /* multiplier for rttvar; 2 bits */
#define TCP_RTTVAR_SHIFT        2       /* multiplier for rttvar; 2 bits */
#define TCP_REXMTVAL(tp) \
        (((tp)->srtt >> TCP_RTT_SHIFT) + (tp)->rttvar)
//
// #include <netinet/tcp_seq.h>
//
#define SEQ_LT(a,b)     ((int)((a)-(b)) < 0)
#define SEQ_LEQ(a,b)    ((int)((a)-(b)) <= 0)
#define SEQ_GT(a,b)     ((int)((a)-(b)) > 0)
#define SEQ_GEQ(a,b)    ((int)((a)-(b)) >= 0)
#define TCP_ISSINCR     (125*1024)     /* increment for tcp_iss each second */

//
// TCP connection state values
//
enum Tcps {
    TcpsClosed = 0,		// closed
    TcpsListen,			// listening for connection
    TcpsSynSent,		// have sent SYN (active open)
    TcpsSynReceived,		// have sent and received Syn (awaiting ACK)
    TcpsEstablished,		// established (data transfer)
    TcpsCloseWait,		// received FIN, waiting for application close
    TcpsFinWait1,		// have closed, sent FIN, awaiting ACK/FIN
    TcpsClosing,		// simultaneous close, awaiting ACK
    TcpsLastAck,		// received FIN, have closed, waiting ACK
    TcpsFinWait2,		// have closed, awaiting FIN
    TcpsTimeWait		// 2MSL wait state after active close
};

//
// TCP reassembly queue element.  There is (potentially at least) an
// SLL of these things per TCP connection.
//
typedef struct Reassem {
    TcpSeq seq;			// sequence number for start of this segment
    int len;			// length of segment
    int flags;			// flags from TCP header
    TcpDataList data;		// data elements for segment
    struct Reassem *next; 	// next segment
};

//
// TCP protocol control block.  Like the Internet PCB, maintain a single
// DLL of TcpPcbs with a single "header" TcpPcb.  Not too surprisingly,
// this TCP is based heavily on the "real" struct tcpcb.
//
typedef struct TcpPcb {

    struct TcpPcb *next;	// next pointer for DLL
    struct TcpPcb *prev;	// prev pointer for DLL

    // Common PCB information
    InAddr tcp_faddr;		// foreign address
    unsigned short tcp_fport;	// foreign port
    InAddr tcp_laddr;		// local address
    unsigned short tcp_lport;	// local port

    // Socket-layer stuff
    TcpDataList so_snd;        // retransmit queue
    int so_snd_sb_cc; 		// number of bytes in send buffer
    int so_rcv_sb_cc;		// number of bytes in receive buffer
    int so_rcv_sb_hiwat;	// highwater ("full") mark for receive buffer
    Reassem *seg_next;		// reassembly queue
      
    // INSANE-specific stuff
    ScheduledObject *tcp_up;	// owner of packets for this PCB
    int rcv_bytes;		// number of bytes received on this connection
    int snd_bytes;		// number of bytes sent this connection

    // TCP state
    Tcps state;			// connection state
    short timer[TCPT_NTIMERS];	// TCP timers
    short rxtshift;		// exponent of retransmit exponential backoff
    short rxtcur;		// current retransmission timeout (ticks)
    short dupacks;		// #consecutive duplicate ACKs received
    u_short maxseg;		// maximum segment size to send
    char force;			// 1 if forcing out a byte
    u_short flags;		// flags

    // Receive variables
    TcpSeq rcv_adv;             // advertised window by other end

    // Retransmit
    TcpSeq snd_max; 		// maximum sequence number seen

    // Congestion control
    u_long snd_cwnd;            // congestion window size
    u_long snd_ssthresh;	// slowstart threshhold

    // Send sequence variables
    TcpSeq snd_una; 		// send unacknowledged
    TcpSeq snd_nxt;		// send next
    TcpSeq snd_up;		// send urgent pointer
    TcpSeq snd_wl1;		// window update segment sequence number
    TcpSeq snd_wl2;		// window update segment ack number
    TcpSeq iss;			// initial send sequence number
    u_long snd_wnd;		// send window

    // Retransmit timer variables
    short idle;			// inactivity time
    short rtt;			// round-trip time
    TcpSeq rtseq;		// sequence number being timed
    short srtt;			// smoothed round-trip time, fixed point,
				// clock ticks scaled by 8
    short rttvar;		// variance in round-trip time, fixed point,
				// clock ticks scaled by 4
    u_short rttmin;		// minimum rtt allowed
    u_long max_sndwnd;		// largest window peer has offered

    // Receive sequence variables
    u_long rcv_wnd; 		// receive window
    TcpSeq rcv_nxt;		// receive next
    TcpSeq rcv_up;		// receive urgent pointer
    TcpSeq irs;			// initial receive sequence number

};

//
// TCP call block
//
typedef struct TcpCall {
    TcpPcb *pcb;		// pointer to PCB (analogous to socket number)
    short th_ulen;		// length
    TcpDataList data;		// data
};

class Tcp : public ScheduledObject {

    friend int
    TcpCmd(ClientData clientData, Tcl_Interp *interp,
	   int argc, char *argv[]);
    friend int
    TcpObjectCmd(ClientData clientData, Tcl_Interp *interp,
		 int argc, char *argv[]);

    public:

    Tcp(const String &name, Simulation *si);
    virtual ~Tcp();
    virtual String Output(void);
    virtual void Handler(Event *e);

    // TCP API functions
    TcpPcb *UserListen(double time, unsigned short port, ScheduledObject *app);
    TcpPcb *UserConnect(double time, InAddr addr, unsigned short port,
			ScheduledObject *app, unsigned short lport = 0);
    void UserClose(double time, TcpPcb *tp);
    void UserSend(double time, TcpPcb *tp, short len, TcpDataList &data);
    void UserSend(double time, TcpPcb *tp, short len, TcpData *data);
    void UserSend(double time, TcpPcb *tp, short len);
    void UserSet(double time, TcpPcb *tp, int parm);
    void UserReset(double time, TcpPcb *tp, int parm);

    protected:				  

    // TCP globals
    TcpPcb *tcb;		// head of TCP control block
    int rexmtthresh;		// number of duplicate ACKs for fast retransmit
    int mssdflt;		// default Maximum Segment Size
    int rttdflt;		// default RTT if no data
    int keepidle;		// keepalive:  idle time before first probe
    int keepintvl;		// keepalive:  interval between probes
				// (also interval for connect)
    int maxidle;		// keepalive:  time after probing before
				// giving up
    int keepcnt;		// keepalive:  number of keepalive probes
    TcpSeq tcp_iss;		// default initial sequence number
    static const int Backoff[];	// exponential backoff array
    static const int Outflags[]; // flags for TCP header, indexed by state
    
    Ip *down;			// Ip pbject
    String downName;		// name of IP object
    double time;		// hack to get current simulation time to
				// INSANE-specific procedures that need it

    static Uniform *skewGenerator2; // Random offset to skew TCP timers
    static Uniform *skewGenerator5; // to prevent phased effects

    // TCP timing parameters
    double latencyToIp;	        // downward latency (towards IP)
    double latencyToApp;	// socket delays to application

    // TCP timer management routines
    void Fast(void);
    void Slow(void);
    void CancelTimers(TcpPcb *tp);
    TcpPcb *Timers(TcpPcb *tp, int timer);
    void SetPersist(TcpPcb *tp);
    void XmitTimer(TcpPcb *tp, short rtt);
    TcpPcb *NewTcpPcb(void);
    unsigned short EphemeralPort();

    // TCP connection management routines
    TcpPcb *Drop(TcpPcb *tp);
    TcpPcb *Close(TcpPcb *tp);
    void Mss(TcpPcb *tp);
    TcpPcb *PcbCheck(TcpPcb *tp);

    // TCP I/O routines
    int Out(TcpPcb *tp, int force);
    void Respond(TcpPcb *tp, IpPacket *m, TcpSeq ack, TcpSeq seq, int flags);
    void In(IpPacket *ipPacket);
    int Reass(TcpPcb *tp, TcpSeq seq, int len, int flags, TcpDataList &data);

};

extern int TcpCmd(ClientData clientData, Tcl_Interp *interp,
		  int argc, char *argv[]);
extern int TcpObjectCmd(ClientData clientData, Tcl_Interp *interp,
			int argc, char *argv[]);
extern void TcpDeleteProc(ClientData clientData);

extern ostream& operator<<(ostream &s, Tcps state);

#endif // TCP_H
