// -*- c++ -*-
// $Id: TcpDataList.h,v 1.3 1996-02-16 07:36:09-08 bmah Exp $
//   based on:
// $ID: AtmCellList.h,v 1.3 1995-09-12 10:15:48-07 bmah Exp $
//
// TcpDataList.h
//
// Copyright (c) 1996 The Regents of the University of California
// All Rights Reserved
// Created by Bruce A. Mah
// Department of Electrical Engineering and Computer Sciences,
//   Computer Science Division
//

#ifndef TCPDATALIST_H
#define TCPDATALIST_H

#include <stdlib.h>
#include <stdio.h>
#include "TcpData.h"

class TcpDataList {

    public:

    //
    // Constructor initializes the pointers to the ends of the lists
    //
    TcpDataList() {
	head = NULL;
	tail = NULL;
	length = 0;
    }

    //
    // Copy constructor
    //
    TcpDataList(TcpDataList &t) {
	head = NULL;
	tail = NULL;
	length = 0;

	TcpData *current;
	for (current = t.head; current != NULL; current = current->next) {
	    TcpData *data = new TcpData(*current);
	    EnqueueTail(data);
	}
	Sanity(*this);
	Sanity(t);
    }

    //
    // Destructor deallocates all of the cells that were in the list.
    //
    ~TcpDataList() {
	TcpData *current, *next;
	for (current = head; current != NULL; current = next) {
	    next = current->next;
	    delete current;
	}
    }

    // Enqueue and dequeue member functions do the expected things
    EnqueueTail(TcpData *data) {
	data->next = NULL;
	if (tail) {
	    tail->next = data;
	}
	else {
	    head = data;
	}
	tail = data;
	length++;

	Sanity(*this);
	
    }

    EnqueueHead(TcpData *data) {
	data->next = head;
	head = data;
	if (tail == NULL) {
	    tail = data;
	}
	length++;

	Sanity(*this);

    }

    TcpData *Dequeue() {
	TcpData *data;

	data = head;
	if (head) {
	    head = head->next;
	    if (head == NULL) {
		tail = NULL;
	    }
	    length--;
	}

	Sanity(*this);

	return data;
    }

    // Append this list to another.  Current list is cleared.
    void AppendTo(TcpDataList &other) {
	if (other.length == 0) {
	    other.head = head;
	}
	else {
	    other.tail->next = head;
	}
	if (length) {
	    other.tail = tail;
	}
	other.length += length;

	Sanity(other);
	
	head = NULL;
	tail = NULL;
	length = 0;

	Sanity(*this);

    }

    // Copy data tags within a given range of TCP sequence numbers
    // to a new list.
    void CopyData(TcpDataList &otherList, TcpSeq seq, int len) {
	TcpData *current;
	for (current = head; current != NULL; current = current->next) {
	    if ((current->seq >= seq) && (current->seq < (seq+len))) {
		TcpData *newCopy = new TcpData(*current);
		otherList.EnqueueTail(newCopy);
	    }
	}
	Sanity(otherList);
	Sanity(*this);
    }

    // Trim away all data tags not within a given range of TCP sequence
    // numbers.
    void Trim(TcpSeq seq, int len) {
	TcpData *current, *next, *prev;
	for (current = head, prev = NULL;
	     current != NULL;
	     current = next) {
	    next = current->next;
	    if ((current->seq < seq) || (current->seq >= (seq+len))) {
		if (prev) {
		    // delete non-first element
		    prev->next = current->next;
		}
		else {
		    // delete first element
		    head = current->next;
		}
		if (tail == current) {
		    // deleting last element
		    tail = prev;
		}
		delete current;
		length--;
	    }
	}
	Sanity(*this);
    }

    protected:
    TcpData *head, *tail;
    int length;

    // Sanity check on lists
    void Sanity(TcpDataList &list) {
	if (list.length < 0) { 
	    cerr << "TcpDataList:  negative length!\n";
	    abort();
	}
	else if (list.length == 0) {
	    if ((list.head != NULL) || (list.tail != NULL)) {
		cerr << "TcpDataList:  zero length but non-null pointer!\n";
		abort();
	    }
	}
	else if (list.length > 0) {
	    if ((list.head == NULL) || (list.tail == NULL)) {
		cerr << "TcpDataList:  non-zero length but null pointer!\n";
		abort();
	    }
	}
    }

};

#endif // TCPDATALIST_H
