// -*- c++ -*-
//
// $Id: TelnetClient.h,v 1.15 1996-06-07 09:36:29-07 bmah Exp $
//   based on:
// $ID: SampleTcpClient.h,v 1.3 1995-10-09 10:37:08-07 bmah Exp bmah $
//
// TelnetClient.h
// Bruce A. Mah (bmah@CS.Berkeley.EDU)
//
// Copyright (c) 1995, 1996 The Regents of the University of California
// All Rights Reserved
// Created by Bruce A. Mah
// Department of Electrical Engineering and Computer Sciences,
//   Computer Science Division
//

#ifndef TELNETCLIENT_H
#define TELNETCLIENT_H

#include "AppTcpClient.h"
#include "Simulation.h"
#include <SmplHist.h>

//
// Configuration constants
//
const int TelnetClientSendLength = 1; // Length of telnet client keystrokes
				      // in bytes.
const double TelnetClientZombieTime = 5.0; // Length of time that telnet
					   // client sticks around after
					   // its timeout expires.  This
					   // time must be at least as
					   // long as the latency of
					   // posting of a TcpArrival
					   // event to prevent crashes in
					   // some odd race conditions.

class TelnetClient : public AppTcpClient {

    friend int
    TelnetClientCmd(ClientData clientData, Tcl_Interp *interp,
	   int argc, char *argv[]);
    friend int
    TelnetClientObjectCmd(ClientData clientData, Tcl_Interp *interp,
		 int argc, char *argv[]);

    public:

    TelnetClient(const String &name, Simulation *si, double initTime,
		 String &t, InAddr s, short p, int nt);
    virtual ~TelnetClient();

    virtual String Output(void);
    virtual void Handler(Event *e);

    protected:				  

    double startTime;		// object start time
    double endTime;		// object end time
    double dataStartTime;	// data sending start time

    double lifeTime;		// connection lifetime

    int numTimestamps;		// number of timestamps we keep
    int tagNumber;		// tag number for client->server packets
    int lastReply;		// last reply tag received
    double *timestamps;		// timestamp array
    bool dormant;		// client dormant because ran out of timestamps

    int totalBytesSent;
    int totalBytesReceived;
    int overruns;		// number of times user overran timestamp slots

    SampleHistogram *rttStat;	// round trip statistics

};

extern int TelnetClientCmd(ClientData clientData, Tcl_Interp *interp,
		  int argc, char *argv[]);
extern int TelnetClientObjectCmd(ClientData clientData, Tcl_Interp *interp,
			int argc, char *argv[]);
extern void TelnetClientDeleteProc(ClientData clientData);

#endif // TELNETCLIENT_H
