// -*- c++ -*-
//
// $Id: Udp.h,v 1.8 1996-02-02 15:56:38-08 bmah Exp $
//
// Udp.h
// Bruce A. Mah (bmah@CS.Berkeley.EDU)
//
// Copyright (c) 1995, 1996 The Regents of the University of California
// All Rights Reserved
// Created by Bruce A. Mah
// Department of Electrical Engineering and Computer Sciences,
//   Computer Science Division
//

#ifndef UDP_H
#define UDP_H

#include "ScheduledObject.h"
#include "Ip.h"
#include "IpPacket.h"
#include "UdpData.h"
#include "UdpDataList.h"

//
// UDP protocol control block.  Like the Internet PCB, we maintain
// a circular DLL of UdpPcbs, with a single "header" UdpPcb.
//
typedef struct UdpPcb {
    InAddr udp_faddr;		// foreign address
    unsigned short udp_fport;	// foreign port
    InAddr udp_laddr;		// local address
    unsigned short udp_lport;	// local port
    
    ScheduledObject *udp_up;	// Owner of packets for this PCB

    struct UdpPcb *next;	// next pointer for DLL
    struct UdpPcb *prev;	// prev pointer for DLL
};

//
// UDP call block...used for UDP sends and receives.
//
typedef struct UdpCall {
    UdpPcb *pcb;		// pointer to PCB (analogous to socket number)
    InAddr udp_faddr;		// foreign address
    unsigned short udp_fport;	// foreign port
    short uh_ulen;		// length
    UdpDataList data;		// data
};

class Udp : public ScheduledObject {

    friend int
    UdpCmd(ClientData clientData, Tcl_Interp *interp,
	   int argc, char *argv[]);
    friend int
    UdpObjectCmd(ClientData clientData, Tcl_Interp *interp,
		 int argc, char *argv[]);

    public:

    Udp(const String &name, Simulation *si) :
    ScheduledObject(name, si) {

	// Initialize first UdpPcb record.  This record contains the
	// first "emphemeral" port number to allocate.
	udb = new UdpPcb;
	udb->udp_lport = 1024;
	udb->next = udb;
	udb->prev = udb;

    }

    virtual String Output(void);
    virtual void Handler(Event *e);

    UdpPcb *UserBind(double time,
		     InAddr faddr, unsigned short fport,
		     InAddr laddr, unsigned short lport,
		     ScheduledObject *owner);
    void UserClose(double time, UdpPcb *pcb);
    void UserSend(double time, UdpPcb *up, short len, UdpData *data,
		  UdpPayloadType type);
    void UserSend(double time, UdpPcb *up, short len, UdpData *data);
    void UserSend(double time, UdpPcb *up, short len);

    protected:				  

    UdpPcb *udb;		// DLL of UDP PCBs
    Ip *down;			// Ip object
    String downName;		// Name of IP object

};

extern int UdpCmd(ClientData clientData, Tcl_Interp *interp,
		  int argc, char *argv[]);
extern int UdpObjectCmd(ClientData clientData, Tcl_Interp *interp,
			int argc, char *argv[]);
extern void UdpDeleteProc(ClientData clientData);

#endif // UDP_H
