// -*- c++ -*-
// $Id: UdpDataList.h,v 1.2 1996-01-31 16:44:40-08 bmah Exp $
//   based on:
// $ID: TcpDataList.h,v 1.1 1995-10-19 14:22:33-07 bmah Exp $
//   based on:
// $ID: AtmCellList.h,v 1.3 1995-09-12 10:15:48-07 bmah Exp $
//
// UdpDataList.h
//
// Copyright (c) 1995, 1996 The Regents of the University of California
// All Rights Reserved
// Created by Bruce A. Mah
// Department of Electrical Engineering and Computer Sciences,
//   Computer Science Division
//

#ifndef UDPDATALIST_H
#define UDPDATALIST_H

#include <stdlib.h>
#include <stdio.h>
#include "UdpData.h"

class UdpDataList {

    public:

    //
    // Constructor initializes the pointers to the ends of the lists
    //
    UdpDataList() {
	head = NULL;
	tail = NULL;
	length = 0;
    }

    //
    // Destructor deallocates all of the cells that were in the list.
    //
    ~UdpDataList() {
	UdpData *current, *next;
	for (current = head; current != NULL; current = next) {
	    next = current->next;
	    delete current;
	}
    }

    // Enqueue and dequeue member functions do the expected things
    EnqueueTail(UdpData *data) {
	data->next = NULL;
	if (tail) {
	    tail->next = data;
	}
	else {
	    head = data;
	}
	tail = data;
	length++;

	Sanity(*this);

    }

    EnqueueHead(UdpData *data) {
	data->next = head;
	head = data;
	if (tail == NULL) {
	    tail = data;
	}
	length++;

	Sanity(*this);
	
    }

    UdpData *Dequeue() {
	UdpData *data;

	data = head;
	if (head) {
	    head = head->next;
	    if (head == NULL) {
		tail = NULL;
	    }
	    length--;
	}

	Sanity(*this);

	return data;
    }

    // Append this list to another.  Current list is cleared.
    void AppendTo(UdpDataList &other) {
	if (other.length == 0) {
	    other.head = head;
	}
	else {
	    other.tail->next = head;
	}
	if (length) {
	    other.tail = tail;
	}
	other.length += length;

	Sanity(other);

	head = NULL;
	tail = NULL;
	length = 0;

	Sanity(*this);

    }

    int length;

    protected:
    UdpData *head, *tail;

    // Sanity check on lists
    void Sanity(UdpDataList &list) {
	if (list.length < 0) { 
	    cerr << "UdpDataList:  negative length!\n";
//	    abort();
	}
	else if (list.length == 0) {
	    if ((list.head != NULL) || (list.tail != NULL)) {
		cerr << "UdpDataList:  zero length but non-null pointer!\n";
//		abort();
	    }
	}
	else if (list.length > 0) {
	    if ((list.head == NULL) || (list.tail == NULL)) {
		cerr << "UdpDataList:  non-zero length but null pointer!\n";
//		abort();
	    }
	}
    }

};

#endif // UDPDATALIST_H
