static char rcsid[] = "$Id: Zipf.cc,v 1.2 1995-11-27 22:09:58-08 bmah Exp $";
// -*- c++ -*-
// $Id: Zipf.cc,v 1.2 1995-11-27 22:09:58-08 bmah Exp $
//
// Zipf.h
// Bruce A. Mah (bmah@CS.Berkeley.EDU)
//
// Copyright (c) 1995 The Regents of the University of California
// All Rights Reserved
// Created by Bruce A. Mah
// Department of Electrical Engineering and Computer Sciences,
//   Computer Science Division
//
// Random variable class to produce values from a Zipf's Law distribution.
// This distribution is the discrete version of the Pareto distribution,
// and can be used to model the popularity of items chosen from a finite
// population, where the popularity of the nth most popular item
// is (1/n).
//
// This distribution has one parameter, namely the number of items.
// Values returned from the distribution are integers, in the range
// [1..items] inclusive.
//

#include "Zipf.h"

//
// Zipf::operator()
//
// Input:  none
//
// Output:  Value from a Zipf's Law distribution.  Note that this
// function has integral return values, although they are cast as doubles.
//
double
Zipf::operator()() {

    double u;
    double tryCdf;
    int n;

    // Get a Uniform[0,1] random variable
    u = pGenerator->asDouble();

    // Invert the CDF
    tryCdf = 0.0;
    n = 1;

    while (true) {

	if (n > pItems) {
	    abort();
	}

	tryCdf += (1.0 / (double) n);
	if (tryCdf / c >= u) {
	    break;
	}
	n++;
    }

    return (double) n;

}
