// -*- c++ -*-
// $Id: Zipf.h,v 1.2 1995-11-27 22:10:08-08 bmah Exp $
//
// Zipf.h
// Bruce A. Mah (bmah@CS.Berkeley.EDU)
//
// Copyright (c) 1995 The Regents of the University of California
// All Rights Reserved
// Created by Bruce A. Mah
// Department of Electrical Engineering and Computer Sciences,
//   Computer Science Division
//
// Zipf's Law random number distribution class.
//

#ifndef ZIPF_H
#define ZIPF_H

#include <Random.h>

class Zipf : public Random {

public:
    int items();
    int items(int it);

    Zipf(int it, RNG *gen);

    virtual double operator()();    

protected:
    int pItems;			// number of items
    double c;			// pre-computed constant

};

//
// Zipf::items
//
// Input:  None
//
// Output:  Number of items parameter.
//
// Access method to examine the items parameter.
//
inline int
Zipf::items()
{
    return pItems;
}

//
// Zipf::items
//
// Input:  New items parameter.
//
// Output:  Old items parameter
//
// Access method to set the items parameter.  The old value is returned.
//
inline int
Zipf::items(int it)
{
    int t;
    register int i;

    // Save old value and set new value.
    t = pItems;
    pItems = it;

    // Compute new c value (a constant which is a little expensive to
    // recompute for every invocation).
    c = 0.0;
    for (i = 1; i <= pItems; i++) {
	c += (1.0 / (double) i);
    }
    
    return t;
}

//
// Zipf::Zipf
//
// Input:  Number of items to consider (it), random number generator (gen)
//
// Output:  None.
//
inline
Zipf::Zipf(int it, RNG *gen) :
Random(gen)
{
    (void) items(it);
}

#endif // ZIPF_H
