// -*- c++ -*-
//
// $Id: apiproto.h,v 1.5 1996-04-16 11:03:20-07 bmah Exp $
//
// apiproto.h
// Bruce A. Mah (bmah@CS.Berkeley.EDU)
//
// Copyright (c) 1996 The Regents of the University of California
// All Rights Reserved
// Created by Bruce A. Mah
// Department of Electrical Engineering and Computer Sciences,
//   Computer Science Division
//
// Definitions for API side of an ATM-like signalling protocol.  This 
// protocol is implemented by the Sig and Sighost classes, and is roughly
// equivalent to either the Real-Time Channel Adminstration Protocol (RCAP)
// or the Generic Universal Line Protocol (GULP).
//

#ifndef APIPROTO_H
#define APIPROTO_H

#include "SigMessage.h"

//
// Types of signalling messages (based heavily on RCAP)
//
enum ApiType {
    ApiTypeEstablish,		// request to establish a new connection
				// (establish_request)
    ApiTypeReturn,		// return values
    ApiTypeDelete,		// teardown request
};

class ScheduledObject;

//
// API message
//
struct ApiMessage{

    ApiType type;		// message type (ALL)

    ScheduledObject *client;	// client who sent request
    
    int vcid;			// VCID to operate on (ALL)

    int requestId;		// request ID number (all)
    
    SigReturnCode code;		// return code (ApiTypeReturn)

    SigAddress sourceAddress;	// source of establishment (SigTypeEstablish
    SigService sourceService;	// only)

    SigAddress destinationAddress; // destination of establishment
    SigService destinationService; // (SigTypeEstablish only)

    void *traffic;		// traffic specification (SigTypeEstablish
				// and SigTypeEstablishAck only).  In a
				// real signalling protocol, we'd just
				// put this data in the signalling message,
				// but to support our object paradigm we've
				// chosen to put a pointer to a scheduler-
				// dependent parameter block.
    int trafficLength;		// length of traffic descriptor

};

extern String ApiMessageTypeString(ApiType type);

#endif // APIPROTO_H
