// -*- c++ -*-
// $Id: inline.h,v 1.12 1996-01-31 11:54:14-08 bmah Exp $
//
// inline.h
// Bruce A. Mah (bmah@CS.Berkeley.EDU)
//
// Copyright (c) 1995, 1996 The Regents of the University of California
// All Rights Reserved
// Created by Bruce A. Mah
// Department of Electrical Engineering and Computer Sciences,
//   Computer Science Division
//
// Inline function definitions, optionally to be included for improved
// performance.  This header file defines some object methods which
// probably ought to be declared inline but, for various reasons, cannot
// be defined in the header files for their respective classes.
// This is really a hack, as there are now duplicate function definitions
// in different source files.
//
#ifndef INLINE_H
#define INLINE_H

//
// ScheduledObject class inlined methods
//
#ifdef SCHEDULEDOBJECT_H

#include "util.h"
#include "Tcl.h"
#include "String.ScheduledObjectPtr.CHMap.h"
#include "Calendar.h"
#include "Simulation.h"

//
// ScheduledObject::Post
//
// Input:  time, recipient of event (object), object type (type),
//         event data (data)
//
// Output:  None
//
// Post an object to the simulation object's scheduler.
//
inline void ScheduledObject::Post(double time, ScheduledObject *object,
				  EventType type, void *data)
{
    Event *e = new Event(time, object, type, data);
    e->ev_sender = this;
    sim->scheduler->Insert(e);
}

//
// ScheduledObject::Lookup
//
// Input:  Object pointer (theObject), object name (theName), fatal flag
//         (fatal, defaults to true)
//
// Output:  None
//
// Look up the name of an object.  If the cache flag for *this* object
// is set and the pointer is valid (non-NULL), then bypass the lookup.
// If the fatal flag is set and the lookup fails, then panic.
//
inline void
ScheduledObject::Lookup(ScheduledObjectPtr &theObject,
			const String &theName, bool fatal)
{
    if (!cache || !theObject) {
	theObject = (*registry)[theName];
	if (!theObject && fatal) {
	    panic("ScheduledObject::Lookup failure: " + name +
		  " tried to lookup " + theName);
	}
    }
}

inline void
ScheduledObject::Lookup(Aal *&theObject, const String &theName,
	       bool fatal) {
    ScheduledObject *temp = (ScheduledObject *) theObject;
    Lookup(temp, theName, fatal);
    theObject = (Aal *) temp;
};

inline void
ScheduledObject::Lookup(CellGoBackN *&theObject, const String &theName,
	       bool fatal) {
    ScheduledObject *temp = (ScheduledObject *) theObject;
    Lookup(temp, theName, fatal);
    theObject = (CellGoBackN *) temp;
};

inline void
ScheduledObject::Lookup(DeviceDriver *&theObject, const String &theName,
	       bool fatal) {
    ScheduledObject *temp = (ScheduledObject *) theObject;
    Lookup(temp, theName, fatal);
    theObject = (DeviceDriver *) temp;
};

inline void
ScheduledObject::Lookup(Ip *&theObject, const String &theName,
	       bool fatal) {
    ScheduledObject *temp = (ScheduledObject *) theObject;
    Lookup(temp, theName, fatal);
    theObject = (Ip *) temp;
};

inline void
ScheduledObject::Lookup(SwitchModule *&theObject, const String &theName,
			bool fatal) {
    ScheduledObject *temp = (ScheduledObject *) theObject;
    Lookup(temp, theName, fatal);
    theObject = (SwitchModule *) temp;
};

inline void
ScheduledObject::Lookup(Tcp *&theObject, const String &theName,
			bool fatal) {
    ScheduledObject *temp = (ScheduledObject *) theObject;
    Lookup(temp, theName, fatal);
    theObject = (Tcp *) temp;
};

inline void
ScheduledObject::Lookup(Udp *&theObject, const String &theName,
			bool fatal) {
    ScheduledObject *temp = (ScheduledObject *) theObject;
    Lookup(temp, theName, fatal);
    theObject = (Udp *) temp;
};

#endif // SCHEDULEDOBJECT_H

#endif // INLINE_H
