static char rcsid[] = "$Id: tclRandom.cc,v 1.1 1996-01-28 01:28:21-08 bmah Exp $";
//
// $Id: tclRandom.cc,v 1.1 1996-01-28 01:28:21-08 bmah Exp $
//
// tclRandom.cc
// Bruce A. Mah (bmah@CS.Berkeley.EDU)
//
// Copyright (c) 1996 The Regents of the University of California
// All Rights Reserved
// Created by Bruce A. Mah
// Department of Electrical Engineering and Computer Sciences,
//   Computer Science Division
//
// Some glue to allow Tcl programs to access some of the random
// number generators present on the C++ side of the house.
//

#include "tclRandom.h"
#include <RNG.h>
#include <Uniform.h>
#include "Zipf.h"

//
// Global declarations
//
extern RNG *rng;

//
// uniformCmd
//
// (Tcl command procedure)
//
// Generate a [0,1] uniform random number.  Takes no arguments.
//
int
uniformCmd(ClientData clientData, Tcl_Interp *interp,
	   int argc, char *argv[])
{
    static Uniform distribution(0.0, 1.0, rng);

    if (argc == 1) {

	double x;		// random value

	// Get the random value, return to caller.
	x = distribution();
	sprintf(interp->result, "%f", x);
	return(TCL_OK);

    }
    else {
	Tcl_SetResult(interp, "usage:  uniform", TCL_STATIC);
	return(TCL_ERROR);
    }
}

//
// zipfCmd
//
// (Tcl command procedure)
//
// Generate a Zipf's Law distributed random number.  Takes one
// argument, which must be the number of items to select from.
// If called with no arguments, generates a number, using the
// same items parameter as the last time called.
//
int
zipfCmd(ClientData clientData, Tcl_Interp *interp,
	int argc, char *argv[])
{

    // The Zipf's Law random number distribution
    static Zipf distribution(1, rng);

    // Basically...if there was an argument, then process it
    // and give new arguments to the distribution object.
    // Fall through to the 0-argument case.
    if (argc == 2) {
	int rc = TCL_OK;	// Tcl return value
	int n;			// integer parameter for distribution
	
	rc = Tcl_GetInt(interp, argv[1], &n);
	if (rc == TCL_OK) {

	    // Tell the distribution how many items we're drawing from.
	    (void) distribution.items(n);
	    argc--;
	    
	}
	else {
	    return rc;
	}

    }

    // If no arguments given (or fell through from above), then
    // return a value to the caller.
    if (argc == 1) {

	int x;		// random value

	// Get the random value.  Note that the distribution returns
	// doubles, which we then cast to ints.
	x = (int) distribution();
	
	// Return the value to the caller
	sprintf(interp->result, "%d", x);
	return(TCL_OK);
	
    }
    else {
	Tcl_SetResult(interp, "usage:  zipf [n]", TCL_STATIC);
	return(TCL_ERROR);
    }

}


