static char rcsid[] = "$Id: user.cc,v 1.30 1996-05-29 14:35:18-07 bmah Exp $";
//
// $Id: user.cc,v 1.30 1996-05-29 14:35:18-07 bmah Exp $
// user.cc
//
// Bruce A. Mah (bmah@CS.Berkeley.EDU)
//
// Copyright (c) 1995, 1996 The Regents of the University of California
// All Rights Reserved
// Created by Bruce A. Mah
// Department of Electrical Engineering and Computer Sciences,
//   Computer Science Division
//
// Implements the procedures for the INSANE user interface and
// Tcl scripts.
//

#ifdef HAVE_READLINE
extern "C" {
#include <stdio.h>
#include "readline.h"
#include "history.h"
}
#endif // HAVE_READLINE

#include "Tcl.h"
#include "Simulation.h"
#include "Aal.h"
#include "AtmDeviceDriver.h"
#include "AtmTest.h"
#include "AudioSink.h"
#include "AudioSource.h"
#include "CellGoBackN.h"
#include "CellInputPort.h"
#include "CellQueueFifo.h"
#include "CellQueueRcsp.h"
#include "CellSink.h"
#include "EventSink.h"
#include "FtpClient.h"
#include "FtpServer.h"
#include "HttpClient.h"
#include "HttpServer.h"
#include "Ip.h"
#include "Lan.h"
#include "LanDeviceDriver.h"
#include "LoopbackDeviceDriver.h"
#include "NullDeviceDriver.h"
#include "NntpServer.h"
#include "PeriodicProcess.h"
#include "PoissonProcess.h"
#include "SampleTcpClient.h"
#include "SampleTcpServer.h"
#include "Sig.h"
#include "SigHost.h"
#include "SigHostRcsp.h"
#include "SigRcsp.h"
#include "SmtpClient.h"
#include "SwitchModule.h"
#include "Tcp.h"
#include "TelnetClient.h"
#include "TelnetServer.h"
#include "Udp.h"
#include "VideoSink.h"
#include "VideoSource.h"
#include "VideoSource2.h"
#include "tclRandom.h"
#include "user.h"

Tcl *tcl;
extern const char *InsaneInit;

#ifdef HAVE_READLINE
static char *LineRead = (char *) NULL;
#endif

//
// InitUser
//
// Input:  command line initialization string (cmdlineInit)
//         configuration file (configFile)
//         simulation object (sim)
//
// Output:  return value (0 if no error)
//
// Initialize Tcl environment, evaluate command line initialization string,
// and read initial simulation configuration
// from the configuration file.
//
// The command line initialization string is intended to support
// minor changes to simulation configurations without the need
// to generate new configuration files for each run.
//
int InitUser(String &cmdlineInit, String &configFile, Simulation *sim)
{

    int rc = TCL_OK;		// return code from Tcl
    String tclCommand;		// Tcl command under composition
    extern String LibDir;	// global variable from main.cc

    // Get new Tcl interpreter
    tcl = new Tcl;

    // Perform other Tcl initialization
    rc = tcl->Init();
    if (rc != TCL_OK) {
	if (!(tcl->result.empty())) {
	    cerr << tcl->result << '\n';
	}
	return(-1);
    }

    // Set autoload directory
    tclCommand = "set auto_path [linsert $auto_path 0 " + LibDir + "]";
    rc = tcl->Eval(tclCommand);
    if (rc != TCL_OK) {
	if (!(tcl->result.empty())) {
	    cerr << tcl->result << '\n';
	}
	return(-1);
    }

    // Perform InsaneInit Tcl initialization
    rc = tcl->Eval(InsaneInit);
    if (rc != TCL_OK) {
	if (!(tcl->result.empty())) {
	    cerr << tcl->result << '\n';
	}
	return(-1);
    }

    // Register Tcl command handlers for objects
    tcl->CreateCommand("Aal",
		       AalCmd,
		       NULL,
		       NULL);
    tcl->CreateCommand("AtmDeviceDriver",
		       AtmDeviceDriverCmd,
		       NULL,
		       NULL);
    tcl->CreateCommand("AtmTest",
		       AtmTestCmd,
		       NULL,
		       NULL);
    tcl->CreateCommand("AudioSink",
		       AudioSinkCmd,
		       NULL,
		       NULL);
    tcl->CreateCommand("AudioSource",
		       AudioSourceCmd,
		       NULL,
		       NULL);
    tcl->CreateCommand("CellGoBackN",
		       CellGoBackNCmd,
		       NULL,
		       NULL);
    tcl->CreateCommand("CellInputPort",
		       CellInputPortCmd,
		       NULL,
		       NULL);
    tcl->CreateCommand("CellQueueFifo",
		       CellQueueFifoCmd,
		       NULL,
		       NULL);
    tcl->CreateCommand("CellQueueRcsp",
		       CellQueueRcspCmd,
		       NULL,
		       NULL);
    tcl->CreateCommand("CellSink",
		       CellSinkCmd,
		       NULL,
		       NULL);
    tcl->CreateCommand("EventSink",
		       EventSinkCmd,
		       NULL,
		       NULL);
    tcl->CreateCommand("FtpClient",
		       FtpClientCmd,
		       NULL,
		       NULL);
    tcl->CreateCommand("FtpServer",
		       FtpServerCmd,
		       NULL,
		       NULL);
    tcl->CreateCommand("HttpClient",
		       HttpClientCmd,
		       NULL,
		       NULL);
    tcl->CreateCommand("HttpServer",
		       HttpServerCmd,
		       NULL,
		       NULL);
    tcl->CreateCommand("Ip",
		       IpCmd,
		       NULL,
		       NULL);
    tcl->CreateCommand("Lan",
		       LanCmd,
		       NULL,
		       NULL);
    tcl->CreateCommand("LanDeviceDriver",
		       LanDeviceDriverCmd,
		       NULL,
		       NULL);
    tcl->CreateCommand("LoopbackDeviceDriver",
		       LoopbackDeviceDriverCmd,
		       NULL,
		       NULL);
    tcl->CreateCommand("NntpServer",
		       NntpServerCmd,
		       NULL,
		       NULL);
    tcl->CreateCommand("NullDeviceDriver",
		       NullDeviceDriverCmd,
		       NULL,
		       NULL);
    tcl->CreateCommand("PeriodicProcess",
		       PeriodicProcessCmd,
		       NULL,
		       NULL);
    tcl->CreateCommand("PoissonProcess",
		       PoissonProcessCmd,
		       NULL,
		       NULL);
    tcl->CreateCommand("SampleTcpClient",
		       SampleTcpClientCmd,
		       NULL,
		       NULL);
    tcl->CreateCommand("SampleTcpServer",
		       SampleTcpServerCmd,
		       NULL,
		       NULL);
    tcl->CreateCommand("Sig",
		       SigCmd,
		       NULL,
		       NULL);
    tcl->CreateCommand("SigHost",
		       SigHostCmd,
		       NULL,
		       NULL);
    tcl->CreateCommand("SigHostRcsp",
		       SigHostRcspCmd,
		       NULL,
		       NULL);
    tcl->CreateCommand("SigRcsp",
		       SigRcspCmd,
		       NULL,
		       NULL);
    tcl->CreateCommand("SmtpClient",
		       SmtpClientCmd,
		       NULL,
		       NULL);
    tcl->CreateCommand("SwitchModule",
		       SwitchModuleCmd,
		       NULL,
		       NULL);
    tcl->CreateCommand("Tcp",
		       TcpCmd,
		       NULL,
		       NULL);
    tcl->CreateCommand("TelnetClient",
		       TelnetClientCmd,
		       NULL,
		       NULL);
    tcl->CreateCommand("TelnetServer",
		       TelnetServerCmd,
		       NULL,
		       NULL);
    tcl->CreateCommand("Udp",
		       UdpCmd,
		       NULL,
		       NULL);
    tcl->CreateCommand("VideoSink",
		       VideoSinkCmd,
		       NULL,
		       NULL);
    tcl->CreateCommand("VideoSource",
		       VideoSourceCmd,
		       NULL,
		       NULL);
    tcl->CreateCommand("VideoSource2",
		       VideoSource2Cmd,
		       NULL,
		       NULL);
    tcl->CreateCommand("sim",
		       SimulationCmd,
		       sim,
		       NULL);
    tcl->CreateCommand("quit",
		       QuitCmd,
		       NULL,
		       NULL);
    tcl->CreateCommand("uniform",
		       uniformCmd,
		       NULL,
		       NULL);
    tcl->CreateCommand("zipf",
		       zipfCmd,
		       NULL,
		       NULL);

    // Evaluate command line initialization string
    rc = tcl->Eval(cmdlineInit);
    if (rc != TCL_OK) {
	tcl->AddErrorInfo("\n    from\nthe command line initialization string \"" + cmdlineInit + "\"");
	cerr << tcl->GetVar("errorInfo", TCL_GLOBAL_ONLY) << "\n";
	return(-1);
    }

    // Read configuration file
    rc = tcl->EvalFile(configFile);

    if (rc == TCL_OK) {
	return(0);
    }
    else {
	tcl->AddErrorInfo("\n    from within\nthe INSANE configuration file \"" + configFile + "\"");
	cerr << tcl->GetVar("errorInfo", TCL_GLOBAL_ONLY) << "\n";
	return(-1);
    }
    
}

//
// QuitCmd
//
// Input:
//
// Output:
//
// Immediate quit from insane, no cleanup actions or end-of-run
// statistics.  This command only makes sense from the CLI.
//
int QuitCmd(ClientData clientData, Tcl_Interp *interp,
		  int argc, char *argv[]) {
    exit(0);
}

//
// Cli
//
// Input:  Simulation object (sim)
//
// Output:  None
//
// Tcl command line interface (CLI).
//
void Cli(Simulation *sim)
{

    const int MaxLine = 1000;
    
    String command;
    char buf[MaxLine];
    char c;
    int rc;			// return code from Tcl interpreter

    // Set this flag so we stay in the CLI.  The "sim go" command
    // will reset the flag to return the main event loop (as will
    // a few selected other commands).
    sim->stayInCli = true;

    // Output some useful information.
    cout << "CLI:  time " << sim->scheduler->clock << "\n";

    // lather, rinse, repeat
    while (sim->stayInCli) {

#ifdef HAVE_READLINE

	// Read a string, and return a pointer to it.  Returns NULL on EOF.

	// If the buffer has already been allocated, return the memory
	// to the free pool.
	if (LineRead)
	  {
	      free (LineRead);
	      LineRead = (char *)NULL;
	  }
	
	// Get a line from the user (use Integer Basic prompt).
	LineRead = readline ("> ");
	     
	/* If the line has any text in it, save it on the history. */
	if (LineRead && *LineRead) {
	    add_history (LineRead);
	}

	command = (String) LineRead;

#else
	// Read command line
	cout << "] "; // tribute to AppleSoft BASIC
	cin.get(buf, MaxLine, '\n');
	command = buf;

	// Eat newline.  If it wasn't a newline, the command line was
	// too long for the buffer...complain and ignore the whole line.
	if ((cin.get(c)) && (c != '\n')) {

	    cout << '\b';
	    while ((cin.get(c)) && (c != '\n')) {
	    }
	    continue;

	}
#endif
	// Evaluate command line (put in history list).
	rc = tcl->RecordAndEval(command, 0);
	if (rc != TCL_OK) {
	    tcl->AddErrorInfo("\n    from\nthe INSANE debugger command line");
	    cout << tcl->GetVar("errorInfo", TCL_GLOBAL_ONLY) << "\n";
	}
	else {
	    if (!(tcl->result.empty())) {
		cout << tcl->result << '\n';
	    }
	}
    }
}
