static char rcsid[] = "$Id: main.cc,v 1.7 1996-10-27 16:55:12-08 bmah Exp $";
//
// $Id: main.cc,v 1.7 1996-10-27 16:55:12-08 bmah Exp $
//
// main.cc
// Bruce A. Mah (bmah@CS.Berkeley.EDU)
//
// Main program for INSANE monitor program
//
// Copyright (c) 1996 The Regents of the University of California
// All Rights Reserved
// Created by Bruce A. Mah
// Department of Electrical Engineering and Computer Sciences,
//   Computer Science Division
//

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>

#include <iostream.h>
#include <String.h>
extern "C" {
#include "tcl.h"
#include "tk.h"
}

#include "GiddyFace.h"

//
// Configuration
//
const int BufferLength = 1024;
extern char *ui;
extern char *version;

//
// Globals
//
Tcl_Interp *interp;		// Tcl interpreter
char *argPort;			// port number
char *Buffer;			// incoming packet buffer
static char tclHandleUpdate[] = "HandleUpdate $UpdatePacket";

//
// UdpHandler
//
// (Tk file handler)
//
// Handles inbound status report packets.  clientData argument is the
// socket to read from.  Puts the value from the inbound UDP packet
// in a Tcl global variable (UpdatePacket) and calls the Tcl procedure
// HandleUpdate.
//
void UdpHandler(ClientData clientData, int mask)
{

    int sock = (int) clientData;	// socket
    int len;				// length of status update packet
    int rc;				// Tcl return code
    char *value;			// variable value
    
    if ((len = recv(sock,
		    Buffer,
		    BufferLength,
		    0)) < 0) {
	perror("recv");
    }

    value = Tcl_SetVar(interp, "UpdatePacket", Buffer, TCL_LEAVE_ERR_MSG);
    if (value == NULL) {
	cerr << "Tcl_SetVar error\n";
	Tcl_AddErrorInfo(interp,
			 "\n    from\nUdpHandler()");
	cerr << Tcl_GetVar(interp, "errorInfo", TCL_GLOBAL_ONLY) << "\n";
    }

    rc = Tcl_Eval(interp, tclHandleUpdate);
    if (rc != TCL_OK) {
	Tcl_AddErrorInfo(interp,
			 "\n    from\nUdpHandler()");
	cerr << Tcl_GetVar(interp, "errorInfo", TCL_GLOBAL_ONLY) << "\n";
    }

}

//
// main
//
// Input:  the usual suspects (argc, argv)
//
// Output:
//
int main(int argc, char **argv)
{

    int rc;			// Tcl/Tk return code

    //
    // Parse command line arguments
    //
    if (argc != 2) {
	cerr << argv[0] << " port\n";
	exit(1);
    }
    argPort = argv[1];
    
    //
    // Do Tcl/Tk initialization
    //
    interp = Tcl_CreateInterp();

    //
    // Create the main window, menus, etc.
    //
    Tk_Window w;

#if ( (TK_MAJOR_VERSION < 4) || ((TK_MAJOR_VERSION == 4) && (TK_MINOR_VERSION < 1)))
    w = Tk_CreateMainWindow(interp, NULL, "InsaneMon", "insanemon");
#endif TK_MAJOR_VERSION

    //
    // Initialize Tcl and Tk
    //
    rc = Tcl_Init(interp);
    if (rc != TCL_OK) {
	cerr << "Tcl_Init failed\n";
	exit(1);
    }
    rc = Tk_Init(interp);
    if (rc != TCL_OK) {
	cerr << "Tk_Init failed\n";
	exit(1);
    }

    //
    // Get version info
    // 
    rc = Tcl_Eval(interp, version);
    if (rc != TCL_OK) {
	Tcl_AddErrorInfo(interp,
			 "\n    from\nINSANE monitor version settings");
	cerr << Tcl_GetVar(interp, "errorInfo", TCL_GLOBAL_ONLY) << "\n";
	exit(1);
    }

    //
    // Run our UI script
    // 
    rc = Tcl_Eval(interp, ui);
    if (rc != TCL_OK) {
	Tcl_AddErrorInfo(interp,
			 "\n    from\nINSANE monitor UI initialization");
	cerr << Tcl_GetVar(interp, "errorInfo", TCL_GLOBAL_ONLY) << "\n";
	exit(1);
    }

    //
    // Open socket for status messages
    //
    int s; 			// socket
    int flagReuse;		// address use flag
    struct sockaddr_in addressLocal; // local address
    struct hostent *theHost;	// host entry structure

    s = socket(AF_INET, SOCK_DGRAM, 0);
    if (s < 0) {
        perror("socket");
        exit(1);
    }
    if (setsockopt(s, SOL_SOCKET, SO_REUSEADDR, (char *) &flagReuse,
                   sizeof(flagReuse)) < 0) {
        perror("setsockopt(SO_REUSEADDR)");
        exit(1);
    }
    addressLocal.sin_family = AF_INET;
    addressLocal.sin_port = htons(atoi(argPort));
    addressLocal.sin_addr.s_addr = INADDR_ANY;
    if (bind(s, (struct sockaddr *) &addressLocal,
             sizeof(addressLocal)) < 0) {
        perror("bind");
        exit(1);
    }

    // Get space for a packet buffer
    Buffer = new char[BufferLength];

    //
    // Register callback on the socket.  ClientData for this
    // callback is the socket number.
    //
    Tk_CreateFileHandler(s, TK_READABLE, UdpHandler, (ClientData) s);

    //
    // More initialization
    //
    rc = Tk_DefineBitmap(interp, Tk_GetUid("GiddyFace"),
			 (char *) GiddyFace_bits,
			 GiddyFace_width, GiddyFace_height);
    if (rc != TCL_OK) {
	Tcl_AddErrorInfo(interp,
			 "\n    from\nInsaneMon Tk_DefineBitmap(GiddyFace)");
	cerr << Tcl_GetVar(interp, "errorInfo", TCL_GLOBAL_ONLY) << "\n";
	exit(1);
    }

    //
    // Let Tk handle the rest!
    // 
    Tk_MainLoop();

    //
    // Cleanup stuff
    //
    close(s);
    exit(0);
    
}
