#!/usr/sww/bin/wish -f
#
# $Id: ui.tcl,v 1.5 1996-03-05 20:20:40-08 bmah Exp $
#
# ui.tcl
# Bruce A. Mah (bmah@CS.Berkeley.EDU)
#
# Copyright (c) 1996 The Regents of the University of California
# All Rights Reserved
# Created by Bruce A. Mah
# Department of Electrical Engineering and Computer Sciences,
#   Computer Science Division
#
# Set up user interface for InsaneMon.
#

#
# Create menu bar and menus
#
frame .mbar -relief raised -bd 2
pack .mbar -side top -fill x

menubutton .mbar.file -text File -menu .mbar.file.menu
menu .mbar.file.menu -tearoff 0
.mbar.file.menu add command -label Quit -command "exit"

menubutton .mbar.edit -text Edit -menu .mbar.edit.menu
menu .mbar.edit.menu -tearoff 0
.mbar.edit.menu add command -label Undo -command "" -state disabled
.mbar.edit.menu add separator
.mbar.edit.menu add command -label Cut -command "" -state disabled
.mbar.edit.menu add command -label Copy -command "" -state disabled
.mbar.edit.menu add command -label Paste -command "" -state disabled
.mbar.edit.menu add separator
.mbar.edit.menu add command -label Clear -command "ClearRawCommand"

menubutton .mbar.help -text Help -menu .mbar.help.menu
menu .mbar.help.menu -tearoff 0
.mbar.help.menu add command -label "About InsaneMon..." -command "About"

pack .mbar.file -side left
pack .mbar.edit -side left
pack .mbar.help -side right
focus .mbar

#
# Create toolbox buttons
#
frame .toolbox
pack .toolbox -side top -fill x

button .toolbox.clearRaw -text "Clear Raw" -command "ClearRawCommand"
pack .toolbox.clearRaw -side left -fill y

button .toolbox.clearStatus -text "Clear Status" -command "ClearStatCommand"
pack .toolbox.clearStatus -side left -fill y

set Mute 0
checkbutton .toolbox.mute -text "Mute Raw Updates" -variable Mute -relief raised
pack .toolbox.mute -side right -fill y -ipadx 2

#
# Create status display with listbox
#
frame .stat
pack .stat -side top -fill both
listbox .stat.list -yscrollcommand ".stat.scroll set" -width 30
scrollbar .stat.scroll -orient vertical -command ".stat.list yview"
pack .stat.list -side left -fill both -expand 1
pack .stat.scroll -side left -fill y

text .stat.text -relief raised -bd 2 -height 6 -width 60
pack .stat.text -side right -fill both -expand 1 

# Set up a binding so that when user releases their mouse button
# on an entry, an expanded display for that entry shows up in .stat.text
bind .stat.list <ButtonRelease-1> { StatListSelect %W %y }

set SelectedEntry ""

#
# Create raw update display
#
frame .raw
pack .raw -side top -fill both -expand 1
text .raw.text -relief raised -bd 2 -yscrollcommand ".raw.scroll set"
.raw.text configure -setgrid 1
scrollbar .raw.scroll -command ".raw.text yview"
pack .raw.scroll -side right -fill y
pack .raw.text -side left -fill both -expand 1

#
# About
#
# Input:  None
#
# Output:  None
#
# Display an "About..." dialog box.
#
proc About {} {
    global abouttrigger
    global version copyright build

    toplevel .about -borderwidth 10

    label .about.logo -bitmap GiddyFace
    pack .about.logo -side top

    message .about.msg -width 3i -justify center \
	    -text "$version\nby\nBruce A. Mah\n\n$copyright\n\nSimulation monitor for an Internet Simulated ATM Networking Environment\n\n$build\n"
    pack .about.msg -side top -fill x

    button .about.ok -text OK -command {set abouttrigger 1}
    pack .about.ok -side top -ipadx 5

    .mbar.help.menu entryconfigure "About InsaneMon..." -state disabled

    tkwait variable abouttrigger

    .mbar.help.menu entryconfigure "About InsaneMon..." -state normal

    destroy .about

}

#
# ClearRawCommand
#
# Clear raw display
#
proc ClearRawCommand {} {
    .raw.text delete 1.0 end
}

#
# ClearStatCommand
#
# Clear status display
#
proc ClearStatCommand {} {

    global LastUpdate
    global SelectedEntry

    .stat.list delete 0 end
    .stat.text delete 1.0 end

    foreach index [array names LastUpdate] {
	unset LastUpdate($index)
    }

    set SelectedEntry ""

}

#
# HandleUpdate
#
# Input:  Update string received in an update packet
#
# Output:  none
#
# Deal with a status update packet.
#
proc HandleUpdate {update} {

    global Mute
    global LastUpdate
    global SelectedEntry

    # Save update line, we'll need it
    set rawline $update

    ParsePacket parsed $update
    if {!$Mute} {

	#
	# Format update line
	#
	.raw.text insert end "$rawline\n"
	.raw.text see end
	
    }

    # Find our own index for this host
    set processIndex "$parsed(Host):$parsed(Pid)"

    # If we haven't heard of it yet, then make a new entry for it
    if {![info exists LastUpdate($processIndex)]} {
	.stat.list insert end "$processIndex"
    }
    set LastUpdate($processIndex) $rawline

    if {$processIndex == $SelectedEntry} {
	ExpandStatEntry $processIndex
    }

}

#
# StatListSelect
#
# Input:  window (src), Y coordinate (y)
#
# Output:  None
# 
# Fill in the status text for a given entry in the listbox
#
proc StatListSelect {src y} {

    global SelectedEntry

    # Find out which item was selected
    set index [$src nearest $y]
    
    # Highlight it, make it the selection.
    $src select set anchor $index

    # Figure out which entry (host/pid) was selected
    set SelectedEntry [$src get $index]

    # Fill in the listbox
    ExpandStatEntry $SelectedEntry

}

#
# ExpandStatEntry
#
# Input:  Entry name (host/pid)
#
# Output:  None
#
# Fill in .stat.text with a given entry...
#
proc ExpandStatEntry {name} {
    global LastUpdate
    .stat.text delete 1.0 end

    if {$name != ""} {

	if {[info exists LastUpdate($name)]} {

	    ParsePacket entry $LastUpdate($name)

	    if {[info exists entry(Time)] && [info exists entry(Start)]} {
		set runningTime [expr $entry(Time) - $entry(Start)]
	    }

	    if {[info exists entry(Host)]} {
		.stat.text insert end "Host: $entry(Host)"
		if {[info exists entry(Arch)]} {
		    .stat.text insert end " ($entry(Arch))"
		}
		.stat.text insert end "\n"
	    }

	    if {[info exists entry(Pid)]} {
		.stat.text insert end "PID: $entry(Pid)\n"
	    }

	    if {[info exists entry(Args)]} {
		.stat.text insert end "Args: $entry(Args)\n"
	    }
	    
	    if {[info exists entry(Status)]} {
		.stat.text insert end "Status: $entry(Status)\n"
	    }

	    if {[info exists runningTime]} {
		.stat.text insert end "Running time: $runningTime\n"

		if {[info exists entry(Events)]} {
		    .stat.text insert end "Events processed: $entry(Events) ([expr $entry(Events)/$runningTime] events/second)\n"
		}

		if {[info exists entry(SimTime)]} {
		    .stat.text insert end "Simulation time: $entry(SimTime) ([expr $runningTime/$entry(SimTime)]:1 slowdown)\n"
		}

	    }

	} else {
	    .stat.text insert end "No updates received"
	}
    }
}
    
#
# ParsePacket
#
proc ParsePacket {s update} {

    upvar $s stats 

    while {[llength $update] > 0} {
	    
	set key [lindex $update 0]
	set value [lindex $update 1]
	    
	if {$key == "system"} {
	    set stats(System) $value
	} elseif {$key == "host"} {
	    set stats(Host) $value
	} elseif {$key == "pid"} {
	    set stats(Pid) $value
	} elseif {$key == "args"} {
	    set stats(Args) $value
	} elseif {$key == "status"} {
	    set stats(Status) $value
	} elseif {$key == "start"} {
	    set stats(Start) $value
	} elseif {$key == "time"} {
	    set stats(Time) $value
	} elseif {$key == "events"} {
	    set stats(Events) $value
	} elseif {$key == "simtime"} {
	    set stats(SimTime) $value
	} elseif {$key == "hosttype"} {
	    set stats(Arch) $value
	}
	    
	set update [ lreplace $update 0 1 ]
    }
}
